#!/usr/bin/perl
# Update resource limits for some system

require './server-manager-lib.pl';
&error_setup($text{'limits_err'});
&error_if_cloudmin_replica();

# Get the server
&ReadParse();
$server = &get_managed_server($in{'id'});
$server || &error($text{'edit_egone'});
&can_action_error($server, "limits");
$t = $server->{'manager'};
$limits = &get_owner_limits();

# Validate inputs
if (defined($in{'memory'})) {
	$in{'memory_def'} || $in{'memory'} =~ /^\d+$/ ||
		&error($text{'limits_ememory'});
	}
if (defined($in{'cpu'})) {
	$in{'cpu_def'} || $in{'cpu'} =~ /^\d+$/ && $in{'cpu'} > 0 ||
		&error($text{'limits_ecpu'});
	}
if (defined($in{'weight'})) {
	$in{'weight'} =~ /^\d+$/ && $in{'weight'} > 0 ||
		&error($text{'limits_eweight'});
	}
if (defined($in{'disk'})) {
	$in{'disk_def'} || $in{'disk'} =~ /^\d+$/ ||
		&error($text{'limits_edisk'});
	}
if (defined($in{'rate'})) {
	$in{'rate_def'} || $in{'rate'} =~ /^\d+$/ ||
		&error($text{'limits_erate'});
	}
if (defined($in{'bw'})) {
	$in{'bw_def'} || $in{'bw'} =~ /^[1-9]\d*$/ ||
		&error($text{'limits_ebw'});
	}

# Apply them, telling the user
&ui_print_unbuffered_header(&server_in($server), $text{'limits_title'}, "");
&obtain_lock_server($server);

# Make sure status is up to date
&check_server_status($server, 0, 1);

# Update memory
if (defined($in{'memory'}) || defined($in{'memory_def'})) {
	$m = $in{'memory_def'} ? 0 : $in{'memory'}*$in{'memory_units'};
	&$first_print(&text('limits_setting', $m ? &nice_size($m)
						 : $text{'edit_lunlimit'}));
	$gfunc = "type_".$t."_get_memory";
	$oldm = &$gfunc($server);
	if ($m == $oldm) {
		# No need to change
		&$second_print($text{'limits_already'});
		}
	else {
		# Check limits
		$incr = $m - $oldm;
		if ($limits && $limits->{'max_ram'} &&
		    $limits->{'ram'} + $incr > $limits->{'max_ram'}) {
			# Over RAM limit
			&$second_print(&text('limits_eramlimit',
				&nice_size($limits->{'max_ram'}),
				&nice_size($limits->{'ram'})));
			}
		elsif ($limits && $limits->{'max_ram'} && !$m) {
			# Set to unlimited but not allowed
			&$second_print(&text('limits_eramlimit2',
				&nice_size($limits->{'max_ram'})));
			}
		else {
			# Call set function and check
			$mfunc = "type_".$t."_save_memory";
			$err = &$mfunc($server, $m);
			if ($err) {
				&$second_print(&text('limits_failed', $err));
				}
			else {
				&$second_print($text{'create_done'});
				$cfunc = "type_".$t."_can_memory";
				$reboot++ if (&$cfunc($server) == 2);
				}
			}
		}
	}

# Update CPU and weight
if (defined($in{'cpu'}) || defined($in{'weight'}) || defined($in{'cpu_def'})) {
	$cpudesc = $in{'cpu_def'} ? $text{'edit_lunlimit'} : $in{'cpu'}."%";
	if (defined($in{'weight'})) {
		&$first_print(&text('limits_settingcpu',
				    $cpudesc, $in{'weight'}));
		}
	else {
		&$first_print(&text('limits_settingcpu2', $cpudesc));
		}
	$gfunc = "type_".$t."_get_cpu";
	($oldc, $oldw) = &$gfunc($server);
	if ($oldc == ($in{'cpu_def'} ? 0 : $in{'cpu'}) &&
	    $oldw == $in{'weight'}) {
		# No need to change
		&$second_print($text{'limits_already'});
		}
	else {
		# Check CPU limit
		$c = !defined($in{'cpu_def'}) ? undef :
		     $in{'cpu_def'} ? 0 : $in{'cpu'};
	        $w = !defined($in{'weight'}) ? undef : $in{'weight'};
		$incr = $c - $oldc;
		if ($limits && $limits->{'max_cpu'} &&
		    $limits->{'cpu'} + $incr > $limits->{'max_cpu'} &&
		    defined($c)) {
			# Over CPU limit
			&$second_print(&text('limits_ecpulimit',
				$limits->{'max_cpu'}, $limits->{'cpu'}));
			}
		elsif ($limits && $limits->{'max_cpu'} && !$c &&
		       defined($c)) {
			# Set to unlimited but not allowed
			&$second_print(&text('limits_ecpulimit2',
				$limits->{'max_cpu'}));
			}
		else {
			# Call set function and check
			$cfunc = "type_".$t."_save_cpu";
			$err = &$cfunc($server, $c, $w);
			if ($err) {
				&$second_print(&text('limits_failed', $err));
				}
			else {
				&$second_print($text{'create_done'});
				$cfunc = "type_".$t."_can_cpu";
				$reboot++ if (&$cfunc($server) == 2 ||
					      &$cfunc($server) == 4);
				}
			}
		}
	}

# Update disk space
if (defined($in{'disk'}) || defined($in{'disk_def'})) {
	$d = $in{'disk_def'} ? 0 : $in{'disk'}*$in{'disk_units'};
	&$first_print(&text('limits_settingdisk', $d ? &nice_size($d)
						 : $text{'edit_lunlimit'}));
	$gfunc = "type_".$t."_get_disk";
	$oldd = &$gfunc($server);
	if ($d == $oldd) {
		# No need to change
		&$second_print($text{'limits_already'});
		}
	else {
		# Check limits
		$incr = $d - $oldd;
		if ($limits && $limits->{'max_disk'} &&
		    $limits->{'disk'} + $incr > $limits->{'max_disk'}) {
			# Over disk limit
			&$second_print(&text('limits_edisklimit',
				&nice_size($limits->{'max_disk'}),
				&nice_size($limits->{'disk'})));
			}
		elsif ($limits && $limits->{'max_disk'} && !$d) {
			# Set to unlimited but not allowed
			&$second_print(&text('limits_edisklimit2',
				&nice_size($limits->{'max_disk'})));
			}
		else {
			# Call set function and check
			$dfunc = "type_".$t."_save_disk";
			$err = &$dfunc($server, $d);
			if ($err) {
				&$second_print(&text('limits_failed', $err));
				}
			else {
				&$second_print($text{'create_done'});
				$cfunc = "type_".$t."_can_disk";
				$reboot++ if (&$cfunc($server) == 2);
				}
			}
		}
	}

# Bandwidth limit
if (defined($in{'bw'}) && &can_action($server, "bw")) {
	$bw = $in{'bw_def'} ? '' : $in{'bw'}*$in{'bw_units'};
	&$first_print(&text($in{'bw_def'} ? 'limits_settingbwu' :
				'limits_settingbw', &nice_size($bw)));
	if ($server->{'bw_limit'} != $bw) {
		$server->{'bw_limit'} = $bw;
		&$second_print($text{'create_done'});
		}
	else {
		# No need to change
		&$second_print($text{'limits_already'});
		}
	}

# Update network rate
if (defined($in{'rate'}) || defined($in{'rate_def'})) {
	$d = $in{'rate_def'} ? 0 : $in{'rate'}*$in{'rate_units'};
	&$first_print(&text('limits_settingrate', $d ? &nice_size($d)
						 : $text{'edit_lunlimit'}));
	$gfunc = "type_".$t."_get_network_rate";
	$oldd = &$gfunc($server);
	if ($d == $oldd) {
		# No need to change
		&$second_print($text{'limits_already'});
		}
	else {
		# Call set function and check
		$dfunc = "type_".$t."_save_network_rate";
		$err = &$dfunc($server, $d);
		if ($err) {
			&$second_print(&text('limits_failed', $err));
			}
		else {
			&$second_print($text{'create_done'});
			$cfunc = "type_".$t."_can_network_rate";
			$reboot++ if (&$cfunc($server) == 2);
			}
		}
	}

&save_managed_server($server);
&release_lock_server($server);
&webmin_log("limits", "server", $server->{'host'});

# Offer to reboot if needed
if ($reboot && $server->{'status'} ne 'down') {
	print &ui_form_start("save_serv.cgi");
	print &ui_hidden("id", $in{'id'});
	print &ui_hidden("reboot", 1);
	print &ui_hidden("confirm", 1);
	print &ui_hidden("updown", 1);
	print "<b>$text{'limits_needreboot'}</b><p>\n";
	print &ui_form_end([ [ undef, $text{'limits_reboot'} ] ]);
	}

&ui_print_footer("edit_serv.cgi?id=$in{'id'}", $text{'edit_return'},
		 "", $text{'index_return'});


