#!/usr/bin/perl
# Add, update to remove a system doing xen hosting

require './server-manager-lib.pl';
&can_action_error(undef, "global");
&foreign_require("net", "net-lib.pl");
&ReadParse();
&error_setup($text{'xen_err'});
&error_if_cloudmin_replica();
$xen = &get_managed_server($in{'id'});
$xen || &error($text{'xen_egone'});
&obtain_lock_server($xen);

if ($in{'delete'}) {
	# Just mark this system as no longer a VServe host
	$xen->{'xen_host'} = 0;
	&save_managed_server($xen);
	}
elsif ($in{'find'}) {
	# Redirect to find form
	&redirect("find_form.cgi?id=$xen->{'id'}&type=xen");
	return;
	}
else {
	if ($in{'new'}) {
		# Is the newly selected server actually capable of hosting
		# Xen instances?
		&can_ssh_status($xen) || &error($text{'xen_edown'});
		$err = &check_xen_host_software($xen);
		$err && &error(&text('xen_esoft', $err));
		}

	if (!$module_info{'onehost'}) {
		# Save location group
		&parse_host_location_field($xen, \%in);
		}

	# Validate and save other inputs, starting with the directory
	$xen->{'xen_dir'} = $in{'dir'};
	if (&can_ssh_status($xen)) {
		($out, $ex) = &execute_command_on_server($xen,
				"mkdir ".quotemeta($in{'dir'}).
				" ; test -d ".quotemeta($in{'dir'}));
		$ex && &error($text{'xen_edir'});
		}

	# Save and check LVM
	if ($in{'lvm_def'}) {
		delete($xen->{'xen_vg'});
		}
	else {
		$in{'lvm'} =~ /^[A-Za-z0-9\.\-\_]+$/ ||
			&error($text{'xen_elvm'});
		if ($in{'new'} || &can_ssh_status($xen)) {
			$err = &check_xen_host_lvm($xen, $in{'lvm'});
			&error(&text('xen_elvmerr', $err)) if ($err);
			}
		$xen->{'xen_vg'} = $in{'lvm'};
		}

	# DNS zone
	$in{'zone'} =~ /^[a-z0-9\.\-\_]+$/i || &error($text{'xen_ezone'});
	$xen->{'xen_zone'} = $in{'zone'};

	# IP allocation range
	&parse_ip_range_input(\%in, $xen, "xen");

	# Host system interfaces
	if ($in{'iface_def'}) {
		$xen->{'xen_iface'} = undef;
		}
	else {
		$xen->{'xen_iface'} = join(' ', split(/\0/, $in{'iface'}));
		}

	# Source for resolv.conf
	$in{'resolv_def'} || -r $in{'resolv'} ||
		&error($text{'xen_eresolv'});
	$xen->{'xen_resolv'} = $in{'resolv_def'} ? undef : $in{'resolv'};

	if ($in{'filetype'}) {
		# Chosen by user
		$xen->{'xen_filetype'} = $in{'filetype'};
		}
	else {
		# If Xen 3.2, use tap:aio .. otherwise, use file
		if (&get_xen_software_version($xen) >= 3.2) {
			$xen->{'xen_filetype'} = "tap:aio";
			}
		else {
			$xen->{'xen_filetype'} = "file";
			}
		}

	# Migration IP and port
	if (!$module_info{'onehost'}) {
		if ($in{'livehost_def'}) {
			delete($xen->{'xen_livehost'});
			delete($xen->{'xen_liveport'});
			}
		else {
			gethostbyname($in{'livehost'}) ||
				&error($text{'xen_elivehost'});
			$xen->{'xen_livehost'} = $in{'livehost'};
			$in{'liveport'} =~ /^\d+$/ ||
				&error($text{'xen_eliveport'});
			$xen->{'xen_liveport'} = $in{'liveport'};
			}
		}

	# Parse minimum resources
	&parse_host_floor_table($xen, \%in);

	if (!$module_info{'onehost'}) {
		# Cache directory
		&parse_host_images_cache_table($xen, \%in, "xen");
		}

	# Backup dir
	&parse_default_backup_dest_table($xen, \%in);

	$xen->{'xen_host'} = 1;
	&save_managed_server($xen);

	if ($in{'autoadd'} && $in{'autologin'}) {
		# Added, redirect to find page
		&redirect("find.cgi?id=$xen->{'id'}&type=xen&copyauth=1&".
			  "running=".($in{'autoadd'} == 1 ? 1 : 0));
		}
	elsif ($in{'autoadd'} && !$in{'autologin'}) {
		# Redirect to find form
		&redirect("find_form.cgi?id=$xen->{'id'}&type=xen&".
			  "running=".($in{'autoadd'} == 1 ? 1 : 0));
		}
	}

&release_lock_server($xen);
&webmin_log($in{'delete'} ? "unhost" : "host", "server", $xen->{'host'},
	    { 'type' => 'xen' });
if ($module_info{'onehost'}) {
	&redirect("");
	}
else {
	&redirect("list_xens.cgi");
	}

