#!/usr/bin/perl

=head1 setup-xen-kernel.pl

Configure GRUB to boot the first Xen-capable kernel.

This script is designed to be called from the Cloudmin installer. It checks
if the default kernel in GRUB supports Xen, and if not changes the default
to use one.

=cut

package server_manager;
$main::no_acl_check++;
$ENV{'WEBMIN_CONFIG'} ||= "/etc/webmin";
$ENV{'WEBMIN_VAR'} ||= "/var/webmin";
if ($0 =~ /^(.*)\/[^\/]+$/) {
	chdir($pwd = $1);
	}
else {
	chop($pwd = `pwd`);
	}
$0 = "$pwd/setup-bind-zone.pl";
require './server-manager-lib.pl';
$< == 0 || die "$0 must be run as root";
&set_all_text_print();
&usage_if_cloudmin_replica();
&foreign_require("grub", "grub-lib.pl");
&can_action_usage(undef, "global");

# Parse command line
while(@ARGV > 0) {
	my $a = shift(@ARGV);
	&usage("Unknown flag $a");
	}

# Check if we even have grub
if (!&foreign_installed("grub")) {
	print STDERR "GRUB not found on this system!\n";
	exit(1);
	}

# Get the config and find the default kernel
$conf = &grub::get_menu_config();
$def = &grub::find_value("default", $conf);
@titles = &grub::find("title", $conf);
$deftitle = $titles[$def];
for($i=0; $i<@titles; $i++) {
	if ($titles[$i]->{'kernel'} =~ /xen/) {
		$xenidx = $i;
		$xentitle = $titles[$i];
		last;
		}
	}
if ($deftitle->{'kernel'} =~ /xen/) {
	print "Xen kernel $deftitle->{'kernel'} is already in use\n";
	exit(0);
	}
elsif (!$xentitle) {
	print STDERR "No Xen-capable kernel found!\n";
	exit(2);
	}

# Set the default
&grub::save_directive($conf, 'default', { 'name' => 'default',
					  'value' => $xenidx });
&flush_file_lines();
print "Selected Xen kernel $deftitle->{'kernel'}\n";

sub usage
{
print "$_[0]\n\n" if ($_[0]);
print "Configure GRUB to boot the first Xen-capable kernel.\n";
print "\n";
print "cloudmin setup-xen-kernel\n";
exit(1);
}

