# Functions for open source XEN

sub type_xen_desc
{
return $text{'xen_typedesc'};
}

# type_xen_parent(&server, [&all-servers])
# Returns the parent system that hosts this xen
sub type_xen_parent
{
my ($server, $servers) = @_;
if ($servers) {
	my ($vhost) = grep { $_->{'id'} == $server->{'xen_on'} } @$servers;
	return $vhost;
	}
else {
	return &get_managed_server($server->{'xen_on'});
	}
}

sub type_xen_must_image
{
return 1;			# Xen instances always come from an image
}

sub type_xen_can_manual_image
{
return 1;			# An image can be created from a CD or raw disk
}

sub type_xen_is_virtual
{
return 1;
}

sub type_xen_can_transfer
{
my ($server) = @_;
return $server->{'xen_mount'} && !$server->{'nohost'} ||	# Via host
       $server->{'status'} eq 'nowebmin' ||	# Can transfer if SSH is up
       $server->{'status'} eq 'downwebmin' ||
       $server->{'status'} eq 'nowebminlogin' ||
       $server->{'status'} eq 'novirt' ||
       $server->{'status'} eq 'virt';
}

# Can change the password if totally down, or can SSH in
sub type_xen_can_password
{
my ($server) = @_;
return $server->{'status'} ne 'nohost' &&
       $server->{'status'} ne 'nossh' &&
       $server->{'status'} ne 'nosshlogin';
}

# Xen instances can have a hostname separate from the DNS name
sub type_xen_can_custom_hostname
{
return 1;
}

# Can reset, if the host is up
sub type_xen_can_reset
{
my ($server) = @_;
return $server->{'status'} ne 'nohost';
}

# Returns links for creating and adding xens (if this system supports
# them, or remote systems do)
sub type_xen_create_links
{
my @vhosts = grep { $_->{'xen_host'} } &list_managed_servers();
my @rv;
if (@vhosts) {
	push(@rv, { 'create' => 0,
		    'desc' => $text{'xen_addlink'} });
	my @images = &list_server_images('xen');
	if (@images) {
		push(@rv, { 'create' => 1,
			    'desc' => $text{'xen_createlink'} });
		}
	}
return @rv;
}

# Returns a link to the page list xen-capable systems
sub type_xen_global_links
{
if ($module_info{'onehost'}) {
	# Page for just this host
	return ( { 'link' => 'edit_xen.cgi?id=0',
		   'icon' => 'images/xens.gif',
		   'title' => $text{'xen_editone'},
		   'cat' => 'hosts' } );
	}
else {
	# List of Xen hosts
	return ( { 'link' => 'list_xens.cgi',
		   'icon' => 'images/xens.gif',
		   'title' => $text{'xens_title'},
		   'cat' => 'hosts' } );
	}
}

# type_xen_server_actions(&server)
# Returns extra links for Xen instances
sub type_xen_server_actions
{
my ($server) = @_;
my @rv;
if (($server->{'xen_vnc_display'} || $config{'xen_vnc'}) &&
    $server->{'status'} ne 'down' && $server->{'status'} ne 'nohost' &&
    &can_action($server, "console")) {
	# Xen VNC console access link
	push(@rv, { 'id' => 'vnc',
		    'desc' => $text{'edit_vnc'},
		    'cat' => 'power',
		    'link' => "vnc.cgi?id=$server->{'id'}" });
	}
return @rv;
}

# type_xen_add_form()
# Returns inputs for adding an existing VServer. This is just a matter of
# selecting the host and the server.
sub type_xen_add_form
{
my $rv;

# Hosting system
my @vhosts = grep { $_->{'xen_host'} } &list_managed_servers();
$rv .= &ui_table_row($text{'add_xenhost'},
		&ui_select("xen_on", $vhosts[0]->{'id'},
			[ map { [ $_->{'id'}, $_->{'host'} ] } @vhosts ]));

if (@vhosts == 1) {
	# Can just show a drop-down of existing xen instances, except those we
	# are already hosting)
	my $instances = &list_xen_instances($vhosts[0]);
	my %already = map { $_->{'xen'}, $_ }
			     &list_managed_servers();
	$rv .= &ui_table_row($text{'add_xeninstance'},
	    &ui_select("xen_instance", undef,
		[ map { [ $_->{'instance'} ] }
		      grep { !$already{$_->{'instance'}} } @$instances] ));
	}
else {
	# Need to show xen name field, with popup
	$rv .= &ui_table_row($text{'add_xeninstance'},
	     &ui_textbox("xen_instance", undef, 20)." ".
	     &xen_instance_chooser_button("xen_instance",
					      "xen_on"));
	}

# Configuration file on host
$rv .= &ui_table_row($text{'add_xenconf'},
		&ui_textbox("xen_conf", undef, 60));

return $rv;
}

# type_xen_add_parse(&server, &in)
# Parse inputs from type_xen_add_form
sub type_xen_add_parse
{
my ($server, $in) = @_;

# Save Xen host
$server->{'xen_on'} = $in->{'xen_on'};
my $vhost = &type_xen_parent($server);

# Check and save Xen instance
my $instances = &list_xen_instances($vhost);
$in->{'xen_instance'} || &error($text{'add_exeninstance'});
my ($got) = grep { $_->{'instance'} eq $in->{'xen_instance'} } @$instances;
$got || &error(&text('add_exeninstance2', $vhost->{'host'}));
$server->{'xen'} = $in->{'xen_instance'};

# Check and save Xen config file
$in->{'xen_conf'} || &error($text{'add_exenconf'});
my $temp = &transname();
my $err = &get_file_from_server($vhost, $in->{'xen_conf'}, $temp);
$err && &error($text{'add_exenconf2'});
my $conf = &parse_xen_config($temp);
my $kernel = &xen_find_value("kernel", $conf);
my $bootloader = &xen_find_value("bootloader", $conf);
if (!$bootloader && !$kernel) {
	&error($text{'add_exenconf4'});
	}
if ($bootloader =~ /pygrub/) {
	$server->{'xen_pygrub'} = $bootloader;
	$server->{'xen_bootloader'} = $bootloader;
	}
elsif ($kernel =~ /pv-grub/) {
	$server->{'xen_pygrub'} = 1;
	$server->{'xen_bootloader'} = $kernel;
	}
my $cname = &xen_find_value("name", $conf);
!$cname || $cname eq $server->{'xen'} || &error($text{'add_exenconf3'});
$server->{'xen_conf'} = $in->{'xen_conf'};
$server->{'ip_addresses'} = &xen_find_value("address", $conf);

# Get root disk from .cfg file
&set_xen_filesystem($server, $conf);
}

# set_xen_filesystem(&server, &conf)
# Updates the xen_filesystem field in a server object based on disks, if
# possible
sub set_xen_filesystem
{
my ($server, $conf) = @_;
my $disks = &xen_find_value("disk", $conf);
my $root = &xen_find_value("root", $conf);
if ($root =~ /^\/dev\/(\S+)/) {
	$root = $1;
	}
else {
	$root = undef;
	}
if ($root &&
    $disks->[0] =~ /^(file|phy|tap:aio):(\S+),\Q$root\E,(\S+)$/) {
	# Can get root disk from root directive
	$server->{'xen_filesystem'} = $2;
	}
elsif ($disks->[0] =~ /^(file|phy|tap:aio):(\S+),(\S+),(\S+)$/) {
	# Just assume first
	$server->{'xen_filesystem'} = $2;
	}
if ($server->{'xen_filesystem'} =~ /^\/dev\/(\S+)\/(\S+)$/) {
	# Looks like an LVM root
	$server->{'xen_filesystem_vg'} = $1;
	$server->{'xen_filesystem_lv'} = $2;
	}
return $server->{'xen_filesystem'};
}

# type_xen_reboot(&server)
# Use the xm reboot command
sub type_xen_reboot
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
my @st = &stat_file_on_server($vhost, $server->{'xen_conf'});
if (@st && $st[9] > $server->{'xen_lastboot'}) {
	# The config file was modified since the last boot. We need to do
	# a full stop/start to apply it
	my $err = &type_xen_shutdown($server);
	return $err if ($err);
	my ($ok, $err) = &type_xen_startup($server);
	return $ok ? undef : $err;
	}
else {
	# Just do a reboot
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"xm reboot $server->{'xen'} -w");
	&compute_ip_addresses($server);
	return $err ? $err : $ex ? $out : undef;
	}
}

# type_xen_shutdown(&server, [force])
# Use the xm shutdown command to shut down
sub type_xen_shutdown
{
my ($server, $force) = @_;
my $vhost = &type_xen_parent($server);
my ($out, $ex, $err, $status);
if (!$force) {
	($out, $ex, $err, $status) = &execute_command_on_server($vhost,
		"xm shutdown $server->{'xen'} -w", 60);
	}
if ($status == 4 || $force) {
	# shutdown timed out .. escalate to destruction
	($out, $ex, $err, $status) = &execute_command_on_server($vhost,
		"xm destroy $server->{'xen'}");
	}
return $err ? $err : $ex ? $out : undef;
}

# type_xen_startup(&server)
# Use the xm create command to start up, and wait till it looks like init
# started OK. This has to be done in a pty, as the Xen console won't work
# if it thinks it is writing to a pipe or file.
sub type_xen_startup
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
my $conf = &get_xen_config($server, 1);
my $bootloader = &xen_find_value("bootloader", $conf);
my ($pinged, $sshed);
if (!$bootloader &&
    &has_command_on_server($vhost, "perl") &&
    !&put_file_to_server($vhost, "$module_root_directory/pty.pl",
				 "/tmp/pty.pl")) {
	if (!$server->{'xen_console'}) {
		$server->{'xen_console'} = $server->{'xen_conf'};
		$server->{'xen_console'} =~ s/\.cfg$//;
		$server->{'xen_console'} .= ".console";
		}
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"(perl /tmp/pty.pl xm create -c $server->{'xen_conf'} ; echo SERVER-MANAGER-COMPLETE) >$server->{'xen_console'} 2>&1 </dev/null &");
	return (0, $err) if ($err);
	return (0, $out) if ($ex);
	my $tries = 0;
	my $pid;
	while($tries++ < 60) {
		sleep(1);
		my $data = &get_file_data_from_server($vhost,
					$server->{'xen_console'});
		if ($data =~ /PTY\s+PID:\s+(\d+)/) {
			$pid = $1;	# got PID of xm create command
			}
		if ($data =~ /SERVER-MANAGER-COMPLETE/) {
			# xm command exited, which indicates a boot failure
			$data =~ s/SERVER-MANAGER-COMPLETE\n//g;
			return (0, "<pre>".&html_escape($data)."</pre>");
			}
		elsif ($data =~ /INIT:/) {
			# init started, so it looks like we are mostly done.
			last;
			}
		}
	&execute_command_on_server($vhost, "kill $pid") if ($pid);
	}
else {
	# Since the Xen host doesn't have Perl, we can only call xm create
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"xm create $server->{'xen_conf'}");
	return (0, $err) if ($err);
	return (0, $out) if ($ex);

	# Wait for xm list to find it, for up to 30 seconds
	my $tries = 0;
	my $found = 0;
	while($tries++ < 30) {
		my $instances = &list_xen_instances($vhost);
		if (ref($instances)) {
			my ($got) = grep { $_->{'instance'} eq
					   $server->{'xen'} } @$instances;
			if ($got) {
				$found = 1;
				last;
				}
			}
		}
	return (0, &text('xen_estartxm', 30)) if (!$found);

	# Wait up to 60 seconds for ping
	$pinged = &wait_for_ping($server);
	}

# Wait for SSH to come up
if ($pinged ne '0') {
	$sshed = &wait_for_ssh($server);
	}

# If a CPU credit is set, apply it now
if ($server->{'xen_credit_cpu'}) {
	&execute_command_on_server($vhost,
		"xm sched-credit -d $server->{'xen'} ".
		"-c $server->{'xen_credit_cpu'}");
	}
if ($server->{'xen_credit_weight'}) {
	&execute_command_on_server($vhost,
		"xm sched-credit -d $server->{'xen'} ".
		"-w $server->{'xen_credit_weight'}");
	}

# Record the boot time. This can be later used by the reboot function
# to check if a shutdown / startup is needed
$server->{'xen_lastboot'} = time();

# Re-check which IPs are really active
if ($sshed) {
	&compute_ip_addresses($server);
	}

# Return an error if something went wrong
if (defined($pinged) && !$pinged) {
	return (2, &text('xen_estartping', 60));
	}
if (defined($sshed) && !$sshed) {
	return (3, &text('xen_estartssh', 60));
	}
return (1, undef);
}

# type_xen_depends(&parent, &vserver)
# Returns 1 if the Xen instance hosted by the given parent
sub type_xen_depends
{
my ($parent, $server) = @_;
return $server->{'xen_on'} eq $parent->{'id'};
}

# type_xen_check_login(&server)
# Check if the Xen instance is up, and if we can SSH in. Returns a status
# code and error message
sub type_xen_check_login
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
if (!$vhost) {
	return (5, "Xen host does not exist!");
	}
if ($vhost->{'status'} eq 'down' ||
    $vhost->{'status'} eq 'nossh' ||
    $vhost->{'status'} eq 'nohost' ||
    $vhost->{'status'} eq 'nosshlogin') {
	return (5, "Xen host $vhost->{'host'} is down");
	}
my $instances = &list_xen_instances($vhost);
if (!ref($instances)) {
	return (1, "Could not fetch Xen instances : $instances");
	}
my ($got) = grep { $_->{'instance'} eq $server->{'xen'} } @$instances;
if (!$got) {
	return (6, "Xen instance is down");
	}
elsif ($got->{'state'} eq 'p') {
	return (8, "Xen instance is paused");
	}
#elsif ($got->{'state'} ne 'r' && $got->{'state'} ne 'b') {
#	return (1, "Xen instance is hung (state $got->{'state'})");
#	}

# Tell check_server_ssh_login that we need to SSH
return ( );
}

# type_xen_post_refresh(&server, regenerate)
# Called after a refresh to compute total disk size, if needed
sub type_xen_post_refresh
{
my ($server, $regen) = @_;
if ($server->{'xen_disktotal'} eq '' || $regen) {
	&type_xen_compute_disksize($server);
	}
if (($server->{'xen_network_rate'} eq '' || $regen) &&
    $server->{'status'} ne 'nohost') {
	$server->{'xen_network_rate'} = &type_xen_get_network_rate($server);
	}
}

# type_xen_post_host_refresh(&server, re-generate, &info)
# Called after a Xen host system is refreshed, to re-start xenconsoled and
# maybe get LVM info
sub type_xen_post_host_refresh
{
my ($server, $regen, $info) = @_;
my $procs = &list_processes_on_server($server);
return if (!ref($procs));	# Couldn't list
my ($c) = grep { $_->{'args'} =~ /(^|\s|\/)xenconsoled($|\s)/ } @$procs;
if (!$c) {
	# Not running .. need to restart
	my $cmd = $server->{'xen_consoled_cmd'} ||
		     "xenconsoled --log none --timestamp none";
	&execute_command_on_server($server,
		"$cmd >/dev/null 2>&1 </dev/null &");
	}
else {
	# Running .. save the command line, so we can restart it later
	$server->{'xen_consoled_cmd'} = $c->{'args'};
	}

# If using LVM, see how much space is free on the VG
if ($server->{'xen_vg'} &&
    ($server->{'status'} eq 'virt' || $server->{'status'} eq 'novirt')) {
	&remote_foreign_require($server, "lvm", "lvm-lib.pl");
	my ($vg) = grep { $_->{'name'} eq $server->{'xen_vg'} }
		   &remote_foreign_call($server, "lvm", "list_volume_groups");
	if ($vg) {
		$server->{'xen_vg_total'} =
			$vg->{'pe_total'}*$vg->{'pe_size'}*1024;
		$server->{'xen_vg_used'} =
			$vg->{'pe_alloc'}*$vg->{'pe_size'}*1024;
		$server->{'xen_vg_free'} = $server->{'xen_vg_total'} -
					   $server->{'xen_vg_used'};
		}
	}
elsif (!$server->{'xen_vg'}) {
	delete($server->{'xen_vg_total'});
	delete($server->{'xen_vg_used'});
	delete($server->{'xen_vg_free'});
	}

# Use xentop to see how much RAM is really used and free for Xen. If the host
# has been configured with a dom0 RAM limit via grub, the Linux "free" command
# will report that total RAM is less than actually exists.
if (&can_ssh_status($server)) {
	my ($out, $ex, $err) = &execute_command_on_server($server,
		"xm info ; xm list");
	if (!$ex && !$err) {
		my ($total, $hostfree, $hostmem, $xenmem);
		foreach my $l (split(/\r?\n/, $out)) {
			if ($l =~ /^total_memory\s+:\s+(\d+)/) {
				$total = $1;
				}
			elsif ($l =~ /^free_memory\s+:\s+(\d+)/) {
				$hostfree = $1;
				}
			elsif ($l =~ /^Domain-0\s+\d+\s+(\d+)/) {
				$hostmem = $1;
				}
			elsif ($l =~ /^\S+\s+\d+\s+(\d+)\s+\d+\s+/) {
				$xenmem += $1;
				}
			}
		if ($total) {
			# Memory used is usage on dom0 (as shown by free),
			# plus usage by all Xen instances.
			$server->{'xen_mem_total'} = $total*1024*1024;
			$server->{'xen_mem_used'} =
				$xenmem*1024*1024 +
				($info->{'mem'}->[0] -
				 $info->{'mem'}->[1])*1024;
			$server->{'xen_mem_free'} =
				$server->{'xen_mem_total'} -
				$server->{'xen_mem_used'};
			}
		else {
			delete($server->{'xen_mem_total'});
			delete($server->{'xen_mem_used'});
			delete($server->{'xen_mem_free'});
			}
		}
	}
}

# type_xen_edit_form(&server)
# Show information about some Xen system
sub type_xen_edit_form
{
my ($server) = @_;
my $rv;

# Hosted on
my $vhost = &type_xen_parent($server);
$rv .= &ui_table_row($text{'edit_xenhost'},
	&can_action($vhost, "view") ?
	     "<a href='edit_serv.cgi?id=$vhost->{'id'}'>$vhost->{'host'}</a>" :
	     $vhost->{'host'});

# Xen instance name
$rv .= &ui_table_row($text{'edit_xeninstance'},
		     $server->{'xen'});

# Actual Xen memory
if ($config{'xen_mem_mode'}) {
	my $instances = &list_xen_instances($vhost);
	if (ref($instances)) {
		my ($v) = grep { $_->{'instance'} eq $server->{'xen'} }
				  @$instances;
		if ($v) {
			$rv .= &ui_table_row($text{'edit_xenmem'},
					     &nice_size($v->{'mem'}));
			}
		}
	}

# Xen config file
$rv .= &ui_table_row($text{'edit_xenconf'},
		     "<tt>$server->{'xen_conf'}</tt>");

return $rv;
}

# type_xen_server_info(&server)
# Returns an array of information for list-systems.pl to display
sub type_xen_server_info
{
my ($server) = @_;
my @rv;
my $vhost = &type_xen_parent($server);
push(@rv, [ "Hosted on", $vhost->{'host'} ]);
push(@rv, [ "Xen instance", $server->{'xen'} ]);
if ($config{'xen_mem_mode'}) {
	my $instances = &list_xen_instances($vhost);
	if (ref($instances)) {
		my ($v) = grep { $_->{'instance'} eq $server->{'xen'} }
				  @$instances;
		if ($v) {
			push(@rv, [ "Xen memory now",&nice_size($v->{'mem'}) ]);
			}
		}
	}
push(@rv, [ "Xen config file", $server->{'xen_conf'} ]);
if ($server->{'xen_disktotal'}) {
	push(@rv, [ "Xen total disk size",
		    &nice_size($server->{'xen_disktotal'}) ]);
	}
return @rv;
}

# type_xen_server_host_info(&server)
# Returns an array of information for list-systems.pl to display, for a Xen
# host system
sub type_xen_server_host_info
{
my ($server) = @_;
my @rv;      
if ($server->{'xen_vg_total'}) {
	push(@rv, [ "LVM disk space",
		    &nice_size($server->{'xen_vg_total'}) ]);
	push(@rv, [ "LVM disk space free",
		    &nice_size($server->{'xen_vg_free'}) ]);
	}
if ($server->{'xen_mem_total'}) {
	push(@rv, [ "Xen host memory",
		    &nice_size($server->{'xen_mem_total'}) ]);
	push(@rv, [ "Xen host memory free",
		    &nice_size($server->{'xen_mem_free'}) ]);
	}
if (&can_ssh_status($server)) {
	my @bridges = &list_host_bridges($server);
	if (@bridges) {
		push(@rv, [ "Xen network bridges", join(" ", @bridges) ]);
		}
	}
return @rv;
}

# type_xen_create_form()
# Returns HTML for Xen-specific inputs
sub type_xen_create_form
{
my $rv;

# Check if any vhosts have more than one bridge
my @vhosts = grep { $_->{'xen_host'} && &can_host_server($_) }
		     &list_managed_servers();
my $max_hostifaces = 0;
foreach my $vhost (@vhosts) {
	my @hostifaces = split(/\s+/, $vhost->{'xen_iface'});
	if (scalar(@hostifaces) > $max_hostifaces) {
		$max_hostifaces = scalar(@hostifaces);
		}
	}

# Xen host system
my ($hostf, $hostjs);
if (@vhosts == 1 && $module_info{'onehost'} eq 'xen') {
	# Can only be hosted locally
	$rv .= &ui_hidden("xen_on", $vhosts[0]->{'id'});
	}
else {
	($hostf, $hostjs) = &show_host_server_field($text{'add_xenhost'},
						   "xen", "xen", "host_change");
	$rv .= $hostf;
	}

# IP addresses
if (&can_action(undef, "ips")) {
	my $nm = "255.255.255.0";
	if ($max_hostifaces <= 1) {
		# No host system has more than one bridge
		$rv .= &ui_table_row($text{'create_xenip'},
			&ui_radio("xen_ip_def", 1,
			  [ [ 1, $text{'create_xenipauto'} ],
			    [ 0, &text('create_xenipman',
				       &ui_textbox("xen_ip_0", undef, 20),
				       &ui_textbox("xen_netmask_0", $nm, 15))
			    ] ]));

		# Default gateway
		$rv .= &ui_table_row($text{'create_xengw'},
			&ui_opt_textbox("xen_gw_0", undef, 20,
					$text{'create_xengwdef'},
					$text{'create_xengwip'}));
		}
	else {
		# Some hosts have more than one .. need to make IP allocation
		# section dynamic
		my $itable;
		$itable .= "<style>\n";
		$itable .= ".opener_shown {display:inline}\n";
		$itable .= ".opener_hidden {display:none}\n";
		$itable .= "</style>\n";
		$itable .= &ui_hidden("xen_max_hostifaces", $max_hostifaces);

		# Add a hidden section for each host, with a row for each
		# of its interfaces
		my @locs = &list_vhost_locations(\@vhosts);
		my $style = @locs ? "opener_hidden" : "opener_shown";
		foreach my $vhost (@vhosts) {
			my $vtable = &ui_columns_start([
				$text{'create_xenifacecol'},
				$text{'create_xenipcol'},
				$text{'create_xennetmaskcol'},
				$text{'create_xengwcol'}]);
			my @ifaces = split(/\s+/, $vhost->{'xen_iface'});
			if (!@ifaces) {
				@ifaces = ( $text{'create_xenifacedef'} );
				}
			my $id = $vhost->{'id'};
			for(my $i=0; $i<@ifaces; $i++) {
				$vtable .= &ui_columns_row([
				  $ifaces[$i],
				  &ui_textbox("${id}_xen_ip_$i", undef, 15),
				  &ui_textbox("${id}_xen_netmask_$i", $nm, 15),
				  &ui_opt_textbox("${id}_xen_gw_$i", undef, 15,
						  $text{'create_xengwdef'}),
				  ]);
				}
			$vtable .= &ui_columns_end();
			$itable .= "<div style=$style ".
				   "id=$vhost->{'id'}>".$vtable."</div>\n";
			}

		# Add a section for when a location group is chosen, if any.
		# This has a row for every possible bridge.
		if (&can_select_host_location() && @locs) {
			my $vtable = &ui_columns_start([
				$text{'create_xenifacecol'},
				$text{'create_xenipcol'},
				$text{'create_xennetmaskcol'},
				$text{'create_xengwcol'}]);
			for(my $i=0; $i<$max_hostifaces; $i++) {
				$vtable .= &ui_columns_row([
				  &text('create_xenbr', $i),
				  &ui_textbox("loc_xen_ip_$i", undef, 15),
				  &ui_textbox("loc_xen_netmask_$i", $nm, 15),
				  &ui_opt_textbox("loc_xen_gw_$i", undef, 15,
						  $text{'create_xengwdef'}),
				  ]);
				}
			$vtable .= &ui_columns_end();
			$itable .= "<div style=opener_shown ".
				   "id=loc>".$vtable."</div>\n";
			}

		$rv .= &ui_table_row($text{'create_xenip2'},
			&ui_radio("xen_ip_def", 1,
				  [ [ 1, $text{'create_xenipauto'} ],
				    [ 0, $text{'create_xenipman2'}."<br>\n".
					 $itable ] ]));

		# Javascript for host change
		$rv .= "<script>\n";
		$rv .= "function host_change(id)\n";
		$rv .= "{\n";
		foreach my $vhost (@vhosts) {
			$rv .= "var divobj = ".
			       "document.getElementById('$vhost->{'id'}');\n";
			$rv .= "divobj.className = id == '$vhost->{'id'}' ? ".
			       "'opener_shown' : 'opener_hidden';\n";
			}
		$rv .= "var divobj = document.getElementById('loc');\n";
		$rv .= "if (divobj) {\n";
		$rv .= "    divobj.className = id == 'loc' ? 'opener_shown' : ".
		       "'opener_hidden';\n";
		$rv .= "    }\n";
		$rv .= "}\n";
		$rv .= $hostjs;
		$rv .= "</script>\n";
		}
	}

# Warn the user if we have no DNS zone
if (@vhosts == 1 && $module_info{'onehost'} eq 'xen' &&
    !$vhosts[0]->{'xen_zone'}) {
	$rv .= &ui_table_row(" ",
		"<font color=#ff000>".
		&text('create_xendnswarn',
		      "edit_xen.cgi?id=0", "../bind8/").
		"</font>");
	}

return $rv;
}

# type_xen_limits_create_form()
# Returns HTML for Xen-specific inputs related to CPU, RAM and disk limits
sub type_xen_limits_create_form
{
my $rv;

# Physical memory
my $limits = defined(&get_owner_limits) ? &get_owner_limits() : undef;
my $defmem = $config{'xen_mem'}*1024*1024;
my $mem = $defmem;
if ($limits && $limits->{'max_ram'}) {
	$mem = $limits->{'max_ram'} - $limits->{'ram'};
	$mem = $defmem if ($mem > $defmem);
	$mem = 128*1024*1024 if ($ram < 128*1024*1024);
	}
$rv .= &ui_table_row($text{'create_xenmemory'},
	&ui_textbox("xen_memory", int($mem/(1024*1024)), 8)." MB");

# Filesystem size
$rv .= &ui_table_row($text{'create_xendisksize'},
	&ui_opt_textbox("xen_disk", undef, 8,$text{'create_xendiskdef'})." MB");

# Swap space
$rv .= &ui_table_row($text{'create_xenswapsize'},
	&ui_opt_textbox("xen_swap", $config{'xen_swap'}, 8,
			$text{'create_xennoswap'})." MB");

# CPU allocation
my $cpu;
if ($limits && $limits->{'max_cpu'}) {
	$cpu = $limits->{'max_cpu'} - $limits->{'cpu'};
	$cpu = 100 if ($cpu > 100);
	$cpu = 1 if ($cpu < 1);
	}
$rv .= &ui_table_row($text{'create_xencpu'},
	&ui_opt_textbox("xen_cpu", $cpu, 4, $text{'domains_unlimited'})."%");

# Network rate limit
$rv .= &ui_table_row($text{'create_xenrate'},
	&ui_radio("xen_rate_def", 1,
		  [ [ 1, $text{'edit_lunlimit'} ],
                    [ 0, &ui_ratebox("xen_rate", undef) ] ]));

return $rv;
}

# type_xen_advanced_create_form()
# Returns HTML for Xen-specific inputs
sub type_xen_advanced_create_form
{
my $rv;

# Network interface MAC address
$rv .= &ui_table_row($text{'create_xenmac'},
	&ui_opt_textbox("xen_mac", undef, 30, $text{'create_xenmacauto'},
			$text{'create_xenmacether'}));

# Interface name
$rv .= &ui_table_row($text{'create_xenvif'},
	&ui_opt_textbox("xen_vif", undef, 10, $text{'create_xenvifauto'},
			$text{'create_xenvifset'}));

# Instance UUID
$rv .= &ui_table_row($text{'create_xenuuid'},
	&ui_opt_textbox("xen_uuid", undef, 50, $text{'create_xenuuidauto'},
			$text{'create_xenuuidset'}));

# Kernel boot type
$rv .= &ui_table_row($text{'xen_pygrub'},
	&ui_radio("xen_pygrub", $config{'xen_pygrub'},
		  [ [ 0, $text{'xen_pygrub0'} ],
		    [ 1, $text{'xen_pygrub1'} ] ]));

# LVM / local files option
my @lvmhosts = grep { $_->{'xen_host'} && $_->{'xen_vg'} }
		    &list_managed_servers();
if (@lvmhosts) {
	$rv .= &ui_table_row($text{'xen_onlvm'},
		&ui_radio("xen_lvm", -1,
			  [ [ -1, $text{'xen_onlvmdef'} ],
			    [ 0, $text{'xen_onlvm0'} ],
			    [ 1, $text{'xen_onlvm1'} ] ]));
	}

# Directory for Xen files
$rv .= &ui_table_row($text{'xen_imagedir'},
	&ui_opt_textbox("xen_imagedir", undef, 40,
			$text{'xen_imagedirdef'}, $text{'xen_imagedirset'}));

# On crash/etc settings
my @grid;
foreach my $o (&list_xen_on_actions()) {
	my @opts = &list_xen_on_options($o);
	push(@grid, $text{'create_xenon'.$o});
	push(@grid, &ui_select("xen_on".$o, $config{'xen_on_'.$o},
		   [ [ undef, "&lt;$text{'default'}&gt;" ],
		     map { [ $_, $text{'create_xenon'.$_} ] } @opts ]));
	}
$rv .= &ui_table_row($text{'create_xenonacts'},
	&ui_grid_table(\@grid, 2));

# Other .cfg file entries
$rv .= &ui_table_row($text{'create_xenmanual'},
	&ui_textarea("xen_manual",
		     join("\n", split(/\t+/, $config{'xen_manual'})), 3, 60));

return $rv;
}

# type_xen_create_parse(&server, &in)
# Parse inputs from type_xen_create_form, and update the server object
sub type_xen_create_parse
{
my ($server, $in) = @_;

# Validate memory parameters
$in{'xen_memory'} =~ /^\d+$/ || &error($text{'create_exenmemory'});
$server->{'xen_memory'} = $in{'xen_memory'};
$server->{'xen_disktotal'} = 0;

# Validate swap file
if (!$in{'xen_swap_def'}) {
	$in{'xen_swap'} =~ /^\d+$/ || &error($text{'create_exenswapsize'});
	$server->{'xen_swap'} = $in{'xen_swap'};
	$server->{'xen_disktotal'} += $in{'xen_swap'}*1024*1024;
	}

# Validate disk size
if (!$in{'xen_disk_def'}) {
	$in{'xen_disk'} =~ /^\d+$/ || &error($text{'create_exendisksize'});
	$server->{'xen_disksize'} = $in{'xen_disk'};
	$server->{'xen_disktotal'} += $in{'xen_disk'}*1024*1024;
	if ($img->{'rawsize'} &&
	    $img->{'rawsize'} > $in{'xen_disk'}*1024*1024) {
		&error(&text('create_exendisksize2',
			     &nice_size($img->{'rawsize'})));
		}
	}
else {
	my $img = &get_server_image($server->{'image'});
	$server->{'xen_disktotal'} += $img->{'rawsize'};
	}

# Validate CPU
if (!$in{'xen_cpu_def'}) {
	$in{'xen_cpu'} =~ /^[1-9]\d*$/ || &error($text{'create_exencpu'});
	$server->{'xen_credit_cpu'} = $in{'xen_cpu'};
	}

# Validate network rate
if (!$in{'xen_rate_def'}) {
	$in{'xen_rate'} =~ /^\d+$/ || &error($text{'limits_erate'});
	$server->{'xen_network_rate'} = $in{'xen_rate'}*$in{'xen_rate_units'};
	}

# Select host system
my $vhost = &parse_host_server_field($server, 'xen', 'xen', $in,
				     $server->{'xen_memory'}*1024*1024,
				     $server->{'xen_disktotal'});
$server->{'xen_on'} = $vhost->{'id'};

# Make sure the hostname doesn't contain a ., or clashes
$in->{'host'} =~ /\./ && &error($text{'create_exenhost'});
my $instances = &list_xen_instances($vhost);
my ($clash) = grep { $_->{'instance'} eq $in->{'host'} } @$instances;
$clash && &error($text{'create_exenclash'});
$server->{'xen'} = $server->{'host'};

if (!$in->{'xen_ip_def'} && &can_action(undef, "ips")) {
	# Validate IP addresses
	my @allhostifaces = split(/\s+/, $vhost->{'xen_iface'});
	if (!@allhostifaces) {
		# No host interfaces set, so just assume default
		@allhostifaces = ( '*' );
		}

	# Work out prefix for inputs
	my $pfx;
	if ($in->{'xen_max_hostifaces'} <= 1) {
		$pfx = '';	# None needed
		}
	elsif ($in->{'xen_def'} == 1) {
		$pfx = 'loc_';
		}
	else {
		$pfx = $vhost->{'id'}.'_';
		}

	my (@ips, @cidrs, @gws, @hostifaces);
	for(my $i=0; defined($in->{$pfx."xen_ip_$i"}); $i++) {
		next if (!$in->{$pfx."xen_ip_$i"});
		&check_ipaddress($in->{$pfx."xen_ip_$i"}) ||
			&error($text{'create_exenip'});
		push(@ips, $in{$pfx."xen_ip_$i"});
		&check_ipaddress($in->{$pfx."xen_netmask_$i"}) ||
			&error($text{'iface_enetmask'});
		push(@cidrs, &net::mask_to_prefix(
				$in->{$pfx."xen_netmask_$i"}));
		if ($in{$pfx."xen_gw_${i}_def"}) {
			push(@gws, "*");
			}
		else {
			&check_ipaddress($in{$pfx."xen_gw_$i"}) ||
				 &error($text{'create_exengw'});
			push(@gws, $in{$pfx."xen_gw_$i"});
			}
		push(@hostifaces, $allhostifaces[$i]);
		}
	@ips || &error($text{'create_exenips'});
	scalar(@ips) <= scalar(@allhostifaces) ||
		&error(&text('create_exenifaces2', $vhost->{'host'},
			     scalar(@allhostifaces), scalar(@ips)));
	$server->{'xen_hostifaces'} = join(' ', @hostifaces);
	$server->{'xen_ip'} = join(' ', @ips);
	$server->{'xen_cidr'} = join(' ', @cidrs);
	$server->{'xen_gw'} = join(' ', @gws);
	}
else {
	# Allocate the IP now, by looking for one that isn't pingable or
	# use by any system
	my $err = &allocate_xen_ip_addresses($server, $vhost);
	&error($err) if ($err);
	}
$server->{'ip_addresses'} = $server->{'xen_ip'};

# MAC address
if (!$in{'xen_mac_def'} && &can_action(undef, "advanced")) {
	$in{'xen_mac'} =~ /^([0-9a-f]{1,2}:)*[0-9a-f]{1,2}$/i ||
		&error($text{'create_exenmac'});
	$server->{'xen_mac'} = $in{'xen_mac'};
	}

# Interface name
if (!$in{'xen_vif_def'} && &can_action(undef, "advanced")) {
	$in{'xen_vif'} =~ /^[a-z0-9]+$/ ||
		&error($text{'create_exenvif'});
	$server->{'xen_vif'} = $in{'xen_vif'};
	}

# Instance UUID
if (!$in{'xen_uuid_def'} && &can_action(undef, "advanced")) {
	$in{'xen_uuid'} =~ /^[a-f0-9\-]+$/ ||
		&error($text{'create_exenuuid'});
	$server->{'xen_uuid'} = $in{'xen_uuid'};
	}

# On crash/etc settings
if (&can_action(undef, "advanced")) {
	foreach my $o (&list_xen_on_actions()) {
		if ($in{'xen_on'.$o}) {
			$server->{'xen_on'.$o} = $in{'xen_on'.$o};
			}
		}
	}

# Save manual entries
if (&can_action(undef, "advanced")) {
	$in{'xen_manual'} =~ s/\r//g;
	$in{'xen_manual'} =~ s/^\s+//g;
	$in{'xen_manual'} =~ s/\s+$//g;
	$server->{'xen_manual'} = join("\t", split(/\n/, $in{'xen_manual'}));
	}

# PyGrub setting
if (&can_action(undef, "advanced")) {
	$server->{'xen_pygrub'} = $in{'xen_pygrub'};
	}
else {
	$server->{'xen_pygrub'} = $config{'xen_pygrub'};
	}

# LVM setting
if ($in{'xen_lvm'} == -1 || !defined($in{'xen_lvm'})) {
	# Use host system default
	}
elsif ($in{'xen_lvm'} == 0) {
	# Force not on LVM
	$server->{'xen_nolvm'} = 1;
	}
elsif ($in{'xen_lvm'} == 1) {
	# Must be on LVM
	$vhost->{'xen_vg'} || &error(&text('xen_eonlvm', $vhost->{'host'}));
	}

# Disk directory setting
if (defined($in{'xen_imagedir_def'}) && !$in{'xen_imagedir_def'}) {
	$in{'xen_imagedir'} =~ /^\/\S+$/ ||
		&error($text{'xen_eimagedir'});
	$server->{'xen_nolvm'} || !$vhost->{'xen_vg'} ||
		&error(&text('xen_eimagedirhost', $vhost->{'host'}));
	$server->{'xen_imagedir'} = $in{'xen_imagedir'};
	}

# If we cannot register in DNS, the hostname has to be set as the IP
my @zones = &list_bind_zones();
my $zone;
if ($vhost->{'xen_zone'}) {
	($zone) = grep { $_->{'name'} eq $vhost->{'xen_zone'} } @zones;
	}
if ($zone) {
	$server->{'host'} .= ".".$vhost->{'xen_zone'};
	}
}

# type_xen_create_args()
# Returns an array of parameters to type_xen_create_args_parse
sub type_xen_create_args
{
my @rv = ( $module_info{'onehost'} eq 'xen' ? ( ) :
		  ( { 'name' => 'host', 'value' => 'host-system' },
		    { 'name' => 'location', 'value' => 'name' } ),
	 { 'name' => 'ip', 'value' => 'address[,address]', 'opt' => 1 },
	 { 'name' => 'cidr', 'value' => 'number[,number]', 'opt' => 1 },
	 { 'name' => 'netmask', 'value' => 'mask,[mask]', 'opt' => 1 },
	 { 'name' => 'gw', 'value' => 'address[,address|*]', 'opt' => 1 },
	 { 'name' => 'bridge', 'value' => 'name[,name]', 'opt' => 1 },
	 { 'name' => 'disksize', 'value' => 'megabytes', 'opt' => 1 },
	 { 'name' => 'memory', 'value' => 'megabytes', 'opt' => 1 },
	 { 'name' => 'cpu', 'value' => 'percent', 'opt' => 1 },
	 { 'name' => 'swap', 'value' => 'megabytes', 'opt' => 1 },
	 { 'name' => 'rate', 'value' => 'bits/second', 'opt' => 1 },
	);
if (&can_action(undef, "advanced")) {
	push(@rv,
	 { 'name' => 'mac', 'value' => 'ethernet-address', 'opt' => 1 },
	 { 'name' => 'uuid', 'value' => 'instance-uuid', 'opt' => 1 },
	 { 'name' => 'vif', 'value' => 'interface-name', 'opt' => 1 },
	 (map { { 'name' => 'on-'.$_,
		 'value' => join('|', &list_xen_on_options($_)),
		 'opt' => 1 } } &list_xen_on_actions()),
	 { 'name' => 'manual', 'value' => 'manual-config-entries', 'opt' => 1 },
	 { 'name' => 'lvm', 'value' => 'true|false', 'opt' => 1 },
	 { 'name' => 'pygrub', 'value' => 'true|false', 'opt' => 1 },
	 { 'name' => 'image-dir', 'value' => 'directory', 'opt' => 1 },
	       );
	}
return @rv;
}

# type_xen_create_args_parse(&args, &server)
# Update a server object with parsed command-line args
sub type_xen_create_args_parse
{
my ($args, $server) = @_;

# Validate memory flags
if ($args->{'memory'}) {
	$args->{'memory'} =~ /^\d+$/ || return "Non-numeric memory allocation";
	$server->{'xen_memory'} = $args->{'memory'};
	}
else {
	$server->{'xen_memory'} = $config{'xen_mem'};
	}

# Validate swap flag
$server->{'xen_disktotal'} = 0;
if ($args->{'swap'}) {
	$args->{'swap'} =~ /^\d+$/ || return "Non-numeric swap allocation";
	$server->{'xen_swap'} = $args->{'swap'};
	}
else {
	$server->{'xen_swap'} = $config{'xen_swap'};
	}
$server->{'xen_disktotal'} += $args->{'swap'}*1024*1024;

# Validate filesystem size flag
my $img = &get_server_image($server->{'image'});
if ($args->{'disksize'}) {
	$args->{'disksize'} =~ /^\d+$/ ||
		return "Non-numeric disk size";
	$server->{'xen_disksize'} = $args->{'disksize'};
	if ($img->{'rawsize'} &&
	    $img->{'rawsize'} > $args->{'disksize'}*1024*1024) {
		return "--xen-disksize must be at least $img->{'rawsize'} ".
		       "bytes, as that is the size of the selected image";
		}
	}
else {
	$server->{'xen_disktotal'} += $img->{'rawsize'};
	}

# Validate CPU flag
if ($args->{'cpu'}) {
	$args->{'cpu'} =~ /^[1-9]\d*$/ || return "Non-numeric CPU percentage";
	$server->{'xen_credit_cpu'} = $args->{'cpu'};
	}

# Validate rate flag
if ($args->{'rate'}) {
	$args->{'rate'} =~ /^[1-9]\d*$/ || return "Non-numeric network rate";
	$server->{'xen_network_rate'} = $args->{'rate'};
	}

# Hosting server
my $vhost;
if ($module_info{'onehost'}) {
	# Xen host is always this box
	($vhost) = grep { $_->{'manager'} eq 'this' } &list_managed_servers();
	}
elsif ($args->{'host'}) {
	# Use a specific host
	($vhost) = grep { $_->{'host'} eq $args->{'host'} }
			      &list_managed_servers();
	$vhost || return "Xen host system $args->{'host'} does not exist";
	}
elsif ($args->{'location'}) {
	# Select a host from the location
	my ($loc) = grep { $_->{'id'} eq $args->{'location'} ||
			   $_->{'desc'} eq $args->{'location'} }
			 &list_locations();
	$loc || return "Location $args->{'location'} does not exist";
	$vhost = &find_host_from_location($server, $loc, 'xen',
					  $server->{'xen_memory'}*1024*1024,
					  $server->{'xen_disktotal'});
	$vhost || return "Failed to select host system from ".
			 "location $loc->{'desc'}";
	}
else {
	# Use default host
	($vhost) = grep { $_->{'xen_default'} } &list_managed_servers();
	$vhost || return "Missing --xen-host";
	}
$vhost->{'xen_host'} ||
	return "System $args->{'host'} is not registered to host Xen instances";
my ($ok, $err) = &usable_host_system($vhost, "xen",
				     $server->{'xen_memory'}*1024*1024,
				     $server->{'xen_disktotal'});
$ok || return "Host system $vhost->{'host'} cannot be used : $err";
$server->{'xen_on'} = $vhost->{'id'};

# Xen instance name
$server->{'host'} =~ /\./ && return "Xen hostnames cannot contain .";
my $instances = &list_xen_instances($vhost);
my ($clash) = grep { $_->{'instance'} eq $server->{'host'} } @$instances;
$clash && return "A Xen instance with the same name already exists";
$server->{'xen'} = $server->{'host'};

if ($args->{'ip'}) {
	# Make sure IPs are valid
	&can_action(undef, "ips") ||
		return "You are not allowed to specify IP addresses";
	my @ips = split(/[, ]+/, $args->{'ip'});
	foreach my $ip (@ips) {
		&check_ipaddress($ip) || return "Invalid Xen IP address $ip";
		}
	my @cidrs;
	if ($args->{'cidr'}) {
		@cidrs = split(/[, ]+/, $args->{'cidr'});
		foreach my $cidr (@cidrs) {
			$cidr =~ /^\d+$/ && $cidr > 0 && $cidr <= 32 ||
				return "Invalid Xen CIDR $cidr";
			}
		}
	elsif ($args->{'netmask'}) {
		foreach my $nm (split(/[, ]+/, $args->{'netmask'})) {
			&check_ipaddress($nm) ||
				return "Invalid Xen netmask $nm";
			push(@cidrs, &net::mask_to_prefix($nm));
			}
		}
	else {
		return "Missing --xen-cidr or --xen-netmask flags";
		}
	scalar(@ips) == scalar(@cidrs) ||
		return "The number of IPs must match the number of CIDRs";
	if ($args->{'bridge'}) {
		# Bridges set explicitly
		my @hostifaces = split(/[, ]+/, $args->{'bridge'});
		scalar(@ips) == scalar(@hostifaces) ||
		    return "The number of IPs must match the number of bridges";
		$server->{'xen_hostifaces'} = join(' ', @hostifaces);
		}
	else {
		# Automatically determine from host system
		my @hostifaces = split(/\s+/, $vhost->{'xen_iface'});
		if (!@hostifaces) {
			# No host interfaces set, so just assume default
			$server->{'xen_hostifaces'} = '*';
			@ips == 1 ||
			    return "Host system $vhost->{'host'} does not ".
				   "have any bridges set, so only one IP ".
				   "address can be specified";
			}
		else {
			scalar(@ips) <= scalar(@hostifaces) ||
			    return "Host system $vhost->{'host'} only has ".
				   scalar(@hostifaces)." bridges set, but you ".
				   "specified ".scalar(@ips)." IP addresses";
			$server->{'xen_hostifaces'} =
				join(' ', @hostifaces[0..$#ips]);
			}
		}
	$server->{'xen_ip'} = join(' ', @ips);
	$server->{'xen_cidr'} = join(' ', @cidrs);

	# Gateway IPs
	if ($args->{'gw'}) {
		# Set on command line
		my @gws = split(/[, ]+/, $args->{'gw'});
		foreach my $gw (@gws) {
			$gw eq '*' || &check_ipaddress($gw) ||
				return "Invalid gateway IP address $gw";
			}
		scalar(@ips) == scalar(@gws) ||
		  return "The number of IPs must match the number of gateways";
		$server->{'xen_gw'} = join(' ', @gws);
		}
	else {
		# Assume all are using default
		$server->{'xen_gw'} = join(' ', map { "*" } @ips);
		}
	}
else {
	# Allocate IP automatically
	my $err = &allocate_xen_ip_addresses($server, $vhost);
	&error($err) if ($err);
	}
$server->{'ip_addresses'} = $server->{'xen_ip'};

if (&can_action(undef, "advanced")) {
	# MAC address, UUID and vif name
	if ($args->{'mac'}) {
		$args->{'mac'} =~ /^([0-9a-f]{1,2}:)*[0-9a-f]{1,2}$/i ||
			return "Invalid MAC address";
		$server->{'xen_mac'} = $args->{'mac'};
		}
	if ($args->{'uuid'}) {
		$args->{'uuid'} =~ /^[a-f0-9\-]+$/ ||
			return "Invalid UUID";
		$server->{'xen_uuid'} = $args->{'uuid'};
		}
	if ($args->{'vif'}) {
		$args->{'vif'} =~ /^[a-z0-9]+$/ ||
			return "Invalid interface name";
		$server->{'xen_vif'} = $args->{'vif'};
		}

	# On crash/etc options
	foreach my $o (&list_xen_on_actions()) {
		if ($args->{'on-'.$o}) {
			&indexof($args->{'on-'.$o},
				 &list_xen_on_options($o)) >= 0 ||
				return "Invalid option for --xen-on-$o";
			$server->{'xen_on'.$o} = $args->{'on-'.$o};
			}
		elsif ($config{'xen_on_'.$o}) {
			$server->{'xen_on'.$o} = $config{'xen_on_'.$o};
			}
		}

	# Manual config entries
	if ($args->{'manual'}) {
		$server->{'xen_manual'} = $args->{'manual'};
		}

	# Validate LVM flag
	if ($args->{'lvm'} =~ /true|yes/i) {
		$vhost->{'xen_vg'} ||
			return "Host system $vhost->{'host'} does not use LVM";
		}
	elsif ($args->{'lvm'} =~ /false|no/i) {
		$server->{'xen_nolvm'} = 1;
		}

	# Validate directory flag
	if ($args->{'image-dir'}) {
		$server->{'xen_nolvm'} || !$vhost->{'xen_vg'} ||
			return "The --xen-image-dir flag cannot be used when ".
			       "disk images are being stored on LVM";
		$args->{'image-dir'} =~ /^\/\S+$/ ||
			return "The --xen-image-dir flag must be followed by ".
			       "an absolute path";
		$server->{'xen_imagedir'} = $args->{'image-dir'};
		}

	# Validate PyGrub flag
	if ($args->{'pygrub'} =~ /true|yes/i) {
		$server->{'xen_pygrub'} = 1;
		}
	elsif ($args->{'pygrub'} =~ /false|no/i) {
		$server->{'xen_pygrub'} = 0;
		}
	else {
		# Use default
		$server->{'xen_pygrub'} = $config{'xen_pygrub'};
		}
	}

# Adjust hostname for DNS
my @zones = &list_bind_zones();
my $zone;
if ($vhost->{'xen_zone'}) {
	($zone) = grep { $_->{'name'} eq $vhost->{'xen_zone'} } @zones;
	}
if ($zone) {
	$server->{'host'} .= ".".$vhost->{'xen_zone'};
	}

return undef;
}

# type_xen_create(&server, [serial, key], [reset])
# Create a new Xen instance, by unzipping the filesystem image in the background
sub type_xen_create
{
my ($server, $serial, $key, $reset) = @_;
my $vhost = &type_xen_parent($server);
my $img = &get_server_image($server->{'image'});
my $xen_dir = $server->{'xen_imagedir'} || $vhost->{'xen_dir'};

# Create base dir if needed
my @st = &stat_file_on_server($vhost, $xen_dir);
if (!@st) {
	&make_dir_on_server($vhost, $xen_dir, 0755, 1);
	}

# Work out the config and image file locations
my ($fsfile, $swfile, $lvmsuffix);
if ($reset) {
	# Filesystem remains the same
	$fsfile = $server->{'xen_filesystem'};
	$swfile = $server->{'xen_swapfile'};
	}
elsif ($vhost->{'xen_vg'} && !$server->{'xen_nolvm'}) {
	# On LVM logical volumes
	$server->{'xen_filesystem_vg'} = $vhost->{'xen_vg'};
	$server->{'xen_filesystem_lv'} = $server->{'xen'}.'_img';
	$fsfile = "/dev/".$server->{'xen_filesystem_vg'}."/".
		  $server->{'xen_filesystem_lv'};
	if ($server->{'xen_swap'}) {
		$server->{'xen_swapfile_vg'} = $vhost->{'xen_vg'};
		$server->{'xen_swapfile_lv'} = $server->{'xen'}.'_swap';
		$swfile = "/dev/".$server->{'xen_swapfile_vg'}."/".
			  $server->{'xen_swapfile_lv'};
		}
	$lvmsuffix = "phy";
	}
else {
	# On regular files
	$fsfile = "$xen_dir/$server->{'xen'}.img";
	if ($server->{'xen_swap'}) {
		$swfile = "$xen_dir/$server->{'xen'}.swap";
		}
	}
$server->{'xen_filesystem'} = $fsfile;
$server->{'xen_swapfile'} = $swfile;
my $cfgfile = "$xen_dir/$server->{'xen'}.cfg";
$server->{'xen_conf'} = $cfgfile;

# Check for Pv-Grub or PyGrub support if requested
my $bootloader;
if ($server->{'xen_pygrub'}) {
	# First look for Pv-Grub
	my $arch = $img->{'arch'} eq 'x86_64' ? '64' : '32';
	my $pvgrub = "/usr/lib/xen/boot/pv-grub-x86_${arch}.gz";
	if (&stat_file_on_server($vhost, $pvgrub)) {
		$bootloader = $pvgrub;
		}

	# Fall back to checking for PyGrub
	if (!$bootloader) {
		my $pygrub = &has_command_on_server($vhost, "pygrub");
		if (!$pygrub) {
			# Try debian-style location
			my $xenver = &get_xen_software_version($vhost);
			$pygrub = &has_command_on_server($vhost,
					"/usr/lib/xen-$xenver/bin/pygrub");
			}
		if ($pygrub) {
			$bootloader = $pygrub;
			}
		}
	$bootloader ||
		return &text('create_exenpygrub', $vhost->{'host'});
	$server->{'xen_bootloader'} = $bootloader;
	}

# Make sure we have a kernel
if (!$bootloader) {
	my ($kok, $kfile, $ifile) = &setup_xen_host_kernel($vhost);
	$kok || return $kfile;
	}

# Work out interface parameters
my @ips = split(/\s+/, $server->{'xen_ip'});
my @ifaces = split(/\s+/, $server->{'xen_hostifaces'});
my @gws = split(/\s+/, $server->{'xen_gw'});
my @vifs;
for(my $i=0; $i<@ips; $i++) {
	my @vifparams;
	push(@vifparams, "ip=".$ips[$i]);
	if ($server->{'xen_mac'} && $i == 0) {
		$mac = $server->{'xen_mac'};
		}
	else {
		$mac = &generate_mac_address($server, $i);
		}
	push(@vifparams, "mac=".$mac);
	if ($server->{'xen_vif'}) {
		push(@vifparams, "vifname=".$server->{'xen_vif'});
		}
	if ($ifaces[$i] && $ifaces[$i] ne '*') {
		push(@vifparams, "bridge=".$ifaces[$i]);
		}
	if ($server->{'xen_network_rate'}) {
		push(@vifparams,
		     "rate=".int($server->{'xen_network_rate'}/1024)."K");
		}
	my $vifparams = join(",", @vifparams);
	push(@vifs, "'$vifparams'");
	}
my $vifs = join(",", @vifs);

# Create the config file, if missing
if (!$reset) {
	my $temp = &transname();
	my $netmask = &net::prefix_to_mask($server->{'xen_cidr'});
	&open_tempfile(CFG, ">$temp");
	&print_tempfile(CFG, "memory = $server->{'xen_memory'}\n");
	if ($config{'xen_mem_max'}) {
		&print_tempfile(CFG, "maxmem = $config{'xen_mem_max'}\n");
		}
	&print_tempfile(CFG, "name = '$server->{'xen'}'\n");
	&print_tempfile(CFG, "vif = [ $vifs ]\n");
	&print_tempfile(CFG, "address = '$ips[0]'\n");
	&print_tempfile(CFG, "netmask = '$netmask'\n");
	my $suffix = $lvmsuffix || $vhost->{'xen_filetype'} || "file";
	if ($swfile) {
		&print_tempfile(CFG, "disk = ['$suffix:$fsfile,sda1,w',".
				     "'$suffix:$swfile,sda2,w']\n");
		}
	else {
		&print_tempfile(CFG, "disk = ['$suffix:$fsfile,sda1,w']\n");
		}
	if ($server->{'xen_uuid'}) {
		&print_tempfile(CFG, "uuid = '$server->{'xen_uuid'}'\n");
		}
	foreach my $o (&list_xen_on_actions()) {
		if ($server->{'xen_on'.$o}) {
			&print_tempfile(CFG, "on_".$o." = '".
					$server->{'xen_on'.$o}."'\n");
			}
		}
	&close_tempfile(CFG);
	my $err = &put_file_to_server($vhost, $temp, $cfgfile);
	return "Failed to create config file : $err" if ($err);
	$server->{'ip_addresses'} = join(" ", @ips);
	}

# If on an LVM, create it first
$last_remote_error = undef;
if ($server->{'xen_filesystem_vg'}) {
	&remote_foreign_require($vhost, "lvm", "lvm-lib.pl");
	my $lv = { 'name' => $server->{'xen_filesystem_lv'},
		      'size' => int($server->{'xen_disksize'}*1024) ||
				int($img->{'rawsize'}/1024) ||
				1024*1024,
		      'perm' => 'rw',
		      'alloc' => 'n',
		      'vg' => $server->{'xen_filesystem_vg'} };
	if ($reset) {
		# Delete the old LV
		my $err = &remote_foreign_call($vhost, "lvm",
				     "delete_logical_volume",
				     { 'device' => $fsfile });
		$err ||= $last_remote_error;
		return "LVM logical volume deletion failed : $err" if ($err);
		}
	my $err = &remote_foreign_call($vhost, "lvm",
					  "create_logical_volume", $lv);
	$err ||= $last_remote_error;
	return "LVM logical volume creation failed : $err" if ($err);
	}
if ($server->{'xen_swapfile_vg'}) {
	my $lv = { 'name' => $server->{'xen_swapfile_lv'},
		      'size' => int($server->{'xen_swap'}*1024),
		      'perm' => 'rw',
		      'alloc' => 'n',
		      'vg' => $server->{'xen_swapfile_vg'} };
	if ($reset) {
		# Delete the old LV
		my $err = &remote_foreign_call($vhost, "lvm",
				     "delete_logical_volume",
				     { 'device' => $swfile });
		$err ||= $last_remote_error;
		return "LVM logical volume deletion failed : $err" if ($err);
		}
	my $err = &remote_foreign_call($vhost, "lvm",
					  "create_logical_volume", $lv);
	$err ||= $last_remote_error;
	return "LVM logical volume creation for swap failed : $err" if ($err);
	}

# Put the image into place, in the background
my @imgfiles = split(/\s+/, $img->{'files'});
my $imgfile = "$server->{'imagetemp'}/$imgfiles[0]";
my $temp = "/tmp/$$.$server->{'xen'}.gunzip";
$server->{'xen_temp'} = $temp;
if (!$img->{'nocompress'}) {
	# Need to un-compress the image
	my $writer = $fsfile =~ /^\/dev\// ?
			"| dd of=".quotemeta($fsfile)." bs=32768" :
			"> ".quotemeta($fsfile);
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"(gunzip -c ".quotemeta($imgfile)." ".$writer." &&".
		" echo SERVER-MANAGER-COMPLETE || echo SERVER-MANAGER-FAILED)".
		" >$temp 2>&1 </dev/null &");
	}
else {
	# Not compressed
	my $cmd;
	if ($fsfile =~ /^\/dev\//) {
		# DD directly into LVM
		$cmd = "dd if=$imgfile of=$fsfile bs=32768";
		}
	else {
		# Can just move or copy
		$cmd = $server->{'imagelocal'} ? "cp" : "mv";
		$cmd .= " ".quotemeta($imgfile);
		$cmd .= " ".quotemeta($fsfile);
		}
	$cmd = "($cmd && echo SERVER-MANAGER-COMPLETE || echo SERVER-MANAGER-FAILED) >$temp 2>&1 </dev/null &";
	my ($out, $ex, $err) = &execute_command_on_server($vhost, $cmd);
	}

return undef;
}

# allocate_xen_ip_addresses(&server, &vhost)
# Allocate one or more IP addresses for a Xen instance from the host's ranges.
# On success, returns undef and sets xen_ip, xen_cidr and xen_iface to space-
# separated lists. On failure returns an error message.
sub allocate_xen_ip_addresses
{
my ($server, $vhost) = @_;

# Work out interfaces on host system
my @hostifaces;
if ($vhost->{'xen_iface'}) {
	@hostifaces = split(/\s+/, $vhost->{'xen_iface'});
	}
else {
	@hostifaces = ( "*" );
	}

# For each, allocate an IP from a range
foreach my $iface (@hostifaces) {
	my ($ip, $cidr, $gw) = &allocate_ip_address(
		$vhost->{'xen_start'}, $vhost->{'xen_end'},
	        $vhost->{'xen_cidr'}, $vhost->{'xen_gws'},
		$vhost->{'xen_ifaces'}, $iface);
	$ip || return &text('create_exenalloc', $vhost->{'host'},
			    &describe_ip_range($vhost->{'xen_start'},
					       $vhost->{'xen_end'},
					       $vhost->{'xen_cidr'},
                                               $vhost->{'xen_ifaces'},
                                               $iface));
	push(@ips, $ip);
	push(@cidrs, $cidr);
	push(@gws, $gw || "*");
	}
$server->{'xen_ip'} = join(" ", @ips);
$server->{'xen_cidr'} = join(" ", @cidrs);
$server->{'xen_hostifaces'} = join(" ", @hostifaces);
$server->{'xen_gw'} = join(" ", @gws);
return undef;
}

# setup_xen_host_kernel(&vhost)
# Copies the kernel and initrd file (if needed) into place on a Xen host.
# Returns 1, the kernel file and initrd file if successful, or 0 and an
# error message on failure.
sub setup_xen_host_kernel
{
my ($vhost) = @_;

# Get kernel
my ($uout, $ex, $err) = &execute_command_on_server($vhost, "uname -r");
$uout =~ s/\r|\n//g;
$uout || return (0, "Could not work out host kernel version : ".
		    "uname reported $uout");

# Work out kernel to use, and copy to Xen dir if needed
my $kfile = "$vhost->{'xen_dir'}/vmlinuz-vm2-xenU";
my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"test -r ".quotemeta($kfile));
if ($ex) {
	# First check for Debian-style Xen kernel
	my $srckfile = "/boot/xen0-linux-".$uout;
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
			"test -r ".quotemeta($srckfile));
	if ($ex) {
		# Try same kernel we are currently running, as seen on CentOS
		$srckfile = "/boot/vmlinuz-".$uout;
		my ($out, $ex, $err) = &execute_command_on_server($vhost,
				"test -r ".quotemeta($srckfile));
		if ($ex) {
			return (0, "Could not find kernel image at $srckfile");
			}
		}
	($out, $ex, $err) = &execute_command_on_server($vhost,
		"cp ".quotemeta($srckfile)." ".quotemeta($kfile));
	if ($ex) {
		return (0, "Failed to copy kernel image : $out");
		}
	}

# Create initrd from kernel if needed
my $ifile = "$vhost->{'xen_dir'}/initrd.vm2.xenU.img";
($out, $ex, $err) = &execute_command_on_server($vhost,
		"test -r ".quotemeta($ifile));
if ($ex) {
	# First check for Debian-style existing initrd file
	my $debifile = "/boot/xen0-linux-".$uout.".initrd.img";
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
			"test -r ".quotemeta($debifile));
	if ($ex) {
		# Newer Debian-style initrd file
		$debifile = "/boot/initrd.img-".$uout;
		($out, $ex, $err) = &execute_command_on_server($vhost,
				"test -r ".quotemeta($debifile));
		}
	if (!$ex) {
		# Found it .. just copy into place
		($out, $ex, $err) = &execute_command_on_server($vhost,
			"cp ".quotemeta($debifile)." ".quotemeta($ifile));
		if ($ex) {
			return (0, "Failed to copy initial ramdisk : $out");
			}
		}
	else {
		($out, $ex, $err) = &execute_command_on_server($vhost,
			"mkinitrd ".quotemeta($ifile)." $uout --with xenblk ".
			"--with xennet --preload xenblk --preload xennet");
		if ($ex) {
			return (0, "Failed to create initial ramdisk : $out");
			}
		}
	}

return (1, $kfile, $ifile);
}

# type_xen_create_done(&server)
# Checks if the filesystem gunzip is complete, and returns OK if so.
# Returns undef if still running, or an error message on failure.
sub type_xen_create_done
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);

# Read the gzip output file
my $data = &get_file_data_from_server($vhost, $server->{'xen_temp'});
if ($data =~ /SERVER-MANAGER-FAILED/) {
	$data =~ s/SERVER-MANAGER-FAILED\n//;
	&execute_command_on_server($vhost,
		"rm -f ".quotemeta($server->{'xen_temp'}));
	return "Failed to extract filesystem : $data";
	}
elsif ($data =~ /SERVER-MANAGER-COMPLETE/) {
	&execute_command_on_server($vhost,
		"rm -f ".quotemeta($server->{'xen_temp'}));
	return "OK";
	}
else {
	return undef;
	}
}

# type_xen_create_post(&server, [serial, key], [reset])
# Called after creation is complete. Here we can mount and fix up config
# files in the filesytem, then start the instance.
sub type_xen_create_post
{
my ($server, $serial, $key, $reset) = @_;
my $vhost = &type_xen_parent($server);
my @ips = split(/\s+/, $server->{'xen_ip'});
my @cidrs = split(/\s+/, $server->{'xen_cidr'});
my @netmasks = map { &net::prefix_to_mask($_) } @cidrs;
$server->{'status'} = 'down';

# Gateways can either be set per IP address, or copy from host system.
# If none are set, assume the first IP uses the host system gateway.
my $defgw = &get_server_default_gateway($vhost);
my @gws = split(/\s+/, $server->{'xen_gw'});
my @setgws = grep { $_ ne "*" } @gws;
if (!@setgws && $defgw) {
	@gws = ( $defgw );
	}

# Create the swap file, if needed
my ($swsize, $swfile);
if ($server->{'xen_swapfile'}) {
	$swsize = $server->{'xen_swap'}*1024*1024;
	$swfile = $server->{'xen_swapfile'};
	my @st = &stat_file_on_server($vhost, $swfile);
	if (!@st || $server->{'xen_swapfile_vg'}) {
		&$first_print(&text('create_xenswap', &nice_size($swsize)));
		my ($out, $ex, $err) = &execute_command_on_server($vhost,
			"dd if=/dev/zero of=".quotemeta($swfile)." bs=1024 ".
			"count=".int($swsize/1024)." && ".
			"mkswap ".quotemeta($swfile));
		if ($err) {
			&$second_print(&text('create_exenswap', $out));
			}       
		else {
			&$second_print($text{'create_done'});
			}
		}
	}

# Expand the filesystem image, if needed
my $fsfile = $server->{'xen_filesystem'};
if ($server->{'xen_disksize'}) {
	my $dksize = $server->{'xen_disksize'}*1024*1024;
	&$first_print(&text('create_xenresize', &nice_size($dksize)));
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"e2fsck -f -p ".quotemeta($fsfile)." ; ".
		"resize2fs ".quotemeta($fsfile)." ".
		$server->{'xen_disksize'}."M");
        if ($ex) {
                &$second_print(&text('create_exenresize', $out));
                }       
        else {
                &$second_print($text{'create_done'});
                }
	}


# Make sure swap and root filesystem files are writable
foreach my $f ($fsfile, $swfile ? ( $swfile ) : ()) {
	if ($f !~ /^\/dev\//) {
		&execute_command_on_server($vhost, "chmod 700 ".quotemeta($f));
		}
	}

# Mount the created filesystem
&$first_print($text{'create_xenmount'});
my $err = &mount_filesystem_if_needed($server);
my $mp = $server->{'xen_mount'};
if ($err || !$mp) {
	&$second_print(&text('create_exenmount', $err));
	return;
	}
&$second_print(&text('create_xenmounted', "<tt>$mp</tt>"));

# Setup kernel in config file
&$first_print($text{'create_xenkernel'});
my $kerr;
my $grubfile;
if ($server->{'xen_pygrub'}) {
	# Make sure filesystem has menu.lst
	foreach my $f ("/boot/grub/grub.conf", "/boot/grub/menu.lst") {
		if (&stat_file_on_server($server, $f)) {
			$grubfile = $f;
			}
		}
	if (!$grubfile) {
		delete($server->{'xen_pygrub'});
		delete($server->{'xen_bootloader'});
		$kerr = $text{'create_xenkernelhost2'};
		}
	}
my $cfg = &get_xen_config($server);
my $bootloader = $server->{'xen_bootloader'};
if ($server->{'xen_pygrub'} && $bootloader =~ /pygrub/) {
	# Configure to use pygrub
	$kerr = $text{'create_xenkernelpy'};
	&xen_save_directive($cfg, "bootloader", $bootloader);
	&xen_save_directive($cfg, "root");
	&xen_save_directive($cfg, "ramdisk");
	&xen_save_directive($cfg, "kernel");
	&xen_save_directive($cfg, "extra");
	}
elsif ($server->{'xen_pygrub'} && $bootloader =~ /pv-grub/) {
	# Configure to use pv-grub
	$kerr = $text{'create_xenkernelpv'};
	&xen_save_directive($cfg, "kernel", $bootloader);
	&xen_save_directive($cfg, "extra", "(hd0)".$grubfile);
	&xen_save_directive($cfg, "root");
	&xen_save_directive($cfg, "ramdisk");
	&xen_save_directive($cfg, "bootloader");
	}
else {
	# Configure to use kernel from host
	($kok, $kfile, $ifile) = &setup_xen_host_kernel($vhost);
	$kerr ||= $text{'create_xenkernelhost'};
	&xen_save_directive($cfg, "kernel", $kfile);
	&xen_save_directive($cfg, "ramdisk", $ifile);
	&xen_save_directive($cfg, "root", "/dev/sda1 ro");
	&xen_save_directive($cfg, "extra");
	&xen_save_directive($cfg, "bootloader");
	}

# Add manual entries to Xen config file
if ($server->{'xen_manual'} && !$reset) {
	my $temp = &transname();
	&open_tempfile(TEMP, ">>$cfg->[0]->{'file'}");
	foreach my $m (split(/\t/, $server->{'xen_manual'})) {
		&print_tempfile(TEMP, "$m\n");
		}
	&close_tempfile(TEMP);
	}

# Write out the Xen config
my $err = &put_xen_config($server, $cfg);
$kerr = $err if ($err);
&$second_print($kerr);

# Fix root in grub config
if ($server->{'xen_pygrub'}) {
	&$first_print($text{'create_xengrub'});
	my $grubconf = &get_file_data_from_server($server, $grubfile);
	if ($grubconf) {
		$grubconf =~ s/root\s+\(hd\d+(,\d+)?\)/root \(hd0\)/;
		&put_file_data_to_server($server, $grubconf, $grubfile);
		}
	&$second_print($text{'create_done'});
	}

# Copy kernel modules to the filesystem, if missing
if (!$server->{'xen_pygrub'}) {
	my ($uout, $ex, $err) = &execute_command_on_server($vhost, "uname -r");
	$uout =~ s/\r|\n//g;
	my $moddir = "$mp/lib/modules/$uout";
	($out, $ex, $err) = &execute_command_on_server($vhost,
		"test -d ".quotemeta($moddir));
	if ($ex) {
		&$first_print(&text('create_xenmods', $uout));
		($out, $ex, $err) = &execute_command_on_server($vhost,
		    "cp -r ".quotemeta("/lib/modules/$uout")." ".
			     quotemeta($moddir));
		if ($ex) {
			&$second_print(&text('create_exenmods',
					     &html_escape($out)));
			}
		else {
			&$second_print($text{'create_done'});
			}
		}
	}

# Set the root password, by directly editing /etc/shadow
if ($server->{'ssh_pass'}) {
	&$first_print($text{'create_xensetpass'});
	my $err = &set_shadow_root_password($server, $server->{'ssh_pass'});
        if ($err) {
                &$second_print(&text('create_exensetpass', $err));
                }       
        else {
                &$second_print($text{'create_done'});
                }
	}

# Create authorized keys file
if ($server->{'key'}) {
	&$first_print($text{'create_xensetkey'});
	my $err = &create_authorized_key($server);
	&$second_print($err ? &text('create_exensetkey', $err)
			    : $text{'create_done'});
	}
else {
	&clear_authorized_key($server);
	}

# Add extra SSH keys
if (&has_addnew_keys($server)) {
	&$first_print($text{'create_xenaddnewkeys'});
	my $err = &addnew_authorized_keys($server);
	&$second_print($err ? &text('create_exenaddnewkeys', $err)
			    : $text{'create_done'});
	}

# Fix other config files that refer to the hostname or IP
&$first_print($text{'create_xenfix'});
my $fullhost = $server->{'unix_hostname'} ?
			$server->{'unix_hostname'} :
	       $vhost->{'xen_zone'} ? 
			$server->{'xen'}.".".$vhost->{'xen_zone'} :
			$server->{'xen'};
my $err = &fix_hostname_config_files($server, $ips[0],
					$server->{'xen'}, $fullhost, $vhost,
					$mp, $vhost->{'xen_resolv'});
if ($err) {
	&$second_print(&text('create_exenfix', $err));
	}       
else {
	&$second_print($text{'create_done'});
	}

# Setup networking config files to use the IP.
&$first_print($text{'create_xennet'});
my $err = &fix_network_config_files($server, $ips[0], $netmasks[0],
				    $gws[0], $vhost, $mp);
if ($err) {
	&$second_print(&text('create_exennet', $err));
	}       
else {
	&$second_print($text{'create_done'});
	}

# Add network configs for extra IPs
if (@ips > 1) {
	&$first_print($text{'create_xennet2'});
	my $err = &fix_additional_network_interface_files($server,
		[ @ips[1..$#ips] ], [ @netmasks[1..$#netmasks] ],
		[ @gws[1..$#gws] ], $vhost, $mp);
	if ($err) {
		&$second_print(&text('create_exennet2', $err));
		}       
	else {
		&$second_print($text{'create_done'});
		}
	}

# Add the swap file to /etc/fstab, if any
&$first_print($text{'create_xenfstab'});
my $fstab = &get_system_fstab($server);
if (!ref($fstab)) {
	&$second_print(&text('create_exenswaptab', $fstab));
	}
else {
	my $changed_fstab;
	if ($server->{'xen_swapfile'}) {
		# Add swap device
		my ($sw) = grep { $_->[0] eq '/dev/sda2' } @$fstab;
		if (!$sw) {
			push(@$fstab, [ '/dev/sda2', 'swap', 'swap',
					'defaults', 0, 0, ]);
			$changed_fstab++;
			}
		}
	if (!$server->{'xen_pygrub'}) {
		# Change /dev/xvd* to /dev/sda1 for root device
		my ($root) = grep { $_->[1] eq '/' } @$fstab;
		if ($root && $root->[0] =~ /^\/dev\/xvd\S+/) {
			$root->[0] = "/dev/sda1";
			$changed_fstab++;
			}
		}
	if ($changed_fstab) {
		# Write out again
		my $err = &put_system_fstab($server, $fstab);
		if ($err) {
			&$second_print(&text('create_exenswaptab', $err));
			}
		else {
			&$second_print($text{'create_done'});
			}
		}
	else {
		&$second_print(&text('create_xenswaptabok'));
		}
	}

# Fix Virtualmin licence files
if ($serial) {
	&$first_print($text{'create_xenlicence'});
	my $err = &fix_virtualmin_licence_files($server, $serial, $key);
	if ($err) {
		&$second_print(&text('create_exenlicence', $err));
		}       
	else {
		&$second_print($text{'create_done'});
		}
	}

# Configure SSH to allow logins by root
&$first_print($text{'create_zonesrootssh'});
my $err = &fix_ssh_root_login($server);
if ($err eq "OK") {
	&$second_print($text{'create_zonesrootsshok'});
	}
elsif ($err eq "NONE") {
	&$second_print($text{'create_zonesrootsshnone'});
	}
elsif ($err) {
	&$second_print($err);
	}
else {
	&$second_print($text{'create_done'});
	}

# Make sure /dev/null is writable, and nash is killed
my $rcl = &transname();
my $err = &get_file_from_server($server, "/etc/rc.local", $rcl);
my $lref = &read_file_lines($rcl);
my ($foundnull, $foundnash, $lnum, $exitline);
foreach my $l (@$lref) {
	if ($l =~ /^chmod.*\/dev\/null/) {
		$foundnull++;
		}
	elsif ($l =~ /^killall(\s+\-\d+)nash/) {
		$foundnash++;
		}
	elsif ($l =~ /^exit/) {
		$exitline = $lnum;
		}
	$lnum++;
	}
$exitline ||= scalar(@$lref);
if (!$foundnull || !$foundnash) {
	splice(@$lref, $exitline, 0, "chmod 777 /dev/null")
		if (!$foundnull);
	splice(@$lref, $exitline, 0, "killall -9 nash nash-hotplug")
		if (!$foundnash);
	&flush_file_lines($rcl);
	&put_file_to_server($server, $rcl, "/etc/rc.local");
	}

# Add DHCP entries
if ($config{'dhcp_add'} && !$reset) {
	my $fakeifaces = &type_xen_list_interfaces($server, 1);
	if (ref($fakeifaces)) {
		&add_all_iface_dhcpd($server, $fakeifaces);
		}
	}

# Clean up /etc/fstab to remove non-existant devices
&$first_print($text{'create_xencleanfstab'});
my $err = &cleanup_server_fstab($server);
if ($err) {
	&$second_print(&text('create_exencleanfstab', $err));
	}       
else {
	&$second_print($text{'create_done'});
	}

# Un-mount
&$first_print($text{'create_xenumount'});
&umount_filesystem_if_needed($server);
&$second_print($text{'create_done'});

# Compute total disk size
&type_xen_compute_disksize($server);

# Register in DNS, if we can
if ($vhost->{'xen_zone'} && !$reset) {
	my $dnsname = $server->{'xen'}.".".$vhost->{'xen_zone'}.".";
	if (&add_bind8_record($dnsname, $ips[0],
			      $vhost->{'xen_zone'})) {
		$server->{'xen_dns'} = $dnsname;
		}
	}

# Setup VNC console in config file
if ($config{'xen_vnc'}) {
	&$first_print($text{'create_xenvnc'});
	my $err = &setup_xen_vnc_console($server);
	&$second_print($err ? &text('create_xenvncfailed', $err)
			    : &text('create_xenvncdone',
				    $server->{'xen_vnc_display'}));
	}

# Start up the instance
&$first_print($text{'create_xenstartup'});
my ($ok, $err) = &type_xen_startup($server);
my $started = 0;
if ($ok != 1) {
	&$second_print(&text('create_exenstartup', $err));
	}       
else {
	&$second_print($text{'create_done'});
	$started = 1;
	}

# Import Virtualmin package keys
if ($serial && $started) {
	&$first_print($text{'create_xenvkeys'});
	my $err = &fix_virtualmin_after_boot($server);
	if ($err) {
		&$second_print(&text('create_exenvkeys', $err));
		}       
	else {
		&$second_print($text{'create_done'});
		}
	}

# Regenerate SSH host key
if ($started) {
	&$first_print($text{'create_xenhostkey'});
	my $err = &regenerate_ssh_host_keys($server);
	&$second_print($err ? &text('create_exensetkey', $err)
			    : $text{'create_done'});
	}

return undef;
}

# type_xen_delete(&server)
# Shut down the server, then delete the filesystem file, config file and
# swap files.
sub type_xen_delete
{
my ($server) = @_;

# Force shutdown
my $vhost = &type_xen_parent($server);
&execute_command_on_server($vhost, "xm destroy $server->{'xen'}");
&type_xen_set_atboot($server, 0);

# Delete virtual disks
my $disks = &type_xen_list_disks($server, 0);
if (ref($disks)) {
	foreach my $d (@$disks) {
		my @st = &stat_file_on_server($vhost, $d->{'real'});
		next if (!@st);		# Already gone
		if (!$d->{'device'} && $d->{'real'} !~ /^\/dev\//) {
			my ($out, $ex, $err) = &execute_command_on_server(
				$vhost, "rm -f ".quotemeta($d->{'real'}));
			return $err if ($err);
			return $out if ($ex);
			}
		elsif ($d->{'lv'}) {
			# Delete the logical volume
			$last_remote_error = undef;
			&remote_foreign_require($vhost, "lvm", "lvm-lib.pl");
			return $last_remote_error if ($last_remote_error);
			&remote_foreign_call($vhost, "lvm",
					     "delete_logical_volume",
					     { 'device' => $d->{'real'} });
			return "Failed to delete LV $d->{'real'} : ".
			       "$last_remote_error" if ($last_remote_error);
			}
		}
	}
else {	
	my @st = &stat_file_on_server($vhost, $server->{'xen_conf'});
	if (@st) {
		return "Failed to get list of disks : $disks";
		}
	}

# Delete config files
foreach my $f ($server->{'xen_conf'}, $server->{'xen_console'}) {
	next if (!$f);
	my ($out, $ex, $err) = &execute_command_on_server($vhost, 
					"rm -f ".quotemeta($f));
	return $err if ($err);
	return $out if ($ex);
	}

# Remove the DNS entry, if we created one
&delete_bind8_record($server->{'xen_dns'}, $vhost->{'xen_zone'},
		     &to_ipaddress($server->{'host'}));

# Remove DHCP entries
&delete_all_iface_dhcpd($server);

return undef;
}

# type_xen_get_file(&server, remotefile, localfile)
# If the filesystem for this Xen instance is locally mounted, get the file
# directly from there
sub type_xen_get_file
{
my ($server, $remotefile, $localfile) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	return &get_file_from_server($vhost, $server->{'xen_mount'}.$remotefile,
					     $localfile);
	}
else {
	return 'FALLBACK';
	}
}

# type_xen_put_file(&server, localfile, remotefile)
# If the filesystem for this Xen instance is locally mounted, write the file
# directly to there
sub type_xen_put_file
{
my ($server, $localfile, $remotefile) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	return &put_file_to_server($vhost, $localfile,
				   $server->{'xen_mount'}.$remotefile);
	}
else {
	return 'FALLBACK';
	}
}

# type_xen_put_multiple_files(&server, &localfiles, remotedir)
# If the filesystem for this Xen instance is locally mounted, write the file
# directly to there
sub type_xen_put_multiple_files
{
my ($server, $localfiles, $remotedir) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	return &put_multiple_files_to_server($vhost, $localfiles,
				   $server->{'xen_mount'}.$remotedir);
	}
else {
	return 'FALLBACK';
	}
}

# type_xen_delete_file(&server, remotefile, noquote)
# If the filesystem for this Xen instance is locally mounted, delete the file
# directly from there
sub type_xen_delete_file
{
my ($server, $remotefile, $noquote) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	return &delete_file_on_server($vhost,
			      $server->{'xen_mount'}.$remotefile, $noquote);
	}
else {
	return 'FALLBACK';
	}
}

# type_xen_checksum_file(&server, remotefile, [data-size])
# If the filesystem for this Xen instance is locally mounted, checksum the file
# directly from there
sub type_xen_checksum_file
{
my ($server, $file, $size) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	return &checksum_file_on_server($vhost,
			      $server->{'xen_mount'}.$file, $size);
	}
else {
	return ( 'FALLBACK' );
	}
}

# type_xen_stat_file(&server, remotefile, localfile)
# If the filesystem for this Xen instance is locally mounted, stat the file
# directly from there
sub type_xen_stat_file
{
my ($server, $remotefile) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	return &stat_file_on_server($vhost, $server->{'xen_mount'}.$remotefile);
	}
else {
	return 'FALLBACK';
	}
}

# type_xen_make_dir(&server, dir, perms, recur)
# If the filesystem for this Xen instance is locally mounted, make the dir
# directly from there
sub type_xen_make_dir
{
my ($server, $remotedir, $perms, $recur) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	return &make_dir_on_server($vhost, $server->{'xen_mount'}.$remotedir,
				   $perms, $recur);
	}
else {
	return 'FALLBACK';
	}
}

# type_xen_list_dir(&server, dir)
# If the filesystem for this Xen instance is locally mounted, list the dir
# directly from there
sub type_xen_list_dir
{
my ($server, $remotedir) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	return &list_dir_on_server($vhost, $server->{'xen_mount'}.$remotedir);
	}
else {
	return 'FALLBACK';
	}
}

# type_xen_get_atboot(&server)
# Returns 1 if some some Xen instance is started at boot time, by looking
# for a link to its config from /etc/xen/auto
sub type_xen_get_atboot
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
my $cfile = $server->{'xen_conf'};
$cfile =~ s/^(.*)\///;
my ($out, $ex, $err) = &execute_command_on_server($vhost,
	"readlink /etc/xen/auto/$cfile");
$out =~ s/\r|\n//g;
return $out eq $server->{'xen_conf'};
}

# type_xen_set_atboot(&server, yesno)
# Enable some Xen system at boot time, by linking it's config file to
# /etc/xen/auto
sub type_xen_set_atboot
{
my ($server, $boot) = @_;
my $vhost = &type_xen_parent($server);
my $cfile = $server->{'xen_conf'};
$cfile =~ s/^(.*)\///;
my ($out, $ex, $err);
if ($boot) {
	&execute_command_on_server($vhost,	# For Redhat
		"chkconfig xendomains on");
	&make_dir_on_server($vhost, "/etc/xen/auto", 0755, 1);
	($out, $ex, $err) = &execute_command_on_server($vhost,
		"rm -f ".quotemeta("/etc/xen/auto/$cfile")." ; ".
		"ln -s ".quotemeta($server->{'xen_conf'})." ".
			 quotemeta("/etc/xen/auto/$cfile"));
	}
else {
	($out, $ex, $err) = &execute_command_on_server($vhost,
		"rm -f ".quotemeta("/etc/xen/auto/$cfile"));
	}
return $err ? $err : $ex ? $out : undef;
}

# type_xen_create_image(&server, &image, tempdir, [nomunge])
# Convert a Xen system into an image file, which is really just the Xen disk.
# Outputs text, and returns 0 on failure.
sub type_xen_create_image
{
my ($server, $image, $tempdir, $nomunge) = @_;
my $vhost = &type_xen_parent($server);

# Mount the disk image
&$first_print($text{'mkimage_xenmount'});
my $err = &mount_filesystem_if_needed($server);
if ($err) {
	&$second_print(&text('create_exenmount', $err));
	return 0;
	}
&$second_print(&text('create_xenmounted', "<tt>$server->{'xen_mount'}</tt>"));

if ($nomunge != 1) {
	# Munge some or all files
	my ($mok, $munged) = &munge_all_files($server, 0, $nomunge == 2);
	return 0 if (!$mok);
	$server->{'munged'} = $munged;
	}

# Check if the system has grub
if (&stat_file_on_server($server, "/boot/grub/grub.conf")) {
	$img->{'xen_pygrub'} = 1;
	}

# Un-mount
&$first_print($text{'mkimage_xenumount'});
&umount_filesystem_if_needed($server);
&$second_print($text{'create_done'});

# Get the size of the raw disk
my $disks = &type_xen_list_disks($server, 0);
my $rawsize;
if (ref($disks)) {
	my ($d) = grep { $_->{'real'} eq $server->{'xen_filesystem'} } @$disks;
	if ($d) {
		$rawsize = $d->{'size'};
		}
	}
if (!$rawsize) {
	my @st = &stat_file_on_server($vhost, $server->{'xen_filesystem'});
	$rawsize = $st[7];
	}
$image->{'rawsize'} = $rawsize;

if (!$image->{'nocompress'}) {
	# Compress disk image to a temp file
	&$first_print($text{'mkimage_xengzip'});
	my $gzfile = $image->{'id'}.".ext3.gz";
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
	    ($server->{'xen_filesystem_lv'} ?
		"dd if=".quotemeta($server->{'xen_filesystem'}).
	          " bs=32768 | gzip -c" :
		"gzip -c ".quotemeta($server->{'xen_filesystem'})).
	    " >".quotemeta("$tempdir/$gzfile"));
	if ($ex) {
		&$second_print(&text('mkimage_exengzip', "<pre>$out</pre>"));
		}
	else {
		&$second_print($text{'create_done'});
		$image->{'files'} = $gzfile;
		}
	}
else {
	# Just use the image file as-is
	&$first_print($text{'mkimage_xengzip2'});
	my $imgfile = $image->{'id'}.".ext3";
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		$server->{'xen_filesystem_lv'} ?
			"dd if=".quotemeta($server->{'xen_filesystem'}).
			  " of=".quotemeta("$tempdir/$imgfile")." bs=32768" :
			"ln -s ".quotemeta($server->{'xen_filesystem'})." ".
			  quotemeta("$tempdir/$imgfile"));
	if ($ex) {
		&$second_print(&text('mkimage_exenlink', "<pre>$out</pre>"));
		}
	else {
		&$second_print($text{'create_done'});
		$image->{'files'} = $imgfile;
		}
	}

return 1;
}

# type_xen_cleanup_image(&server, &image, tempdir)
# Delete all temp files created on the host system for an image, and reverse
# any munge processes that were done to prepare the imaged system.
sub type_xen_cleanup_image
{
my ($server, $image, $tempdir) = @_;
my $vhost = &type_xen_parent($server);

# Remove compressed image
&$first_print($text{'mkimage_xendelete'});
my $gzfile = $image->{'id'}.".ext3.gz";
my ($out, $ex, $err) = &execute_command_on_server($vhost,
				"rm -f ".quotemeta("$tempdir/$gzfile"));
$err = $out if ($ex);
if ($err) {
	&$second_print(&text('mkimage_exendelete', $out));
	return 0;
	}
else {
	&$second_print($text{'create_done'});
	}

# Re-mount Xen filesystem
&$first_print($text{'mkimage_xenmount'});
my $err = &mount_filesystem_if_needed($server);
if ($err) {
	&$second_print(&text('create_exenmount', $err));
	return 0;
	}
&$second_print(&text('create_xenmounted', "<tt>$server->{'xen_mount'}</tt>"));

# Undo all munged files
&unmunge_all_files($server, $server->{'munged'});
delete($server->{'munged'});

# Un-mount Xen filesystem
&$first_print($text{'mkimage_xenumount'});
&umount_filesystem_if_needed($server);
&$second_print($text{'create_done'});

return 1;
}

# Returns 1, indicating that memory can be changed without a reboot
sub type_xen_can_memory
{
return 1;
}

sub type_xen_can_unlimited_memory
{
return 0;
}

# type_xen_get_memory(&server)
# Returns the amount of RAM in bytes this server is granted, or undef if
# not available.
sub type_xen_get_memory
{
my ($server) = @_;
my $conf = &get_xen_config($server, 1);
return undef if (!ref($conf));
my $m = &xen_find_value("memory", $conf);
return $m ? $m*1024*1024 : undef;
}

# type_xen_save_memory(&server, ram)
# Updates the memory granted in the .cfg file, and apply it with xm set-mem
sub type_xen_save_memory
{
my ($server, $mem) = @_;
my $mbs = int($mem/(1024*1024));
my $conf = &get_xen_config($server, 0);
return "Failed to fetch current configuration" if (!ref($conf));
&xen_save_directive($conf, "memory", $mbs);
my $err = &put_xen_config($server, $conf);
return $err if ($err);

# Apply for real, if possible
if ($server->{'status'} ne 'down' && $server->{'status'} ne 'nohost' &&
    !$config{'xen_mem_mode'}) {
	my $vhost = &type_xen_parent($server);
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"xm mem-set ".$server->{'xen'}." ".$mbs);
	$err = $out if ($ex);
	return "The Xen configuration was updated, but an error occurred applying the new memory limit : $err" if ($err);
	}

$server->{'xen_memory'} = $mbs;
return undef;
}

# Returns 3, indicating that CPU and weighting can be changed without a reboot
sub type_xen_can_cpu
{
return 3;
}

# type_xen_get_cpu(&server)
# Returns the percentage of a CPU granted to this server, or 0 for unlimited,
# and the weighting factor
sub type_xen_get_cpu
{
my ($server) = @_;
if ($server->{'status'} eq 'down') {
	# Use limits from config
	return ( $server->{'xen_credit_cpu'} || 0,
		 $server->{'xen_credit_weight'} || 256 );
	}
my $vhost = &type_xen_parent($server);
my ($out, $ex, $err) = &execute_command_on_server($vhost,
				"xm sched-credit -d ".$server->{'xen'});
return undef if ($err || $ex);
if ($out =~ /'cap'\s*:\s*(\d+),\s*'weight'\s*:\s*(\d+)/) {
	# Old Xen 3.0 format
	return ($1, $2);
	}
elsif ($out =~ /\S+\s+\d+\s+(\d+)\s+(\d+)/) {
	# New Xen 3.3 format
	return ($2, $1);
	}
else {
	return undef;
	}
}

# type_xen_save_cpu(&server, cpu, weight)
# Updates the CPU cap and weighting in our config file, and applies them
sub type_xen_save_cpu
{
my ($server, $cpu, $weight) = @_;
my $vhost = &type_xen_parent($server);
if (defined($cpu)) {
	# Apply percent now, and save in Cloudmin server object
	if ($server->{'status'} ne 'down') {
		my ($out, $ex, $err) = &execute_command_on_server($vhost,
			"xm sched-credit -d ".$server->{'xen'}." -c $cpu");
		return $err if ($err);
		return $out if ($ex);
		}
	$server->{'xen_credit_cpu'} = $cpu;
	}
if (defined($weight)) {
	# Apply weight now, and save in Cloudmin server object
	if ($server->{'status'} ne 'down') {
		my ($out, $ex, $err) = &execute_command_on_server($vhost,
			"xm sched-credit -d ".$server->{'xen'}." -w $weight");
		return $err if ($err);
		return $out if ($ex);
		}
	$server->{'xen_credit_weight'} = $weight;
	}
if (defined($cpu) || defined($weight)) {
	# Save in Xen config
	my $conf = &get_xen_config($server, 0);
	if (ref($conf)) {
		if (defined($cpu)) {
			&xen_save_directive($conf, "cpu_cap", $cpu);
			}
		if (defined($weight)) {
			&xen_save_directive($conf, "cpu_weight", $weight);
			}
		&put_xen_config($server, $conf);
		}
	}
return undef;
}

# type_xen_get_cpu_load(&server)
# Computes the current CPU utilization of the given system, by seeing how many
# CPU seconds increased since the last check. Returns a percentage and number
# of seconds it was over, or an empty array of this could not be computed.
sub type_xen_get_cpu_load
{
my ($server) = @_;
return undef if ($server->{'status'} eq 'down' ||
	         $server->{'status'} eq 'nohost');
my $vhost = &type_xen_parent($server);
my $now = time();
my ($out, $ex, $err) = &execute_command_on_server($vhost,
				"xm list ".$server->{'xen'});
if (!$ex && !$err &&
    $out =~ /\Q$server->{'xen'}\E\s+\d+\s+\d+\s+\S+\s+\S+\s+([0-9\.]+)/) {
	# Got current .. compare with last time
	my $current = $1;
	my %secs;
	my @rv;
	&read_file_cached($xen_cpu_seconds_file, \%secs);
	if ($secs{$server->{'id'}}) {
		my ($lastsecs, $lasttime) =
			split(/\s+/, $secs{$server->{'id'}});
		if ($lasttime && $lastsecs < $current) {
			@rv = ( 100 * ($current-$lastsecs) / ($now-$lasttime),
				$now-$lasttime );
			}
		}
	$secs{$server->{'id'}} = $current." ".$now;
	&write_file($xen_cpu_seconds_file, \%secs);
	return @rv;
	}
return ( );
}

# type_xen_can_vcpus(&server)
# Returns 2, meaning that virtual CPUs can be managed but a reboot is needed
sub type_xen_can_vcpus
{
return 2;
}

# type_xen_get_vcpus(&server)
# Returns a list of virtual CPU hash refs for this server. Each contains
# a 'num' and 'pin' fields, where the pin is the list of real CPUs it is
# restricted to. On failure, returns an error message.
# Each pin can either be a single number, or a range like 0-1
sub type_xen_get_vcpus
{
my ($server) = @_;
my $conf = &get_xen_config($server, 1);
return $conf if (!ref($conf));
my $vcpus = &xen_find_value("vcpus", $conf);
my @cpus = split(/,/, &xen_find_value("cpus", $conf));
my $cpu = &xen_find_value("cpu", $conf);
$vcpus ||= 1;
my @rv;
my $allcpus = &type_xen_list_pin_cpus($server);
my $pinall;
if (ref($allcpus) && @$allcpus > 1) {
	$pinall = "0-".(scalar(@$allcpus)-1);
	}
for(my $i=0; $i<$vcpus; $i++) {
	my $c = { 'num' => $i };
	if (@cpus) {
		$c->{'pin'} = $cpus[$i];
		}
	elsif ($cpu ne '' && $i == 0) {
		$c->{'pin'} = $cpu;
		}
	if ($c->{'pin'} && $c->{'pin'} eq $pinall) {
		# Really pinned to any
		delete($c->{'pin'});
		}
	push(@rv, $c);
	}
return \@rv;
}

# type_xen_save_vcpus(&server, &vcpus)
# Updates the list of virtual CPUs, and applies them where possible
sub type_xen_save_vcpus
{
my ($server, $vcpus) = @_;
my $conf = &get_xen_config($server, 0);
return $conf if (!ref($conf));

# Work out list of all real CPUs
my $allcpus = &type_xen_list_pin_cpus($server);
my $pinall;
if (ref($allcpus) && @$allcpus > 1) {
	$pinall = "0-".(scalar(@$allcpus)-1);
	}
else {
	$pinall = 0;
	}

&xen_save_directive($conf, "vcpus", scalar(@$vcpus));
if (@$vcpus == 1) {
	# Set single CPU pinning
	&xen_save_directive($conf, "cpu",
		$vcpus[0]->{'pin'} eq '' ? undef : $vcpus[0]->{'pin'});
	&xen_save_directive($conf, "cpus", undef);
	}
else {
	# Set multi-CPU pinning, if any
	&xen_save_directive($conf, "cpu", undef);
	my @pin = grep { $_->{'pin'} ne '' } @$vcpus;
	if (@pin) {
		my @cpus = map { $_->{'pin'} eq '' ? $pinall : $_->{'pin'} }
				  @$vcpus;
		&xen_save_directive($conf, "cpus", join(",", @cpus));
		}
	else {
		# Several CPUs, no pinning
		&xen_save_directive($conf, "cpus", undef);
		}
	}

# Write out the Xen config
my $err = &put_xen_config($server, $conf);
return $err if ($err);

# Apply CPU count and pins now
if ($server->{'status'} ne 'down') {
	my $vhost = &type_xen_parent($server);
	&execute_command_on_server($vhost,
		"xm vcpu-set ".quotemeta($server->{'xen'})." ".scalar(@$vcpus));
	foreach my $cpu (@$vcpus) {
		&execute_command_on_server($vhost,
			"xm vcpu-pin ".quotemeta($server->{'xen'})." ".
			$cpu->{'num'}." ".
			($cpu->{'pin'} eq '' ? $pinall : $cpu->{'pin'}));
		}
	}

return undef;
}

# type_xen_list_pin_cpus(&server)
# Returns a list of CPUs on a host system. Each is a hash ref with 'num' and
# 'desc' fields.
sub type_xen_list_pin_cpus
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
my ($info, $ex, $err) = &execute_command_on_server($vhost,
				"cat /proc/cpuinfo");
return "Failed to get host system CPUs" if ($ex || $err);
my @rv;
my $cpu;
foreach my $l (split(/\r?\n/, $info)) {
	if ($l =~ /^processor\s*:\s*(\d+)/i) {
		# Start of new CPU
		$cpu = { 'num' => $1 };
		push(@rv, $cpu);
		}
	elsif ($l =~ /^model\s+name\s*:\s*(\S.*)/) {
		$cpu->{'desc'} = $1;
		}
	elsif ($l =~ /^physical\s+id\s*:\s*(\d+)/) {
		$cpu->{'physical'} = $1;
		}
	elsif ($l =~ /^core\s+id\s*:\s*(\d+)/) {
		$cpu->{'core'} = $1;
		}
	}
# Add core info to descriptions
foreach my $cpu (@rv) {
	if (defined($cpu->{'physical'}) && defined($cpu->{'core'})) {
		$cpu->{'desc'} .=
			", IC $cpu->{'physical'}, core $cpu->{'core'}";
		}
	}
return \@rv;
}

# type_xen_can_interfaces(&server)
# Xen interface management is via Webmin or via SSH, so it needs to be up.
# The return value of 2 means that interfaces can also be 'real'
sub type_xen_can_interfaces
{
my ($server) = @_;
if (&can_ssh_status($server)) {
	# Can manage via Webmin or SSH
	return 2;
	}
elsif ($server->{'status'} eq 'down' &&
       (&redhat_server_type($server) || &debian_server_type($server))) {
	# Can manage via direct file manipulation
	return 2;
	}
else {
	# Cannot edit
	return 0;
	}
}

# type_xen_can_mac_address(&server, [&iface])
# Returns 1 if the MAC address for an interface can be edited
sub type_xen_can_mac_address
{
my ($server, $iface) = @_;
return $iface && $iface->{'virtual'} eq '';
}

# Regular interface IP allocation works for Xen
sub type_xen_can_allocate_address
{
return 1;
}

# type_xen_can_gateway(&server)
# Returns 1 indicating that Xen instances can have their default gateway 
# read and set.
sub type_xen_can_gateway
{
my ($server) = @_;
return $server->{'status'} eq 'virt' || $server->{'status'} eq 'novirt' ||
       &redhat_server_type($server) || &debian_server_type($server);
}

# type_xen_list_interfaces(&server, [force-from-cfg])
# Returns a list of network interfaces on the system, or an error message
sub type_xen_list_interfaces
{
my ($server, $fromcfg) = @_;
my $rv;
my $conf = &get_xen_config($server, 1);
my $vifs = &xen_find_value("vif", $conf);

if ($fromcfg) {
	# Build fake interfaces list from .cfg file only
	my $i = 0;
	$rv = [ ];
	foreach my $vif (@$vifs) {
		my %info = map { split(/=/, $_) } split(/,/, $vif);
		my @ips = split(/\s+/, $info{'ip'});
		next if (!@ips);
		my $iface = { 'name' => 'eth'.$i,
			      'fullname' => 'eth'.$i,
			      'address' => shift(@ips),
			      'netmask' => '255.255.255.0',
			      'ether' => $info{'mac'} };
		push(@$rv, $iface);
		my $j = 0;
		foreach my $vip (@ips) {
			my $viface = { 'name' => $iface->{'name'},
				       'fullname' => $iface->{'name'}.':'.$j,
				       'virtual' => $j,
				       'address' => $vip,
				       'netmask' => $iface->{'netmask'} };
			push(@$rv, $viface);
			$j++;
			}
		$i++;
		}
	}
else {
	if ($server->{'status'} eq 'down') {
		# Get from network config files
		my $err = &mount_filesystem_if_needed($server);
		return "Failed to mount filesystem : $err" if ($err);
		$rv = &list_boot_server_interfaces($server);
		&umount_filesystem_if_needed($server);
		}
	else {
		# Get active and boot interfaces
		$rv = &list_active_server_interfaces($server);
		$bootrv = &list_boot_server_interfaces($server);
		if (ref($rv) && ref($bootrv)) {
			my %got = map { $_->{'fullname'}, $_ } @$rv;
			push(@$rv, grep { !$got{$_->{'fullname'}} &&
					  &check_ipaddress($_->{'address'}) }
					@$bootrv);
			}
		}

	# Set nodelete flag on first real interface, filter out interfaces
	# with no IP or non-ethernet
	if (ref($rv)) {
		$rv = [ grep { $_->{'fullname'} =~ /^eth/ &&
			       $_->{'up'} &&
			       $_->{'address'} } @$rv ];
		foreach my $iface (@$rv) {
			if ($iface->{'fullname'} eq 'eth0') {
				$iface->{'nodelete'} = 1
				}
			}
		}

	# Set MAC addresses and bridges based on .cfg file
	my $i = 0;
	foreach my $vif (@$vifs) {
		my %info = map { split(/=/, $_) } split(/,/, $vif);
		my ($iface) = grep { $_->{'fullname'} eq "eth".$i } @$rv;
		if ($iface) {
			$iface->{'ether'} = $info{'mac'};
			$iface->{'bridge'} = $info{'bridge'};
			}
		$i++;
		}

	# Set device IDs based on network-list output
	if ($server->{'status'} ne 'down') {
		my $vhost = &type_xen_parent($server);
		my ($out, $ex, $err) = &execute_command_on_server($vhost,
					"xm network-list $server->{'xen'}");
		my $n = 0;
		foreach my $l (split(/\r?\n/, $out)) {
			if ($l =~ /^(\d+)\s+(\d+)\s+([0-9a-f:]+)/i) {
				my ($devidx, $devmac) = ($1, $3);
				my ($iface) = grep { lc($_->{'ether'}) eq
						     lc($devmac) } @$rv;
				if (!$iface) {
					($iface) = grep { $_->{'fullname'} eq
							  'eth'.$n } @$rv;
					}
				if ($iface) {
					$iface->{'devidx'} = $devidx;
					}
				$n++;
				}
			}
		}
	}

return $rv;
}

# type_xen_list_parent_interfaces(&server)
# Just lists non-virtual interfaces
sub type_xen_list_parent_interfaces
{
my ($server) = @_;
my $rv = &type_xen_list_interfaces($server);
return [ grep { $_->{'virtual'} eq '' } @$rv ];
}

# type_xen_create_interface(&server, &iface, [non-virtual], [bridge])
# Add and activate a virtual interface
sub type_xen_create_interface
{
my ($server, $iface, $nonvirt, $bridge) = @_;

# Find a free interface number, if needed
my $vhost = &type_xen_parent($server);
if ($nonvirt) {
	$iface->{'name'} =~ /^eth\d+$/ ||
		return "Only Ethernet intefaces can be created";
	$iface->{'fullname'} = $iface->{'name'};
	}
else{
	my $ifaces = &type_xen_list_interfaces($server);
	return $ifaces if (!ref($ifaces));
	my $n = 0;
	foreach my $oldiface (@$ifaces) {
		if ($oldiface->{'virtual'} >= $n &&
		    $oldiface->{'name'} eq $iface->{'name'}) {
			$n = $oldiface->{'virtual'} + 1;
			}
		}
	$iface->{'virtual'} = $n;
	$iface->{'fullname'} = $iface->{'name'}.":".$n;
	}

# Allocate IP, if needed
my $idx = $iface->{'name'} =~ /^eth(\d+)/ ? $1 : 0;
my @xen_iface = split(/\s+/, $vhost->{'xen_iface'});
$bridge ||= $xen_iface[$idx];
if (!$iface->{'address'}) {
	my $cidr;
	($iface->{'address'}, $cidr) = &allocate_ip_address(
	    $vhost->{'xen_start'}, $vhost->{'xen_end'}, $vhost->{'xen_cidr'},
	    $vhost->{'xen_gws'}, $vhost->{'xen_ifaces'}, $bridge);
	$iface->{'address'} ||
		return &text('create_exenalloc', $vhost->{'host'},
			    &describe_ip_range($vhost->{'xen_start'},
					       $vhost->{'xen_end'},
					       $vhost->{'xen_cidr'},
					       $vhost->{'xen_ifaces'},
					       $bridge));
	$iface->{'netmask'} = &net::prefix_to_mask($cidr);
	}

# Compute broadcast and network
$iface->{'up'} = 1;
$iface->{'broadcast'} = &net::compute_broadcast($iface->{'address'},
						$iface->{'netmask'});
$iface->{'network'} = &net::compute_network($iface->{'address'},
					    $iface->{'netmask'});

# Add a route to the new IP if system is up
if ($server->{'status'} ne 'down' && !$nonvirt) {
	my $instances = &list_xen_instances($vhost);
	if (ref($instances)) {
		my ($instance) = grep { $_->{'instance'} eq $server->{'xen'} }
				      @$instances;
		if ($instance) {
			my $vhostip = &get_server_ip($vhost);
			my $cmd = "ip route add $iface->{'address'} dev ".
				  "vif".$instance->{'id'}.".0 src ".$vhostip;
			&execute_command_on_server($vhost, $cmd);
			}
		}
	}

# Activate now if system is up
my $mac = &generate_mac_address($server, $idx);
if (&can_ssh_status($server)) {
	if ($nonvirt) {
		# Add Xen network device
		my ($out, $ex, $err) = &execute_command_on_server($vhost,
			"xm network-attach $server->{'xen'}".
			" mac=$mac".
			($bridge ? " bridge=$bridge" : "").
			" ip=$iface->{'address'}");
		if ($ex || $err) {
			return "Failed to connect network interface : ".
			       ($err || $out);
			}
		}
	# Actually ifconfig it up
	my $err = &create_active_server_interface($server, $iface);
	return $err if ($err);
	}

# Activate at boot
my $err = &mount_filesystem_if_needed($server);
return "Failed to mount filesystem : $err" if ($err);
$err = &create_boot_server_interface($server, $iface);
&umount_filesystem_if_needed($server);
return $err if ($err);

# Add IP to vif= line
my $conf = &get_xen_config($server, 0);
my $ips = &xen_find_value("vif", $conf);
my $savevif;
if ($nonvirt) {
	# Add a new vif entry
	$ips->[$idx] = "ip=$iface->{'address'}";
	$ips->[$idx] .= ",mac=$mac";
	if ($bridge) {
		$ips->[$idx] .= ",bridge=$bridge";
		}
	$savevif = 1;
	}
else {
	# Add to IP list in existing vif entry
	if ($ips->[$idx]) {
		my @ethips = &split_xen_vif_ips($ips->[$idx]);
		push(@ethips, $iface->{'address'});
		$ips->[$idx] = &join_xen_vif_ips($ips->[$idx], \@ethips);
		$savevif = 1;
		}
	}
if ($savevif) {
	&xen_save_directive($conf, "vif", $ips);
	my $err = &put_xen_config($server, $conf);
	return "Failed to save Xen config : $err" if ($err);
	}

&add_ip_address($server, $iface->{'address'});
return undef;
}

# type_xen_delete_interface(&server, &iface)
# Delete and de-activate a virtual interface
sub type_xen_delete_interface
{
my ($server, $iface) = @_;
my $vhost = &type_xen_parent($server);
my $idx = $iface->{'name'} =~ /^eth(\d+)/ ? $1 : 0;

# Delete boot interface
my $err = &mount_filesystem_if_needed($server);
return "Failed to mount filesystem : $err" if ($err);
$err = &delete_boot_server_interface($server, $iface);
&umount_filesystem_if_needed($server);
return $err if ($err);

# De-activate interface if possible
if (&can_ssh_status($server)) {
	my $err = &delete_active_server_interface($server, $iface);
	return $err if ($err);

	if ($iface->{'virtual'} eq '' && $iface->{'devidx'} ne '') {
		# Detach Xen interface
		my ($out, $ex, $err) = &execute_command_on_server($vhost,
		    "xm network-detach $server->{'xen'} $iface->{'devidx'}");
		if ($ex || $err) {
			return "Failed to detach network interface : ".
			       ($err || $out);
			}
		}
	}

# Remove from vif= line
my $conf = &get_xen_config($server, 0);
my $ips = &xen_find_value("vif", $conf);
my $savevif;
if ($iface->{'virtual'} eq '') {
	# Delete vif entry
	if ($idx == scalar(@$ips)-1) {
		# Take off last one
		pop(@$ips);
		}
	else {
		# Set to empty
		$ips->[$idx] = '';
		}
	$savevif = 1;
	}
else {
	# Remove IP from vif entry
	if ($ips->[$idx]) {
		my @ethips = &split_xen_vif_ips($ips->[$idx]);
		@ethips = grep { $_ ne $iface->{'address'} } @ethips;
		$ips->[$idx] = &join_xen_vif_ips($ips->[$idx], \@ethips);
		$savevif = 1;
		}
	}
if ($savevif) {
	&xen_save_directive($conf, "vif", $ips);
	my $err = &put_xen_config($server, $conf);
	return "Failed to save Xen config : $err" if ($err);
	}

&delete_ip_address($server, $iface->{'address'});
return undef;
}

# type_xen_modify_interface(&server, &iface, &oldiface)
# Changes the active and boot-time IPs for an interface
sub type_xen_modify_interface
{
my ($server, $iface, $oldiface) = @_;

# Modify boot interface
my $err = &mount_filesystem_if_needed($server);
return "Failed to mount filesystem : $err" if ($err);
$err = &modify_boot_server_interface($server, $iface, $oldiface);
&umount_filesystem_if_needed($server);
return $err if ($err);

# Change active interface if possible
if (&can_ssh_status($server)) {
	my $err = &modify_active_server_interface($server, $iface, $oldiface);
	return $err if ($err);
	}

# Update vif= line in .cfg file
my $conf = &get_xen_config($server, 0);
my $ips = &xen_find_value("vif", $conf);
my $idx = $iface->{'name'} =~ /^eth(\d+)/ ? $1 : 0;
if ($ips->[$idx]) {
	my @ethips = &split_xen_vif_ips($ips->[$idx]);
	@ethips = grep { $_ ne $oldiface->{'address'} } @ethips;
	push(@ethips, $iface->{'address'});
	$ips->[$idx] = &join_xen_vif_ips($ips->[$idx], \@ethips);
	if ($iface->{'virtual'} eq '') {
		$ips->[$idx] =~ s/,mac=[0-9a-f:]+//i;
		if ($iface->{'ether'}) {
			$ips->[$idx] .= ",mac=".$iface->{'ether'};
			}
		}
	&xen_save_directive($conf, "vif", $ips);
	}

# Change the address line in the .cfg file, if needed
my $oldaddress = &xen_find_value("address", $conf);
if ($oldaddress eq $oldiface->{'address'}) {
	&xen_save_directive($conf, "address", $iface->{'address'});
	}

# Write out the config
my $err = &put_xen_config($server, $conf);
return "Failed to save Xen config : $err" if ($err);

&delete_ip_address($server, $oldiface->{'address'});
&add_ip_address($server, $iface->{'address'});
return undef;
}

# type_xen_real_interfaces(&server, &vhost)
# Returns a list of interfaces on the Xen host system associated with some
# virtual server. Each is a hash ref with keys name, bytes_in, bytes_out,
# packets_in, packets_out
sub type_xen_real_interfaces
{
my ($server, $vhost) = @_;

# Get the instance, with caching
my $instances = $xen_instances_cache{$vhost->{'id'}};
if (!$instances) {
	$instances = &list_xen_instances($vhost);
	return $instances if (!ref($instances));
	}
my ($instance) = grep { $_->{'instance'} eq $server->{'xen'} } @$instances;
$xen_instances_cache{$vhost->{'id'}} = $instances;
$instance || return "Could not find running Xen instance for $server->{'xen'}";

# Get network stats, with caching
my ($out, $ex, $err);
$out = $proc_net_dev_cache{$vhost->{'id'}};
if (!$out) {
	($out, $ex, $err) = &execute_command_on_server($vhost,
				"cat /proc/net/dev");
	return $out if ($ex);
	return $err if ($err);
	$proc_net_dev_cache{$vhost->{'id'}} = $out;
	}

# Extract stats for this host
my @rv;
foreach my $dev (split(/\r?\n/, $out)) {
	$dev =~ s/^\s+//;
	my @w = split(/[ \t:]+/, $dev);
	if ($w[0] =~ /^\s*vif(\d+)\.\d+$/ && $1 eq $instance->{'id'} ||
	    $server->{'xen_vif'} && $w[0] eq $server->{'xen_vif'}) {
		# Directions are reversed, as we are looking at a virtual
		# interface that talks *to* the host
		my $iface = { 'name' => $w[0],
				 'bytes_in' => $w[9],
				 'packets_in' => $w[10],
				 'bytes_out' => $w[1],
				 'packets_out' => $w[2] };
		push(@rv, $iface);
		}
	}
return \@rv;
}

# type_xen_can_disks(&server)
# Returns 1, as we can manage disks for xen instances. These are files or
# devices mapped to virtual partition in the instance.
sub type_xen_can_disks
{
my ($server) = @_;
return $server->{'status'} ne 'nohost' ? 1 : 0;
}

# type_xen_can_safely_resize_disk(&server, &disk)
# Returns undef if some disk can be safely resized, or an error message if not
sub type_xen_can_safely_resize_disk
{
my ($server, $disk) = @_;
if (!$disk->{'fs'}) {
	return $text{'disk_eresizefs'};
	}
elsif ($disk->{'fs'} !~ /^ext\d+$/ && $disk->{'fs'} ne 'swap') {
	return $text{'disk_eresizefstype'};
	}
else {
	return undef;
	}
}

# type_xen_list_disks(&server, include-mounts)
# Returns a list of hash refs, one per disk, with fields :
# real - Real file on host system, like /xen/foo.dsk
# virt - Virtual device on instance, like /dev/sda1
# device - Set to 1 if this is a real device, like /dev/md1
# size - Size in bytes, if a file
# vg - LVM volume group, if any
# lv - LVM logical volume
# swap - Set to 1 if this is the swap disk
sub type_xen_list_disks
{
my ($server, $mounts) = @_;
my $vhost = &type_xen_parent($server);
my $conf = &get_xen_config($server, 1);
return "Failed to get Xen config file" if (!ref($conf));

# If LVM is in use, get LVs from the host system
my @lvs;
if ($server->{'xen_filesystem_vg'}) {
	if ($main::list_logical_cache{$vhost->{'id'}}) {
		@lvs = @{$main::list_logical_cache{$vhost->{'id'}}};
		}
	else {
		&remote_foreign_require($vhost, "lvm", "lvm-lib.pl");
		@lvs = &remote_foreign_call($vhost, "lvm",
			"list_logical_volumes", $server->{'xen_filesystem_vg'});
		$main::list_logical_cache{$vhost->{'id'}} = \@lvs;
		}
	}

# Parse disks in Xen config
my $disks = &xen_find_value("disk", $conf);
my @rv;
foreach my $d (@$disks) {
	if ($d =~ /^(file|phy|tap:aio):(\S+),(\S+),(\S+)$/) {
		# A regular file
		my $disk = { 'real' => $2,
			     'virt' => "/dev/$3",
			     'virtdev' => $3,
			     'suffix' => $1,
			     'mode' => $4, };
		$disk->{'device'} = $1 eq "phy";
		if (!$disk->{'device'}) {
			my @st = &stat_file_on_server($vhost, "$2");
			$disk->{'size'} = $st[7];
			}
		if ($server->{'xen_filesystem_vg'} &&
		    $disk->{'real'} =~ /^\/dev\/(\S+)\/(\S+)$/ &&
		    $1 eq $server->{'xen_filesystem_vg'}) {
			# Server has root on LVM .. check if this is too
			$disk->{'vg'} = $1;
			$disk->{'lv'} = $2;
			my ($lv) = grep { $_->{'name'} eq $disk->{'lv'} } @lvs;
			if ($lv) {
				$disk->{'size'} = $lv->{'size'} * 1024;
				}
			}
		$disk->{'index'} = scalar(@rv);
		if ($disk->{'real'} eq $server->{'xen_swapfile'}) {
			$disk->{'swap'} = 1;
			}
		push(@rv, $disk);
		}
	}

# If the caller asked for mounts, get them too
if ($mounts) {
	# Get fstab mounts
	my $fstab = &get_system_fstab($server);
	return $fstab if (!ref($fstab));
	foreach my $d (@rv) {
		my ($f) = grep { &same_xen_virt($_->[0], $d->{'virt'}) }
			       @$fstab;
		if ($f) {
			$d->{'mount'} = $f->[1];
			$d->{'fs'} = $f->[2];
			$d->{'virt_onsystem'} = $f->[0];
			}
		}

	# Check if actually mounted
	if (&can_ssh_status($server)) {
		my %mounted;
		my ($out, $ex, $err) =
			&execute_command_on_server($server, "mount");
		foreach my $l (split(/\r?\n/, $out)) {
			my ($dev, $on, $mp) = split(/\s+/, $l);
			$mounted{$dev} = $mp;
			}
		foreach my $d (@rv) {
			$d->{'mounted'} = $mounted{$d->{'virt'}} ||
					  $mounted{$d->{'virt_onsystem'}};
			}
		}
	}

# If caller asked for disk usage, get it too if we can
if ($mounts == 2 && &can_ssh_status($server)) {
	my ($out, $ex, $err) = &execute_command_on_server(
		$server, "df -k -l || df -k");
	if (!$ex && !$err) {
		foreach my $l (split(/\r?\n/, $out)) {
			my ($fs, $size, $used, $free) = split(/\s+/, $l);
			my ($disk) = grep { $_->{'virt'} eq $fs } @rv;
			if ($disk) {
				$disk->{'df_size'} = $size*1024;
				$disk->{'df_used'} = $used*1024;
				$disk->{'df_free'} = $free*1024;
				}
			}
		}
	}

# If the system is up, set disk block IDs
if ($server->{'status'} ne 'down') {
	my $xenid = &get_xen_instance_id($server);
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"xm block-list $xenid");
	if (!$ex && !$err) {
		my $n = 0;
		foreach my $l (split(/\r?\n/, $out)) {
			if ($l =~ /^(\d+)\s/) {
				$rv[$n++]->{'blockid'} = $1;
				}
			}
		}
	}

return \@rv;
}

# same_xen_virt(device, device)
# Compares two device names, treating /dev/sdN and /dev/xvdN the same
sub same_xen_virt
{
my ($d1, $d2) = @_;
$d1 =~ s/\/xvd/\/sd/g;
$d2 =~ s/\/xvd/\/sd/g;
return lc($d1) eq lc($d2);
}

# type_xen_need_mount(&server)
# Returns 1 if a mount call is needed before reading files from this system
sub type_xen_need_mount
{
my ($server) = @_;
return ($server->{'status'} eq 'down' ||
        $server->{'status'} eq 'create') && !$server->{'xen_mount'};
}

# type_xen_mount_filesystem(&server)
# Mounts a down Xen image's root disk image on /mnt/something, and sets
# xen_mount in the server object
sub type_xen_mount_filesystem
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
my $mp = "/mnt/xen-".$server->{'xen'};
my $err = &make_dir_on_server($vhost, $mp, 0700, 1);
return "Failed to create mount point $mp : $err" if ($err);
my $fsfile = $server->{'xen_filesystem'};
if (!$fsfile) {
	return &text('create_exenfsfile');
	}
my ($out, $ex, $err) = &execute_command_on_server($vhost,
	"mount -o loop ".quotemeta($fsfile)." ".
			 quotemeta($mp));
return "Failed to mount $fsfile on $mp : $out" if ($ex);
$server->{'xen_mount'} = $mp;
return undef;
}

# type_xen_umount_filesystem(&server)
# Removes the mount for a system's disk image
sub type_xen_umount_filesystem
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
my ($out, $ex, $err) = &execute_command_on_server($vhost,
	"umount ".quotemeta($server->{'xen_mount'}));
delete($server->{'xen_mount'});
return undef;
}

# type_xen_new_disk(&server, &disks, real?)
# Returns a structure for a new disk on some system
sub type_xen_new_disk
{
my ($server, $disks, $device) = @_;
my $rv = { };

# Find which device files are used
my %used;
foreach my $d (@$disks) {
	$used{$d->{'virt'}}++;
	}
my $fstab = &get_system_fstab($server);
if (ref($fstab)) {
	foreach my $f (@$fstab) {
		if ($f->[0] =~ /^\/dev\//) {
			$used{$f->[0]}++;
			}
		}
	}

# Pick a /dev/???? file that is free
my $n;
if ($disks->[0]->{'virt'} =~ /^(\/dev\/(s|h|xv)d[a-z])\d+$/) {
	# System uses /dev/sdaN files, pick a free N
	$prefix = $1;
	for($n=1; $used{$prefix.$n}; $n++) { }
	$rv->{'virt'} = $prefix.$n;
	if ($disks->[0]->{'virt_onsystem'} =~ /^(\/dev\/(s|h|xv)d[a-z])\d+$/) {
		$rv->{'virt_onsystem'} = $1.$n;
		}
	}
elsif ($disks->[0]->{'virt'} =~ /^(\/dev\/(s|h|xv)d)[a-z]$/) {
	# System uses whole disks! Pick a free partition on the next disk
	$prefix = $1;
	for($n='a'; $used{$prefix.$n}; $n=chr(ord($n)+1)) { }
	$prefix .= $n;
	for($m=1; $used{$prefix.$m}; $m++) { }
	$rv->{'virt'} = $prefix.$m;
	if ($disks->[0]->{'virt_onsystem'} =~ /^(\/dev\/(s|h|xv)d)[a-z]$/) {
		$rv->{'virt_onsystem'} = $1.$n.$m;
		}
	}
else {
	# No idea .. just guess
	$rv->{'virt'} = "/dev/sda3";
	}

if (!$device) {
	my $vhost = &type_xen_parent($server);
	if ($server->{'xen_filesystem_vg'}) {
		# Create an LVM name
		$rv->{'vg'} = $server->{'xen_filesystem_vg'};
		$rv->{'lv'} = $server->{'xen'}."_".(scalar(@$disks)+1)."_img";
		$rv->{'real'} = "/dev/".$rv->{'vg'}."/".$rv->{'lv'};
		$device = 1;
		}
	else {
		# Pick an image file
		my $xen_dir = $server->{'xen_imagedir'} || $vhost->{'xen_dir'};
		$rv->{'real'} = "$xen_dir/$server->{'xen'}.".
				(scalar(@$disks)+1).".img";
		}
	}
$rv->{'device'} = $device;
return $rv;
}

# type_xen_create_disk(&server, &disk)
# Adds a new virtual disk to a Xen system's config.
# If this is a real file that doesn't exist yet, create it and optionally mkfs
# it. Also add to the /etc/fstab file, if requested.
sub type_xen_create_disk
{
my ($server, $disk) = @_;
my $vhost = &type_xen_parent($server);

my @st = &stat_file_on_server($vhost, $disk->{'real'});
if ($disk->{'device'} && !$disk->{'lv'}) {
	# Check if the device exists
	@st || return "Device $disk->{'real'} does not exist ".
		      "on $vhost->{'host'}";
	}
else {
	if ($disk->{'lv'}) {
		# Create the logical volume
		$last_remote_error = undef;
		&remote_foreign_require($vhost, "lvm", "lvm-lib.pl");
		my $lv = { 'name' => $disk->{'lv'},
			      'size' => int($disk->{'size'}/1024),
			      'perm' => 'rw',
			      'alloc' => 'n',
			      'vg' => $disk->{'vg'} };
		my $err = &remote_foreign_call($vhost, "lvm",
						  "create_logical_volume", $lv);
		$err ||= $last_remote_error;
		return "LVM logical volume creation failed : $err" if ($err);
		}
	elsif (!@st) {
		# Create the file if missing
		$disk->{'size'} || return "File $disk->{'real'} does not exist on $vhost->{'host'}, but no size to create it as was given";
		my $cmd = "dd if=/dev/zero of=".quotemeta($disk->{'real'}).
			     " bs=1024 count=".int($disk->{'size'}/1024);
		my ($out, $ex, $err) = &execute_command_on_server(
						$vhost, $cmd);
		$err = $out if ($ex);
		$err && return "Failed to create file $disk->{'real'} : $err";
		}

	# And make a filesystem on it
	if (!@st && $disk->{'fs'}) {
		my $err = &create_device_filesystem(
			$vhost, $disk->{'real'}, $disk->{'fs'});
		$err && return "Failed to create ".uc($disk->{'fs'}).
			       " filesystem : $err";
		}
	}

# Add the disk to the Xen config
my $conf = &get_xen_config($server, 0);
return "Failed to get Xen config file : $conf" if (!ref($conf));
my $disks = &xen_find_value("disk", $conf);
my $shortvirt = $disk->{'virt'};
$shortvirt =~ s/^\/dev\///;
my $dtype = $disk->{'device'} ? "phy" :
	    $vhost->{'xen_filetype'} ? $vhost->{'xen_filetype'} : "file";
push(@$disks, $dtype.":".$disk->{'real'}.",".$shortvirt.",w");
&xen_save_directive($conf, "disk", $disks);
my $err = &put_xen_config($server, $conf);
return "Failed to save Xen config : $err" if ($err);

# Add block device on server now
my $xenid = &get_xen_instance_id($server);
my $attacherr;
if ($server->{'status'} ne 'down' && $xenid) {
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
	    "xm block-attach $xenid $dtype:$disk->{'real'} $disk->{'virt'} w");
	$attacherr = $err ? $err : $ex ? $out : undef;
	}

# Add to total disk size
if ($server->{'xen_disktotal'} ne '' && (!$disk->{'device'} || $disk->{'lv'})) {
	$server->{'xen_disktotal'} += $disk->{'size'};
	}

# Add to /etc/fstab, if a mount point was given
my $fserr;
if ($disk->{'mount'} && $server->{'status'} ne 'alive') {
	&mount_filesystem_if_needed($server);
	my $err = &make_dir_on_server($server, $disk->{'mount'}, 0755, 1);
	return "Failed to create mount point : $err" if ($err);
	my $fstab = &get_system_fstab($server);
	$fserr = "Failed to read fstab file : $fstab" if (!ref($fstab));
	if (!$fserr) {
		push(@$fstab, [ $disk->{'virt_onsystem'} || $disk->{'virt'},
				$disk->{'mount'},
				$disk->{'fs'} || "ext3",
				'defaults', 0, 1 ]);
		my $err = &put_system_fstab($server, $fstab);
		$fserr = "Failed to update fstab file : $err" if ($err);
		}
	&umount_filesystem_if_needed($server);
	}

# Actually mount if possible
if ($disk->{'mount'} && &can_ssh_status($server) && !$attacherr) {
	my $cmd;
	my $virt = $disk->{'virt_onsystem'} || $disk->{'virt'};
	if ($disk->{'fs'} eq 'swap') {
		$cmd = "swapon ".quotemeta($virt);
		}
	else {
		$cmd = "mount ".quotemeta($virt)." ".
			 	quotemeta($disk->{'mount'});
		}
	my ($out, $ex, $err) = &execute_command_on_server($server, $cmd);
	if ($ex || $err) {
		return "Failed to mount $virt on $disk->{'mount'} : ".
		       ($err || $out);
		}
	}

return $attacherr || $fserr;
}

# type_xen_delete_disk(&server, &disk, [detach-only])
# Removes a disk from the Xen configuration, the system's fstab file, and
# deletes the underlying config file. The disk object must come from a call
# to type_xen_list_disks with the mount flag set to 1.
sub type_xen_delete_disk
{
my ($server, $disk, $detach) = @_;
my $vhost = &type_xen_parent($server);

# Actually un-mount if mounted
if (&can_ssh_status($server) && $disk->{'mounted'}) {
	my ($out, $ex, $err) = &execute_command_on_server($server,
		"umount ".($disk->{'virt_onsystem'} || $disk->{'virt'}));
	if ($err || $ex) {
		return "Failed to un-mount $disk->{'virt'} : ".
		       ($err || $out);
		}
	}

# Dis-connect block device
my $xenid = &get_xen_instance_id($server);
if ($server->{'status'} ne 'down' && $xenid && $disk->{'blockid'}) {
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"xm block-detach $xenid $disk->{'blockid'}");
	if ($ex || $err) {
		return "Failed to detach Xen block device $disk->{'blockid'} :".
		       ($err || $out);
		}
	}

# Remove from the Xen config
my $conf = &get_xen_config($server, 0);
return "Failed to get Xen config file : $conf" if (!ref($conf));
my $disks = &xen_find_value("disk", $conf);
my $suffix = $disk->{'suffix'};
@$disks = grep { !/^\Q$suffix\E:\Q$disk->{'real'}\E,/ } @$disks;
&xen_save_directive($conf, "disk", $disks);
my $err = &put_xen_config($server, $conf);
return "Failed to save Xen config : $err" if ($err);

# Remove from total disk size
if ($server->{'xen_disktotal'} ne '' && (!$disk->{'device'} || $disk->{'lv'})) {
	$server->{'xen_disktotal'} -= $disk->{'size'};
	}

# Remove from fstab
if ($server->{'status'} ne 'alive') {
	my $fstab = &get_system_fstab($server);
	return "Failed to read fstab file : $fstab" if (!ref($fstab));
	my @newfstab = grep { !&same_xen_virt($_->[0], $disk->{'virt'}) }
			    @$fstab;
	if (scalar(@newfstab) != scalar(@$fstab)) {
		my $err = &put_system_fstab($server, \@newfstab);
		$fserr = "Failed to update fstab file : $err" if ($err);
		}
	}

if (!$detach) {
	if (!$disk->{'device'} && $disk->{'real'} !~ /^\/dev\//) {
		# Delete the image file
		&execute_command_on_server($vhost,
			"rm -f ".quotemeta($disk->{'real'}));
		}
	elsif ($disk->{'lv'}) {
		# Delete the logical volume
		&remote_foreign_require($vhost, "lvm", "lvm-lib.pl");
		&remote_foreign_call($vhost, "lvm", "delete_logical_volume",
				     { 'device' => $disk->{'real'} });
		}
	}

return undef;
}

# type_xen_modify_disk(&server, &disk, &old-disk)
# Modifies the virtual device and size of a disk. Returns undef if OK, or an
# error message on failure.
# The disk objects *must* have the mount and fs fields set, if mounted.
sub type_xen_modify_disk
{
my ($server, $disk, $olddisk) = @_;
my $vhost = &type_xen_parent($server);
my $moderr;

# Perform some basic validation
if ($disk->{'size'} != $olddisk->{'size'}) {
	$disk->{'fs'} =~ /^ext\d+$/ || $disk->{'fs'} eq 'swap' ||
 	  $disk->{'fs'} eq '' ||
	    return "The size of a disk with the ".uc($disk->{'fs'}).
		   " filesystem cannot be changed";
	}
my $conf = &get_xen_config($server, 0);
return "Failed to get Xen config file : $conf" if (!ref($conf));

# If the system is up, un-mount the disk
my $mounted = 0;
if (&can_ssh_status($server) && $olddisk->{'mounted'}) {
	my ($out, $ex, $err) = &execute_command_on_server($server,
		"umount ".($olddisk->{'virt_onsystem'} || $olddisk->{'virt'}));
	if ($err || $ex) {
		return "Failed to un-mount $olddisk->{'virt'} : ".
		       ($err || $out);
		}
	$mounted = 1;
	}

# If the system is up, dis-connect the disk
my $connected = 0;
my $xenid = &get_xen_instance_id($server);
if ($server->{'status'} ne 'down' && $xenid && $olddisk->{'blockid'}) {
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"xm block-detach $xenid $olddisk->{'blockid'}");
	if ($ex || $err) {
		return "Failed to detach Xen block device ".
		       "$olddisk->{'blockid'} : ".($err || $out);
		}
	$connected = 1;
	}

# Apply a size change, if we can
if ($disk->{'size'} != $olddisk->{'size'}) {
	# Build the commands to run
	my @cmds;
	if ($disk->{'lv'} && $disk->{'size'} > $olddisk->{'size'}) {
		# Expand logical volume (before filesystem)
		push(@cmds, "lvextend -L".int($disk->{'size'}/1024)."k ".
			    quotemeta($disk->{'real'}));
		}
	if ($disk->{'fs'} eq 'swap') {
		# Just re-make
		push(@cmds, "dd if=/dev/zero of=".quotemeta($disk->{'real'}).
			    " bs=1024 count=".int($disk->{'size'}/1024));
		push(@cmds, "mkswap ".quotemeta($disk->{'real'}));
		}
	elsif ($disk->{'fs'} =~ /^ext\d+$/) {
		# Expand the filesystem
		push(@cmds, "e2fsck -f -p ".quotemeta($disk->{'real'}));
		push(@cmds, "resize2fs ".quotemeta($disk->{'real'}).
			    " ".int($disk->{'size'}/(1024*1024))."M");
		}
	elsif ($disk->{'fs'} eq '' && $disk->{'size'} > $olddisk->{'size'}) {
		# Add blanks to the file, unless on LVM
		if (!$disk->{'lv'}) {
			push(@cmds, "dd if=/dev/zero bs=1024".
			    " count=".
			    int(($disk->{'size'}-$olddisk->{'size'})/1024).
			    " >>".quotemeta($disk->{'real'}));
			}
		}
	elsif ($disk->{'fs'} eq '' && $disk->{'size'} < $olddisk->{'size'}) {
		# Truncate the file, unless on LVM
		if (!$disk->{'lv'}) {
			push(@cmds, "dd if=".quotemeta($disk->{'real'}).
			    " of=".quotemeta($disk->{'real'}.".tmp")." bs=1024".
			    " count=".int($disk->{'size'}/1024));
			push(@cmds, "mv ".quotemeta($disk->{'real'}.".tmp").
			    " ".quotemeta($disk->{'real'}));
			}
		}
	else {
		return "No idea now to resize!";
		}
	if ($disk->{'lv'} && $disk->{'size'} < $olddisk->{'size'}) {
		# Shrink logical volume (after resizing filesystem)
		push(@cmds, "lvreduce -f -L".int($disk->{'size'}/1024)."k ".
			    quotemeta($disk->{'real'}));
		}

	# Run them in turn
	foreach my $cmd (@cmds) {
		my ($out, $ex, $err) = &execute_command_on_server(
			$vhost, $cmd);
		if ($err) {
			$moderr = $err;
			last;
			}
		elsif ($ex) {
			$moderr = "$cmd failed : $out";
			last;
			}
		}

	# Update total disk size
	if (!$moderr &&
	    $server->{'xen_disktotal'} ne '' &&
	    (!$disk->{'device'} || $disk->{'lv'})) {
		$server->{'xen_disktotal'} -= $olddisk->{'size'};
		$server->{'xen_disktotal'} += $disk->{'size'};
		}
	}

if ($disk->{'virt'} ne $olddisk->{'virt'} && !$moderr) {
	# Change the virtual device, both in the Xen config file and fstab

	# Fix the Xen config
	my $disks = &xen_find_value("disk", $conf);
	my $suffix = $disk->{'suffix'};
	my $oldshort = $olddisk->{'virt'};
	$oldshort =~ s/^\/dev\///;
	foreach my $d (@$disks) {
		if ($d =~ /^\Q$suffix\E:\Q$disk->{'real'}\E,\Q$oldshort\E,(.*)$/) {
			my $mode = $1;
			my $short = $disk->{'virt'};
			$short =~ s/^\/dev\///;
			$d = "$suffix:$disk->{'real'},$short,$mode";
			}
		}
	&xen_save_directive($conf, "disk", $disks);
	my $err = &put_xen_config($server, $conf);
	if ($err) {
		$moderr = "Failed to save Xen config : $err";
		}

	# Fix virt_onsystem parameter
	if ($disk->{'virt_onsystem'} =~ /^(\/dev\/(s|h|xv)d)[a-z].*$/) {
		my $prefix = $1;
		if ($disk->{'virt'} =~ /([a-z]\d*)$/) {
			$disk->{'virt_onsystem'} = $prefix.$1;
			}
		}

	# Update fstab, if in there
	if ($server->{'status'} ne 'alive' && !$moderr) {
		my $fstab = &get_system_fstab($server);
		if (!ref($fstab)) {
			$moderr = "Failed to read fstab file : $fstab";
			}
		else {
			my ($f) = grep { &same_xen_virt($_->[0],
						$olddisk->{'virt'}) } @$fstab;
			if ($f) {
				$f->[0] = $disk->{'virt_onsystem'} ||
					  $disk->{'virt'};
				my $err = &put_system_fstab($server, $fstab);
				$moderr = "Failed to update fstab file : $err"
					if ($err);
				}
			}
		}
	}

# If it was connected before, re-connect it now
if ($connected) {
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
	    "xm block-attach $xenid $disk->{'suffix'}:$disk->{'real'} ".
	    "$disk->{'virt'} w");
	if ($err || $ex) {
		return "Failed to attach $disk->{'virt'} : ".($err || $out);
		}
	}

# If it was mounted before, mount it now
if ($mounted && $disk->{'mount'}) {
	my ($out, $ex, $err) = &execute_command_on_server($server,
		"mount ".($disk->{'virt_onsystem'} || $disk->{'virt'})." ".
			 $disk->{'mount'});
	if ($err || $ex) {
		return "Failed to mount $disk->{'virt'} : ".($err || $out);
		}
	}

return $moderr;
}

# type_xen_list_disk_filesystems(&server)
# Returns supported filesystem types. Assumes Linux.
sub type_xen_list_disk_filesystems
{
return ('ext3', 'ext2', 'reiserfs', 'xfs', 'jfs', 'swap');
}

# type_xen_can_fsck(&server, &disk)
# Returns 1 if some disk can have it's filesystem checked. Only true when
# the system is down or the disk is not mounted.
sub type_xen_can_fsck
{
my ($server, $disk) = @_;
return ($server->{'status'} eq 'down' || !$disk->{'mounted'}) &&
       $disk->{'fs'} && $disk->{'fs'} ne 'swap';
}

# type_xen_fsck_disk(&server, &disk)
# Run fsck on a disk to validate and repair the filesystem. Returns 1 if 
# all was OK to start with, 2 if fixes were made and 0 on failure, plus the
# output from fsck
sub type_xen_fsck_disk
{
my ($server, $disk) = @_;
my $vhost = &get_parent_server($server);
my ($out, $ex, $err) = &execute_command_on_server($vhost,
	"fsck -y -f -t $disk->{'fs'} ".quotemeta($disk->{'real'}));
if ($ex > 2 || $err) {
	return (0, $out);
	}
elsif ($ex) {
	return (2, $out);
	}
else {
	return (1, $out);
	}
}

# type_xen_compute_disksize(&server)
# Sets a server's xen_disktotal field to the total size of virtual disks,
# and returns it.
sub type_xen_compute_disksize
{
my ($server) = @_;
if ($server->{'status'} ne 'nohost') {
	my $disks = &type_xen_list_disks($server, 0);
	if (ref($disks)) {
		$server->{'xen_disktotal'} = 0;
		foreach my $d (@$disks) {
			$server->{'xen_disktotal'} += $d->{'size'}
				if (!$d->{'device'} || $d->{'lv'});
			}
		}
	}
return $server->{'xen_disktotal'};
}

# type_xen_get_disk_size(&server)
# Returns the cached amount of disk space allocated to this Xen instance
sub type_xen_get_disk_size
{
my ($server) = @_;
if (!$server->{'xen_disktotal'}) {
	&type_xen_compute_disksize($server);
	}
return $server->{'xen_disktotal'};
}

# type_xen_get_disk_space(&server)
# Returns the amount of disk space used on the host by this Xen instance, which
# is the same as the disk allocated
sub type_xen_get_disk_space
{
my ($server) = @_;
return &type_xen_get_disk_size($server);
}

# type_xen_get_memory_size(&server)
# Returns the memory allocated to this Xen instance, from a cache
sub type_xen_get_memory_size
{
my ($server) = @_;
return $server->{'xen_memory'}*1024*1024;
}

# type_xen_get_cpu_size(&server)
# Returns the cached percentage of CPU allocated to this Xen instance
sub type_xen_get_cpu_size
{
my ($server) = @_;
return $server->{'xen_credit_cpu'} || 0;
}

# type_xen_get_network_rate_size(&server)
# Returns the bits/second of network speed for this Xen instance, from a cache
sub type_xen_get_network_rate_size
{
my ($server) = @_;
return $server->{'xen_network_rate'};
}

# type_xen_need_disk_reboot(&server, &disk)
# A reboot is never needed, now that we can use xm block-attach
sub type_xen_need_disk_reboot
{
my ($server, $disk) = @_;
return 0;
}

# type_xen_get_console_output(&server)
# Returns a status code and text displayed during the Xen boot (if status is 1),
# or an error message (if 0)
sub type_xen_get_console_output
{
my ($server) = @_;
if (!$server->{'xen_console'}) {
	return (0, "No console messages were captured at boot time");
	}
my $vhost = &type_xen_parent($server);
my $out = &get_file_data_from_server($vhost, $server->{'xen_console'});
$out =~ s/SERVER-MANAGER-COMPLETE\r?\n//;
return $out ? (1, $out) : (0, "No console messages were found");
}

# type_xen_supports_console_login(&server)
# Returns 1 if connecting to the console on some system is supported, 0 if not,
# or -1 if possible but not currently setup (which can be done with
# type_xen_create_console)
sub type_xen_supports_console_login
{
my ($server) = @_;
if ($server->{'status'} eq 'nohost' || $server->{'status'} eq 'down' ||
    $server->{'status'} eq 'paused') {
	# Not even running, so cannot connect
	return 0;
	}
else {
	# Check if special user exists
	return &has_login_user($server) ? 1 : -1;
	}
}

# type_xen_console_command(&server)
# Returns the command to be run on the Xen host system to attach to an
# instance's console.
sub type_xen_console_command
{
my ($server) = @_;
my $path = &has_command_on_server($server, "xm");
$path ||= "xm";
return "$path console ".quotemeta($server->{'xen'});
}

# type_xen_can_pause(&server)
# Returns 1 if a system can be paused, 2 if it can be resumed, 0 if neither
sub type_xen_can_pause
{
my ($server) = @_;
if ($server->{'status'} eq 'nohost' || $server->{'status'} eq 'down') {
	# Down, so neither can be done
	return 0;
	}
elsif ($server->{'status'} eq 'paused') {
	# Currently paused, can resume
	return 2;
	}
else {
	# Now running, can cause
	return 1;
	}
}

# type_xen_pause_system(&server)
# Pauses a Xen instance. Returns undef on success or an error message
# on failure.
sub type_xen_pause_system
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"xm pause $server->{'xen'}");
return $err ? $err : $ex ? $out : undef;
}

# type_xen_unpause_system(&server)
# Resumes a Xen instance. Returns undef on success or an error message
# on failure.
sub type_xen_unpause_system
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"xm unpause $server->{'xen'}");
return $err ? $err : $ex ? $out : undef;
}

# type_xen_can_move(&server, [&dest])
# Check if moving a Xen instance is allowed. Only possible if target is up.
# Returns 2 indicating than manual moves are supported too.
sub type_xen_can_move
{
my ($server, $dest) = @_;
if ($server->{'status'} eq 'nohost') {
	return 0;	# Parent is down
	}
elsif (!$dest) {
	return 2;	# Assume OK
	}
elsif ($dest->{'status'} eq 'virt' ||
       $dest->{'status'} eq 'nowebmin' ||
       $dest->{'status'} eq 'novirt') {
	return 2;	# Dest is OK
	}
return 0;
}

# type_xen_move_system(&server, &dest, [shutdown], [allocate-ip])
# Actually move a Xen instance to a new host, printing messages as it goes.
sub type_xen_move_system
{
my ($server, $dest, $shutdown, $allocate) = @_;
my $vhost = &get_parent_server($server);
my (@delfiles, @dellvs);

# Work out old and new dirs. If a custom dir was given for this Xen instance,
# assume that the same should be used on the dest.
my $old_xen_dir = $server->{'xen_imagedir'} || $vhost->{'xen_dir'};
my $new_xen_dir = $server->{'xen_imagedir'} || $dest->{'xen_dir'};

# Check if a shutdown is needed
my $shutwhy;
if (!$shutdown) {
	if ($server->{'xen_filesystem_vg'}) {
		if ($server->{'xen_filesystem_vg'} ne $dest->{'xen_vg'}) {
			$shutdown = 1;
			$shutwhy = $text{'smove_diffvg'};
			}
		}
	elsif ($old_xen_dir ne $new_xen_dir) {
		$shutdown = 1;
		$shutwhy = $text{'smove_diffdir'};
		}
	}

# Check if live migration is possible
my $live = 1;
my $disks = &type_xen_list_disks($server, 0);
my $livehost = $dest->{'xen_livehost'} || &get_server_ip($dest);
my $liveport = $dest->{'xen_liveport'} || $xen_migration_port;
&$first_print($text{'smove_livecheck'});
if ($live && $shutdown) {
	&$second_print($text{'smove_eliveshut'});
	$live = 0;
	}
if ($live && $server->{'status'} eq 'down') {
	&$second_print($text{'smove_elivedown'});
	$live = 0;
	}
if ($live && &has_command_on_server($vhost, "nc")) {
	# Check for live migration port
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"nc -w 4 $livehost $liveport </dev/null", 5);
	if ($ex || $err) {
		&$second_print(&text('smove_eliveport', $livehost, $liveport));
		$live = 0;
		}
	}
if ($live) {
	# Check xend version
	my $vhostver = &get_xen_software_version($vhost);
	my $destver = &get_xen_software_version($dest);
	if ($vhostver && $destver && $vhostver ne $destver) {
		&$second_print(&text('smove_elivever', $vhostver, $destver));
		$live = 0;
		}
	}
if ($live) {
	# Check if all disks are shared
	foreach my $d (@$disks) {
		my @sst = &stat_file_on_server($vhost, $d->{'real'});
		my @dst = &stat_file_on_server($dest, $d->{'real'});
		if (!@dst || $dst[7] != $sst[7]) {
			&$second_print(&text('smove_elivedisk', $d->{'real'}));
			$live = 0;
			last;
			}
		}
	}
if ($live) {
	&$second_print($text{'smove_liveok'});
	}

# Check if need to allocate new IP
my $ip = &get_server_ip($server);
my @xen_iface = split(/\s+/, $dest->{'xen_iface'});
my ($newip, $newcidr, $newgw);
my $change_ip_later;
if ($allocate && $dest->{'xen_start'} &&
    !&range_check_ip_address($ip, $dest->{'xen_start'}, $dest->{'xen_end'},
				  $dest->{'xen_cidr'}, $dest->{'xen_gws'},
				  $dest->{'xen_ifaces'}, $xen_iface[0])) {
	# Allocate the primary IP
	&$first_print($text{'smove_allocating'});
	($newip, $newcidr, $newgw) = &allocate_ip_address(
		$dest->{'xen_start'}, $dest->{'xen_end'},
	        $dest->{'xen_cidr'}, $dest->{'xen_gws'},
		$dest->{'xen_ifaces'}, $xen_iface[0]);
	$newip || return &text('create_exenalloc', $dest->{'host'},
			    &describe_ip_range($dest->{'xen_start'},
					       $dest->{'xen_end'}));
	&$second_print(&text('smove_allocated', $newip));

	if ($live || ($server->{'status'} ne 'down' && !$shutdown)) {
		# Apply the IP change now, before the move
		my $err = &xen_move_change_ip($server, $vhost, $dest,
					      $ip, $newip, $newcidr, $newgw);
		return $err if ($err);
		}
	else {
		$change_ip_later = 1;
		}
	}

my ($dumpsrc, $dumpdest, $was_up);
if ($server->{'status'} ne 'down' && $shutdown) {
	# Shut down before moving
	&$first_print($shutwhy ? &text('smove_shutting2', $shutwhy)
			       : $text{'smove_shutting'});
	$err = &shutdown_server($server);
	if ($err) {
		return &text('smove_shutfailed', $err);
		}
	$server->{'status'} = 'down';	# Assume it worked
	$was_up = 1;
	&$second_print($text{'create_done'});
	}
elsif ($live) {
	# Live migrate system
	&$first_print($text{'smove_living'});
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"xm migrate --live $server->{'xen'} $livehost".
		($liveport == $xen_migration_port ? "" : ":".$liveport));
	if ($ex || $err) {
		return &text('smove_elived', $err || $out);
		}
	&$second_print($text{'create_done'});
	}
elsif ($server->{'status'} ne 'down') {
	# Save a state file, if running
	&$first_print($text{'smove_dumping'});
	$dumpsrc = &tempname_on_server($vhost, $server->{'host'}.".state");
	if (!$dumpsrc) {
		return $text{'smove_etemp'};
		}
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"xm save $server->{'xen'} $dumpsrc");
	if ($ex || $err) {
		return &text('smove_edump', $err || $out);
		}
	&$second_print($text{'create_done'});

	# Transfer state file
        &$first_print($text{'smove_dumpsend'});
        $dumpdest = &tempname_on_server($dest, $server->{'host'}.".state");
	if (!$dumpdest) {
		return $text{'smove_etemp'};
		}
        my $err = &transfer_file_between_servers(
                        $vhost, $dest, $dumpsrc, $dumpdest, 0, 1);
        &delete_file_on_server($vhost, $dumpsrc);
	return $err if ($err);
        &$second_print($text{'create_done'});
	}

# Copy across all disk images, directly if possible. Don't do this if it is a
# live migration, in which case they will already be in place.
my $count = 0;
my $sz = 0;
foreach my $d (@$disks) {
	$d->{'newreal'} = $d->{'real'};
	}
if (!$live) {
	&$first_print($text{'smove_disks'});
	&$indent_print();
	foreach my $d (@$disks) {
		&$first_print(&text('smove_onedisk', $d->{'real'}));
		if ($d->{'device'} && !$d->{'lv'}) {
			# A real device, which shouldn't be touched
			&$second_print(&text('smove_devdisk'));
			next;
			}
		my @sst = &stat_file_on_server($vhost, $d->{'real'});
		my @dst = &stat_file_on_server($dest, $d->{'real'});
		if (@dst && $dst[7] == $sst[7]) {
			# Already there! Perhaps from shared LVM
			&$second_print($text{'smove_alrdisk'});
			next;
			}

		my $newreal;
		if ($d->{'lv'}) {
			# LVM logical volume .. transfer to LV of the same name
			$dest->{'xen_vg'} ||
				return $text{'smove_errvg'};
			$newreal = "/dev/".$dest->{'xen_vg'}."/".$d->{'lv'};

			# Create LV on target first
			$last_remote_error = undef;
			my $lv = { 'name' => $d->{'lv'},
				      'size' => int($d->{'size'}/1024),
				      'perm' => 'rw',
				      'alloc' => 'n',
				      'vg' => $dest->{'xen_vg'} };
			&remote_finished();
			&remote_foreign_require($dest, "lvm", "lvm-lib.pl");
			my $err = &remote_foreign_call($dest, "lvm",
						  "create_logical_volume", $lv);
			$err ||= $last_remote_error;
			$err && return &text('smove_errlv', $err);

			# Transfer over the data, using dd. The size for
			# checksum purposes is rounded down to 1MB to account
			# for differences in LVM block sizes
			my $err;
			if ($d->{'swap'} && $server->{'status'} eq 'down') {
				# Just re-create swap file
				my ($out, $ex, $swerr) =
					&execute_command_on_server($dest,
						"mkswap ".quotemeta($newreal));
				$err = $swerr ? $swerr : $ex ? $out : undef;
				}
			else {
				# Actually copy disk
				$err = &transfer_file_between_servers(
				    $vhost, $dest,
				    $d->{'real'}, $newreal, 1, 1,
				    int($d->{'size'}/(1024*1024))*1024*1024);
				}
			if ($err) {
				return &text('smove_errdisk', $err);
				}
			push(@dellvs, $d->{'real'});
			}
		else {
			# Work out a new filename, and copy over the file
			$newreal = $d->{'real'};
			$newreal =~ s/^.*[\\\/]//;
			$newreal = "$new_xen_dir/$newreal";
			my $err = &transfer_file_between_servers($vhost, $dest,
				$d->{'real'}, $newreal, 0, 1);
			if ($err) {
				return &text('smove_errdisk', $err);
				}
			push(@delfiles, $d->{'real'});
			}

		$d->{'newreal'} = $newreal;
		&$second_print(&text('smove_donedisk',
				     &nice_size($d->{'size'})));
		$count++;
		$sz += $d->{'size'};
		}
	&$outdent_print();
	if ($count) {
		&$second_print(&text('smove_disksdone', $count,
				     &nice_size($sz)));
		}
	else {
		&$second_print(&text('smove_disksnone'));
		}
	}

# Read the source Xen config file
&$first_print(&text('smove_xenconf', $server->{'xen_conf'}));
my $conf = &get_xen_config($server, 0);
my @sst = &stat_file_on_server($vhost, $server->{'xen_conf'});
my @dst = &stat_file_on_server($dest, $server->{'xen_conf'});

# Fix the kernel and path in the config file, if needed
if (!$server->{'xen_pygrub'}) {
	my ($kok, $kfile, $ifile) = &setup_xen_host_kernel($dest);
	$kok || return &text('smove_ekernel', $kfile);
	&xen_save_directive($conf, "kernel", $kfile);
	&xen_save_directive($conf, "ramdisk", $ifile);
	}

# Fix disk paths in the config file, unless this was a live migration in
# which they don't actually change
if (!$live) {
	my @disklist;
	foreach my $d (@$disks) {
		push(@disklist, $d->{'suffix'}.":".$d->{'newreal'}.",".
				$d->{'virtdev'}.",".$d->{'mode'});
		if ($server->{'xen_filesystem'} eq $d->{'real'}) {
			$server->{'xen_filesystem'} = $d->{'newreal'};
			}
		if ($server->{'xen_swap'} eq $d->{'real'}) {
			$server->{'xen_swap'} = $d->{'newreal'};
			}
		}
	&xen_save_directive($conf, "disk", \@disklist);
	}

# Fix Xen VNC IP if set
my $vnclisten = &xen_find_value("vnclisten", $conf);
if ($vnclisten && $vnclisten ne "0.0.0.0") {
	my $ip = &get_server_ip($vhost);
	&xen_save_directive($conf, "vnclisten",
		$ip || &to_ipaddress($vhost->{'host'}));
	}

# Fix location of the config file
if ($server->{'xen_conf'} =~ /^\Q$old_xen_dir\/\E(.*)$/) {
	$server->{'xen_conf'} = "$new_xen_dir/$1";
	}
$server->{'xen_on'} = $dest->{'id'};
my $err = &put_xen_config($server, $conf);
return $err if ($err);
&$second_print(&text('smove_xenconfdone', $server->{'xen_conf'}));

# Don't delete .conf file if shared
if (!@dst || $sst[7] != $dst[7]) {
	push(@delfiles, $server->{'xen_conf'});
	}

# Delete the original disk images and config file
if (@delfiles) {
	&$first_print(&text('smove_delfiles', scalar(@delfiles)));
	push(@delfiles, $server->{'xen_console'}) if ($server->{'xen_console'});
	foreach my $f (@delfiles) {
		my $err = &delete_file_on_server($vhost, $f);
		if ($err) {
			&$second_print(&text('smove_delerr', $f, $err));
			}
		}
	foreach my $f (@dellvs) {
		&remote_foreign_call($vhost, "lvm", "delete_logical_volume",
				     { 'device' => $f });
		}
	&$second_print($text{'create_done'});
	}

# Remove old Xen instance if somehow still running
if ($server->{'status'} ne 'down' && !$live) {
	&execute_command_on_server($vhost, "xm destroy $server->{'xen'}");
	}

# Fix up VNC port in config file, if any
if ($server->{'xen_vnc_display'} || $server->{'xen_vnc_unused'}) {
	&$first_print($text{'create_xenvnc'});
	my $err = &setup_xen_vnc_console($server);
	&$second_print($err ? &text('create_xenvncfailed', $err)
			    : &text('create_xenvncdone'));
	}

# Fix LVM volume group
if ($server->{'xen_filesystem_vg'}) {
	$server->{'xen_filesystem_vg'} = $dest->{'xen_vg'};
	}
if ($server->{'xen_swapfile_vg'}) {
	$server->{'xen_swapfile_vg'} = $dest->{'xen_vg'};
	}

# Change IP now if we didn't do it before the move
if ($change_ip_later) {
	my $err = &xen_move_change_ip($server, $vhost, $dest,
				      $ip, $newip, $newcidr, $newgw);
	return $err if ($err);
	}

# Restore dump on the new host
if ($dumpsrc) {
        &$first_print($text{'smove_undumping'});
        my ($out, $ex, $err) = &execute_command_on_server($dest,
                "xm restore $dumpdest");
	&delete_file_on_server($dest, $dumpdest);
        if ($ex || $err) {
                return &text('smove_eundump', $err || $out);
                }
        &$second_print($text{'create_done'});
        }
elsif ($was_up) {
	# Start on the new host, if it was up before
	&$first_print($text{'smove_starting'});
	my ($ok, $err) = &startup_server($server);
	if (!$ok) {
		return &text('smove_startfailed', $err);
		}
	&$second_print($text{'create_done'});
	}

return undef;
}

# xen_move_change_ip(&server, &vhost, &dest, oldip, newip, newcidr, newgw)
# Change the IP on a xen instance being moved
sub xen_move_change_ip
{
my ($server, $vhost, $dest, $ip, $newip, $newcidr, $newgw) = @_;

# Change the interface IP
&$first_print(&text('smove_ipchanging', $ip, $newip));
my $ifaces = &type_xen_list_interfaces($server);
return $ifaces if (!ref($ifaces));
my ($iface) = grep { $_->{'address'} eq $ip } @$ifaces;
$iface || return $text{'smove_eifaceip'};
my $oldiface = { %$iface };
$iface->{'address'} = $newip;
$iface->{'netmask'} = &net::prefix_to_mask($newcidr);
&modify_primary_ip_address($server, $ip, $newip);
my $err = &type_xen_modify_interface($server, $iface, $oldiface);
return $err if ($err);
$server->{'ip'} = $newip;
&delete_ip_address($server, $ip);
&add_ip_address($server, $newip);
&$second_print($text{'create_done'});

# Change the default gateway if needed
$newgw = "" if ($newgw eq "*");
$newgw ||= &get_server_default_gateway($dest);
my $defgw = &get_server_default_gateway($vhost);
my @gws = split(/\s+/, $server->{'xen_gw'});
my $gw = $gws[0] eq "*" ? $defgw : $gws[0];
if ($newgw ne $gw) {
	&$first_print(&text('smove_gwchanging',
			    $gw || $text{'smove_gwnone'},
			    $newgw || $text{'smove_gwnone'}));
	my $err = &modify_default_gateway($server, $newgw);
	return $err if ($err);
	if ($server->{'xen_gw'}) {
		$gws[0] = $newgw;
		$server->{'xen_gw'} = join(" ", @gws);
		}
	&$second_print($text{'create_done'});
	}

return undef;
}

# type_xen_manual_move_system(&server, &dest)
# Update the configs to reflect a system move already done
sub type_xen_manual_move_system
{
my ($server, $dest) = @_;
my $vhost = &get_parent_server($server);

# Work out old and new dirs. If a custom dir was given for this Xen instance,
# assume that the same should be used on the dest.
my $old_xen_dir = $server->{'xen_imagedir'} || $vhost->{'xen_dir'};
my $new_xen_dir = $server->{'xen_imagedir'} || $dest->{'xen_dir'};

# Check for config
&$first_print($text{'smove_checkconf'});
if ($server->{'xen_conf'} =~ /^\Q$old_xen_dir\/\E(.*)$/) {
	$server->{'xen_conf'} = "$new_xen_dir/$1";
	}
my @st = &stat_file_on_server($dest, $server->{'xen_conf'});
if (!@st) {
	return &text('smove_echeckconf', $server->{'xen_conf'});
	}
&$second_print(&text('smove_checkconfok', $server->{'xen_conf'}));
$server->{'xen_on'} = $dest->{'id'};

# Check if disks exist on target
&$first_print($text{'smove_checkdisks'});
my $disks = &type_xen_list_disks($server, 0);
return $disks if (!ref($disks));
foreach my $d (@$disks) {
	my @dst = &stat_file_on_server($dest, $d->{'real'});
	if (!@dst) {
		return &text('smove_echeckdisk', $d->{'real'});
		}
	}
&$second_print($text{'smove_checkdisksok'});

# Fix up VNC port in config file, if any
if ($server->{'xen_vnc_display'} || $server->{'xen_vnc_unused'}) {
	&$first_print($text{'create_xenvnc'});
	my $err = &setup_xen_vnc_console($server);
	&$second_print($err ? &text('create_xenvncfailed', $err)
			    : &text('create_xenvncdone'));
	}

return undef;
}

# type_xen_find_servers(&vhost, &template-server, running-only, [basedir])
# Searches a Xen host system for running or configured instances, and adds
# them to Cloudmin's control. Returns an array of server objects found. May
# print stuff.
sub type_xen_find_servers
{
my ($vhost, $tmplserver, $running, $dir) = @_;
my @rv;

# Find all the servers we currently know about, for dupe checking
my (%alreadyip, %alreadyxen);
foreach my $s (&list_managed_servers()) {
	foreach my $ip (&get_server_ip($s)) {
		$alreadyip{$ip} = $s;
		}
	}

# Find .cfg files in common locations
my @dirs = $dir ? ( $dir )
		   : &unique("/etc/xen", "/xen", $vhost->{'xen_dir'});
my @files;
&$first_print(&text('find_xendirs', "<tt>".join(", ", @dirs)."</tt>"));
foreach my $dir (@dirs) {
	my ($out, $ex, $err) = &execute_command_on_server(
		$vhost, "ls ".quotemeta($dir)."/*.cfg");
	if (!$ex && !$err) {
		push(@files, split(/\r?\n/, $out));
		}
	}
if (@files) {
	&$second_print(&text('find_xendirsdone', scalar(@files)));
	}
else {
	&$second_print(&text('find_xendirsnone', "<tt>*.cfg</tt>"));
	return ( );
	}

# Find running Xen instances
&$first_print($text{'find_xeninstances'});
my $instances = &list_xen_instances($vhost);
if (ref($instances)) {
	&$second_print(&text('find_xeninstancesdone', scalar(@$instances)));
	}
else {
	&$second_print(&text('find_xeninstanceserr', $instances));
	return ( );
	}
my %runningmap = map { $_->{'instance'}, $_ } @$instances;

# Process each of the Xen config files
foreach my $file (@files) {
	&$first_print(&text('find_xenconfig', "<tt>$file</tt>"));

	# Fetch and parse the config file
	my $localfile = &transname();
	my $err = &get_file_from_server($vhost, $file, $localfile);
	if ($err) {
		&$second_print(&text('find_xenconfigerr', $err));
		next;
		}
	my $conf = &parse_xen_config($localfile);
	if (!$conf) {
		&$second_print($text{'find_xenconfigparse'});
		next;
		}

	# Check for critical variables
	my $name = &xen_find_value("name", $conf);
	if (!$name) {
		&$second_print(&text('find_xenconfigmiss', 'name'));
		next;
		}
	my $kernel = &xen_find_value("kernel", $conf);
	my $bootloader = &xen_find_value("bootloader", $conf);
	if (!$bootloader && !$kernel) {
		&$second_print(&text('find_xenconfigloader'));
		next;
		}
	if ($bootloader =~ /pygrub/) {
		$server->{'xen_pygrub'} = 1;
		$server->{'xen_bootloader'} = $bootloader;
		}
	elsif ($kernel =~ /pv-grub/) {
		$server->{'xen_pygrub'} = 1;
		$server->{'xen_bootloader'} = $kernel;
		}
	my $address = &xen_find_value("address", $conf);
	if (!$address) {
		&$second_print(&text('find_xenconfigmiss', 'address'));
		next;
		}

	# Skip if we already host it
	my ($already) = &get_managed_server_by("xen", $name);
	if ($already) {
		&$second_print(&text('find_xenalready', $already->{'host'}));
		next;
		}
	$already = $alreadyip{$address};
	if ($already) {
		&$second_print(&text('find_xenalready2', $already->{'host'},
				     $address));
		next;
		}

	# Skip if not running, if requested
	if ($running && !$runningmap{$name}) {
		&$second_print(&text('find_notrunning', $name));
		next;
		}

	# Create the server object
	my $revhost = gethostbyaddr(inet_aton($address), AF_INET);
	my $server = { %$tmplserver,
			  'id' => &server_id(),
			  'desc' => "Automatically added Xen instance $name",
			  'type' => 'unknown',
			  'manager' => 'xen',
			  'xen' => $name,
			  'xen_on' => $vhost->{'id'},
			  'xen_conf' => $file,
			  'host' => $revhost || $address,
			};
	&complete_managed_server($server);
	&set_xen_filesystem($server, $conf);

	# Get status one more time, and save the server object
	&save_managed_server($server);
	&check_server_status($server);
	if ($server->{'type'} eq 'unknown' || !$server->{'type'}) {
		my $guess = &guess_os_type($server);
		if ($guess) {
			$server->{'type'} = $guess;
			&check_server_status($server, 0, 1);
			}
		}
	&save_managed_server($server);
	&$second_print(&text('find_xendone', $server->{'host'},
			     &describe_status($server, 1, 1)));
	push(@rv, $server);
	}

return @rv;
}

# Returns 2 if some Xen system can have it's network speed set, but needs
# a reboot
sub type_xen_can_network_rate
{
return 2;
}

# type_xen_get_network_rate(&server)
# Returns the maximum network speed (in bits per second) for a Xen instance.
# Returns 0 if not set.
sub type_xen_get_network_rate
{
my ($server) = @_;
my $conf = &get_xen_config($server, 1);
return undef if (!ref($conf));
my $vifs = &xen_find_value("vif", $conf);
if ($vifs->[0] =~ /rate=(\d+)([KMGT])([Bb])\/s/) {
	my $r = $1;
	$r *= ($2 eq "K" ? 1024 :
	       $2 eq "M" ? 1024*1024 :
	       $2 eq "G" ? 1024*1024*1024 :
	       $2 eq "T" ? 1024*1024*1024*1024 : 1);
	$r *= 8 if ($3 eq "B");
	return $r;
	}
return 0;
}

# type_xen_save_network_rate(&server, rate)
# Updates the network rate for a Xen instance. Returns undef on success or an
# error message on failure.
sub type_xen_save_network_rate
{
my ($server, $rate) = @_;
my $ratebits = $rate;
my $vhost = &type_xen_parent($server);
my $conf = &get_xen_config($server, 0);
my $vifs = &xen_find_value("vif", $conf);
if ($rate) {
	my $units;
	if ($rate % (1024*1024) == 0) {
		$rate /= 1024*1024;
		$units = "M";
		}
	else {
		$rate = int($rate/1024);
		$units = "K";
		}
	if ($vifs->[0] !~ s/rate=(\d+)([KMGT])([Bb])/rate=${rate}${units}b/) {
		$vifs->[0] .= " " if ($vifs->[0]);
		$vifs->[0] .= "rate=${rate}${units}b/s";
		}
	}
else {
	$vifs->[0] =~ s/\s*rate=(\S+)//;
	}
&xen_save_directive($conf, "vif", $vifs);
my $err = &put_xen_config($server, $conf);
return $err if ($err);
$server->{'xen_network_rate'} = $ratebits;
return undef;
}

# can_host_xen(&server)
# Returns 1 if some server is of a type that we can do Xen hosting on
sub can_host_xen
{
my ($server) = @_;
return $server->{'manager'} eq 'this' || $server->{'manager'} eq 'real';
}

# check_xen_host_software(&server)
# Checks if some system has the Xen software installed, and the kernel
# patch needed to use it.
sub check_xen_host_software
{
my ($server) = @_;

# Check if linux
my ($out, $ex, $err) = &execute_command_on_server($server, "uname");
return $text{'xen_elinux'} if ($out !~ /linux/i);

# Check commands
foreach my $cmd ("xm") {
	my $full = &has_command_on_server($server, $cmd);
	return &text('xen_ewhich', "<tt>$cmd</tt>") if (!$full);
	}

# Check kernel
my ($out, $ex, $err) = &execute_command_on_server($server,
				"test -d /proc/xen");
return $text{'xen_ekernel'} if ($ex);

# Check for virbr0 interface, which breaks networking!
my ($out, $ex, $err) = &execute_command_on_server($server, "ifconfig -a");
if ($out =~ /(virbr\d+)/) {
	return &text('xen_evibr', "<tt>$1</tt>",
		"<tt>/etc/libvirt/qemu/networks/default.xml</tt>");
	}

return undef;
}

# check_xen_host_lvm(&server, vg-name)
# Checks if some Xen host system has LVM working, and the given volume group
sub check_xen_host_lvm
{
my ($server, $vg) = @_;

# First check for Webmin
$server->{'status'} eq 'virt' || $server->{'status'} eq 'novirt' ||
	return $text{'xen_elvmwebmin'};

# Must have workable LVM module
$last_remote_error = undef;
if (!&remote_foreign_check($server, "lvm")) {
	return $text{'xen_elvmmod'};
	}
&remote_foreign_require($server, "lvm", "lvm-lib.pl");
if ($last_remote_error) {
	return $text{'xen_elvmmod2'};
	}
if (!&remote_foreign_call($server, "lvm", "foreign_installed", "lvm")) {
	return $text{'xen_elvmmod3'};
	}

# Verify that the volume group exists
my @vgs = map { $_->{'name'} } 
		 &remote_foreign_call($server, "lvm", "list_volume_groups");
@vgs || return $text{'xen_elvmvgs'};
&indexof($vg, @vgs) >= 0 || return &text('xen_elvmvg', join(' ', @vgs));

return undef;
}

# list_xen_instances(&server)
# Returns a array ref of all Xen instances on some real server, or
# an error message
sub list_xen_instances
{
my ($server) = @_;
my ($out, $ex, $err) = &execute_command_on_server($server, "xm list");
return $out if ($ex);
return $err if ($err);
my @rv;
foreach my $l (split(/\r?\n/, $out)) {
	my ($name, $id, $mem, $cpus, $state, $time) = split(/\s+/, $l);
	if ($id ne "ID" && $id != 0) {
		my ($s) = grep { $_ ne "-" } split(//, $state);
		push(@rv, { 'id' => $id,
			    'instance' => $name,
			    'mem' => $mem*1024*1024,
			    'cpus' => $cpus,
			    'state' => $s,
			    'time' => $time });
		}
	}
return \@rv;
}

# parse_xen_config(file)
# Parses a Xen config file (actually a list of python variables!) into an
# array ref.
sub parse_xen_config
{
my ($file) = @_;
my @rv;
my $lnum = 0;
open(CONF, $file);
while(<CONF>) {
	s/\r|\n//g;
	s/^#.*//g;	# strip whole-line comments
	s/\s#.*//g;	# strip end-line comments
	if (/^\s*(\S+)\s*=\s*'([^']*)'/ ||
	    /^\s*(\S+)\s*=\s*"([^"]*)"/ ||
	    /^\s*(\S+)\s*=\s*(\d+)/) {
		# Single-value variable
		push(@rv, { 'name' => $1,
			    'value' => $2,
			    'type' => 0,
			    'line' => $lnum,
			    'eline' => $lnum,
			    'file' => $file });
		}
	elsif (/^\s*(\S+)\s*=\s*\[(.*)\]/) {
		# List of values
		push(@rv, { 'name' => $1,
			    'rawvalue' => $2,
			    'values' => [ &split_xen_array($2) ],
			    'type' => 1,
			    'line' => $lnum,
			    'eline' => $lnum,
			    'file' => $file });
		}
	elsif (/^\s*(\S+)\s*=\s*\[([^\]]*)$/) {
		# List, but split over several lines!
		my $dir = { 'name' => $1,
			       'rawvalue' => $2,
			       'values' => [ &split_xen_array($2) ],
			       'type' => 1,
			       'line' => $lnum,
			       'eline' => $lnum,
			       'file' => $file };
		push(@rv, $dir);
		$lnum++;
		# Read list until ending ]
		while(<CONF>) {
			s/\r|\n//g;
			s/^#.*//g;	# strip whole-line comments
			s/\s#.*//g;	# strip end-line comments
			if (/^(\s*)(.*)\]\s*$/) {
				# End of list, perhaps with some value
				if ($2) {
					$rawvalue .= $1.$2;
					push(@{$dir->{'values'}},
					     &split_xen_array($2));
					}
				last;
				}
			elsif (/\S/) {
				# Some value(s)
				$rawvalue .= $_;
				push(@{$dir->{'values'}},
				     &split_xen_array($_));
				}
			$lnum++;
			}
		$dir->{'eline'} = $lnum;
		}
	elsif (/\S/) {
		# Unknown line?
		print STDERR "Unknown Xen config line $_\n";
		}
	$lnum++;
	}
close(CONF);
return \@rv;
}

# xen_find_value(name, &conf)
# Returns the value or values with some name
sub xen_find_value
{
my ($name, $conf) = @_;
my @rv = map { $_->{'values'} || $_->{'value'} } &xen_find(@_);
return wantarray ? @rv : $rv[0];
}

# xen_find(name, &conf)
# Returns the config object or objects with some name
sub xen_find
{
my ($name, $conf) = @_;
my @rv = grep { $_->{'name'} eq $name } @$conf;
return wantarray ? @rv : $rv[0];
}

# split_xen_array(string)
# Converts a Python array string like 'foo', 6, 'bar' into an Perl array
sub split_xen_array
{
my ($str) = @_;
$str =~ s/^\s*//g;
$str =~ s/\s*$//g;
my @rv;
while(1) {
	if ($str =~ /^\s*'([^']*)',?(.*)/ ||
	    $str =~ /^\s*"([^"]*)",?(.*)/ ||
	    $str =~ /^\s*(\d+),?(.*)/) {
		push(@rv, $1);
		$str = $2;
		}
	else {
		if ($str =~ /\S/) {
			print STDERR "Unknown array element $str\n";
			}
		last;
		}
	}
return @rv;
}

# xen_save_directive(&conf, name, [value])
# Updates a value in a Xen config file. No need to call flush_file_lines
sub xen_save_directive
{
my ($conf, $name, $value) = @_;
my $old = &xen_find($name, $conf);
my $file = $old ? $old->{'file'} : $conf->[0]->{'file'};
my $oldlen = $old ? $old->{'eline'}-$old->{'line'}+1 : undef;
my $lref = &read_file_lines($file);
my $newline;
if (defined($value)) {
	# Work out new config file line
	if (ref($value)) {
		# A list of strings
		$newline = "$name = [".join(",", map { "'$_'" } @$value)."]";
		}
	elsif ($value =~ /^\d+$/ || $value =~ /^\d*\.\d+$/) {
		# A number, no quoting needed
		$newline = "$name = $value";
		}
	elsif ($value =~ /"/) {
		# Need to quote "
		$newline = "$name = '$value'";
		}
	else {
		# Just some string
		$newline = "$name = \"$value\"";
		}
	}
if ($old && defined($value)) {
	# Update a line
	splice(@$lref, $old->{'line'}, $oldlen, $newline);
	$old->{'value'} = $value;
	$old->{'type'} = ref($value) ? 1 : 0;
	$old->{'eline'} = $old->{'line'};
	if ($oldlen != 1) {
		# Need to re-number later lines down
		foreach my $c (@$conf) {
			$c->{'line'} -= $oldlen-1
				if ($c->{'line'} > $old->{'eline'});
			}
		}
	}
elsif ($old && !defined($value)) {
	# Remove a line
	splice(@$lref, $old->{'line'}, $oldlen);
	my $idx = &indexof($old, @$conf);
	splice(@$conf, $idx, 1) if ($idx >= 0);
	foreach my $c (@$conf) {
		$c->{'line'} -= $oldlen if ($c->{'line'} > $old->{'eline'});
		}
	}
elsif (!$old && defined($value)) {
	# Append a line
	push(@$conf, { 'name' => $name,
		       'value' => $value,
		       'type' => ref($value) ? 1 : 0,
		       'line' => scalar(@$lref),
		       'file' => $file });
	push(@$lref, $newline);
	}
&flush_file_lines($file);
}

# get_xen_config(&server, [read-only])
# Returns the Xen config structure for some server, or an error message
sub get_xen_config
{
my ($server, $ro) = @_;
my $vhost = &type_xen_parent($server);
return "Failed to get host system" if (!$vhost);
my $cfile = &transname();
my $err = &get_file_from_server($vhost, $server->{'xen_conf'}, $cfile);
return $err if ($err);
my $conf = &parse_xen_config($cfile);
&unlink_file($cfile) if ($ro);
return $conf || "Failed to parse config file";
}

# put_xen_config(&server, &conf)
# Uploads a Xen configuration (which must be returned by get_xen_config) to the 
# specified server's host. Returns undef if OK, or an error message if not.
sub put_xen_config
{
my ($server, $conf) = @_;
my $vhost = &type_xen_parent($server);
return "Failed to get host system" if (!$vhost);
my $cfile = $conf->[0]->{'file'};
my $err = &put_file_to_server($vhost, $cfile, $server->{'xen_conf'});
return $err;
}

# split_xen_vif_ips(vif-info, [field])
# Given a string like "ip=193.9.101.222,bridge=xenbr0" , return the IP
# addresses as a list
sub split_xen_vif_ips
{
my ($vifinfo, $field) = @_;
$field ||= "ip";
my %info = map { split(/=/, $_) } split(/,/, $vifinfo);
return split(/\s+/, $info{$field});
}

# join_xen_vif_ips(vif-info, &values, [field])
# Given a string like "ip=193.9.101.222,bridge=xenbr0" and a new list of IPs,
# update the IPs in the string and return it
sub join_xen_vif_ips
{
my ($vifinfo, $ips, $field) = @_;
$field ||= "ip";
my @info = split(/,/, $vifinfo);
my $found = 0;
foreach my $i (@info) {
	my ($n, $v) = split(/=/, $i);
	if ($n eq $field) {
		$i = $n."=".join(" ", @$ips);
		$found = 1;
		last;
		}
	}
if (!$found) {
	push(@info, $n."=".join(" ", @$ips));
	}
return join(",", @info);
}

# xen_instance_chooser_button(instance-field, host-field)
# Returns HTML for a popup window for selecting a running xen instance
sub xen_instance_chooser_button
{
my ($ifield, $hfield) = @_;
return "<input type=button onClick='ifield = form.$ifield; chooser = window.open(\"xen_chooser.cgi?host=\"+escape(form.$hfield.value), \"chooser\", \"toolbar=no,menubar=no,scrollbar=no,width=600,height=300\"); chooser.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
}

sub list_xen_on_actions
{
return ("reboot", "shutdown", "crash", "poweroff");
}

sub list_xen_on_options
{
return ('destroy', 'restart', 'preserve', 'rename-restart');
}

# get_xen_software_version(&server)
# Returns the Xen version installed on some host system, like 3.2. Or returns
# undef if it could not be worked out.
sub get_xen_software_version
{
my ($server) = @_;
my ($out, $ex, $err) = &execute_command_on_server($server, "xm info");
if (!$ex && !$err) {
	my $vmajor = $out =~ /xen_major:\s*:\s*([0-9\.]+)/i ? $1 : undef;
	my $vminor = $out =~ /xen_minor:\s*:\s*([0-9\.]+)/i ? $1 : undef;
	if (defined($vmajor) && defined($vminor)) {
		return "$vmajor.$vminor";
		}
	}
return undef;
}

# get_xen_instance_id(&server)
# Returns the Xen ID number for a system
sub get_xen_instance_id
{
my ($server) = @_; 
my $vhost = &type_xen_parent($server);
my $instances = &list_xen_instances($vhost);
return undef if (!ref($instances));
my ($i) = grep { $_->{'instance'} eq $server->{'xen'} } @$instances;
return undef if (!$i);
return $i->{'id'};
}

# get_active_vnc_port(&server)
# Returns the currently assigned dynamic VNC port
sub get_active_vnc_port
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
my $id = &get_xen_instance_id($server);
return undef if (!$id);
my ($out, $ex, $err) = &execute_command_on_server($vhost,
	"xenstore-read /local/domain/$id/console/vnc-port");
if (!$ex && !$err && $out =~ /(\d+)/) {
	return $1;
	}
return undef;
}

# setup_xen_vnc_console(&server)
# Add directives to a Xen instance's .conf file for VNC console access. Returns
# undef on success, or an error message on failure.
sub setup_xen_vnc_console
{
my ($server) = @_;
my $conf = &get_xen_config($server, 0);
return $conf if (!ref($conf));
my $vhost = &type_xen_parent($server);
my $ip = &get_server_ip($vhost);
$server->{'xen_vnc_passwd'} = $server->{'ssh_pass'} ||
			      &generate_random_password();
$server->{'xen_vnc_unused'} = 1;
&xen_save_directive($conf, "vnc", 1);
&xen_save_directive($conf, "vnclisten", "0.0.0.0");
&xen_save_directive($conf, "vncunused", 1);
&xen_save_directive($conf, "vncpasswd", $server->{'xen_vnc_passwd'});
return &put_xen_config($server, $conf);
}

sub type_xen_can_backup
{
return 1;
}

# type_xen_backup_system(&server, dest, shutdown, owner-name)
# Backup a single Xen instance to the given destination, and return a backup
# status hash ref.
sub type_xen_backup_system
{
my ($server, $deststr, $shutdown, $owner) = @_;
my $vhost = &get_parent_server($server);
my $dest = &parse_backup_destination($deststr);
my $onefile = $dest->{'path'} =~ /tar\.gz$/ ? 0 : 1;
my $destserver = &get_backup_dest_server($dest, $vhost);
if ($destserver && $destserver->{'id'} eq $vhost->{'id'}) {
	$dest->{'mode'} = 1;
	}

# Pick a temp file on the host system, if needed
my $tarfile;
if ($dest->{'mode'} == 1) {
	# Target is on host already
	$tarfile = $dest->{'path'};
	}
elsif (&can_transfer_directly($dest)) {
	# Transferring directly to dest host
	if (!$destserver) {
		return { 'ok' => 0, 'msg' => $text{'brun_edestserver'} };
		}
	$tarfile = undef;
	}
else {
	# Pick temp file for tar.gz or just .gz
	$tarfile = &tempname_on_server($vhost, $server->{'host'}.
						($onefile ? '.gz' : '.tar.gz'));
	if (!$tarfile) {
		return { 'ok' => 0,
			 'msg' => &text('brun_etemp', $vhost->{'host'}) };
		}
	}

# Get disk list
my $disks = &type_xen_list_disks($server, 0);
if (!ref($disks)) {
	return { 'ok' => 0,
		 'msg' => &text('brun_edisks', $disks) };
	}
my @lvmdisks = grep { $_->{'lv'} } @$disks;
my @filedisks = grep { !$_->{'device'} } @$disks;

# Exclude swap file from disks
@lvmdisks = grep { !$_->{'swap'} } @lvmdisks;
@filedisks = grep { !$_->{'swap'} } @filedisks;

# Create temp dir for LVM images, if needed
my $tempdir;
if ($server->{'xen_filesystem_lv'} && !$onefile) {
	&remote_foreign_require($vhost, "lvm", "lvm-lib.pl");
	$tempdir = &tempname_on_server($vhost);
	if (!$tempdir) {
		return { 'ok' => 0,
			 'msg' => &text('brun_etemp', $vhost->{'host'}) };
		}
	my $err = &make_dir_on_server($vhost, $tempdir, 0700, 1);
	if ($err) {
		return { 'ok' => 0, 'msg' => &text('brun_emktemp',
					$vhost->{'host'}, $err) };
		}
	}

# Check if LVM on source is clustered
my $clusterlvm;
if ($server->{'xen_filesystem_lv'} && $server->{'status'} ne 'down' &&
    @lvmdisks) {
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
				"vgdisplay ".$lvmdisks[0]->{'vg'});
	if ($out =~ /Clustered\s+yes/i) {
		$clusterlvm = 1;
		}
	}

# If on LVM and we are not shutting down, take an LVM snapshot of each disk
my @snaps;
if ($server->{'xen_filesystem_lv'} && $server->{'status'} ne 'down' &&
    !$clusterlvm) {
	&$first_print(&text('brun_snapshot', $server->{'host'}));
	foreach my $d (@lvmdisks) {
		# Re-open Webmin session, which may have closed due to slowness
		# making snapshots
		&remote_finished();
		&remote_foreign_require($vhost, "lvm", "lvm-lib.pl");
		my $snaplv = { 'size' => int(($d->{'size'}/10)/1024),
			       'is_snap' => 1,
			       'snapof' => $d->{'lv'},
			       'name' => $d->{'lv'}.'_cloudmin_snap',
			       'vg' => $d->{'vg'},
			       'perm' => 'rw',
			       'alloc' => 'n', };
		$snaplv->{'device'} = '/dev/'.$snaplv->{'vg'}.'/'.
				      $snaplv->{'name'};
		&remote_foreign_call($vhost, "lvm",
				     "delete_logical_volume", $snaplv);
		$last_remote_error = undef;
		my $err = &remote_foreign_call($vhost, "lvm",
					  "create_logical_volume", $snaplv);
		$err ||= $last_remote_error;
		if ($err) {
			return { 'ok' => 0,
				 'msg' => &text('brun_esnap',
						$d->{'real'}, $err) };
			}
		push(@snaps, $snaplv);
		}
	}

# Work out LVM device files to copy
my @lvmdevs;
if ($server->{'xen_filesystem_lv'}) {
	if (@snaps) {
		@lvmdevs = map { $_->{'device'} } @snaps;
		}
	else {
		@lvmdevs = map { $_->{'real'} } @lvmdisks;
		}
	}
my $ddfail;

if ($server->{'xen_filesystem_lv'} && $onefile) {
	# On LVM with just one disk - can dd and compress directly into the
	# output file.

	# DD into output file
	if ($tarfile) {
		# Temp file on host
		&$first_print(&text('brun_ddone', $server->{'host'}));
		my $ddcmd =
			"(dd if=$lvmdevs[0] bs=32768 | gzip -c >$tarfile) 2>&1";
		$ddcmd = &niced_backup_command($vhost, $ddcmd);
		my ($out, $ex, $err) = &execute_command_on_server(
					$vhost, $ddcmd);
		if ($ex || $err) {
			$ddfail = &text('brun_edd', $lvmdevs[0], $err || $out);
			last;
			}
		}
	else {
		# Direct SSH
		&$first_print(&text('brun_ddone2', $server->{'host'},
				    $destserver->{'host'}));
		my $ddcmd = "dd if=$lvmdevs[0] bs=32768 | gzip -c";
		$ddcmd = &niced_backup_command($vhost, $ddcmd);
		my $err = &pipe_command_to_server($vhost, $destserver, $ddcmd,
                                                  $dest->{'path'});
		if ($err) {
			$ddfail = &text('brun_edd', $lvmdevs[0], $err);
			last;
			}
		}
	}
elsif ($server->{'xen_filesystem_lv'}) {
	# On LVM, copy each disk into temp files and then tar up

	# DD into temp dir
	&$first_print(&text('brun_dd', $server->{'host'}));
	foreach my $dev (@lvmdevs) {
		my $devfile = $dev;
		$devfile =~ s/^(.*)\///;
		$devfile =~ s/_cloudmin_snap$//;
		my $ddcmd = "dd if=$dev of=$tempdir/$devfile bs=32768 2>&1";
		$ddcmd = &niced_backup_command($vhost, $ddcmd);
		my ($out, $ex, $err) = &execute_command_on_server(
					$vhost, $ddcmd);
		if ($ex || $err) {
			$ddfail = &text('brun_edd', $dev, $err || $out);
			last;
			}
		}

	# Tar up temp dir
	if (!$ddfail && $tarfile) {
		# To temp file
		&$first_print(&text('brun_tarring3', $server->{'host'}));
		my $tarcmd = "cd ".quotemeta($tempdir)." && ".
			     "tar cf - . | gzip -c >".quotemeta($tarfile);
		$tarcmd = &niced_backup_command($vhost, $tarcmd);
		my ($out, $ex, $err) = &execute_command_on_server(
					$vhost, $tarcmd);
		if ($ex || $err) {
			$err = $out if ($ex);
			$ddfail = &text('brun_etar',
					"<tt>".&html_escape($out)."</tt>");
			}
		}
	elsif (!$ddfail) {
		# Via direct SSH
		&$first_print(&text('brun_tarring6', $server->{'host'},
					$destserver->{'host'}));
		my $tarcmd = "cd ".quotemeta($tempdir)." && ".
			     "tar cf - . | gzip -c";
		$tarcmd = &niced_backup_command($vhost, $tarcmd);
		my $err = &pipe_command_to_server($vhost, $destserver, $tarcmd,
                                                  $dest->{'path'});
		if ($err) {
			return { 'ok' => 0,
				 'msg' => &text('brun_etar',
					"<tt>".&html_escape($err)."</tt>") };
			}
		}

	# Remove temp dir of DD images
	&delete_file_on_server($vhost, $tempdir);
	}
else {
	# Work out which disks are regular files, and get relative paths
	if (!@filedisks) {
		return { 'ok' => 0,
			 'msg' => &text('brun_efiledisks') };
		}
	my $dir = $filedisks[0]->{'real'};
	$dir =~ s/\/[^\/]+$//;
	my @paths;
	foreach my $d (@filedisks) {
		if ($d->{'real'} =~ /^\Q$dir\E\/(\S+)$/) {
			push(@paths, $1);
			}
		}

	if ($tarfile) {
		# Run tar and gzip on all of them, into temp file
		&$first_print(&text('brun_tarring2', $server->{'host'}, $dir));
		my $tarcmd = "cd ".quotemeta($dir)." && ".
			"tar cf - ".join(" ", map { quotemeta($_) } @paths).
			" | gzip -c >".quotemeta($tarfile);
		$tarcmd = &niced_backup_command($vhost, $tarcmd);
		my ($out, $ex, $err) = &execute_command_on_server(
					$vhost, $tarcmd);
		if ($ex || $err) {
			$err = $out if ($ex);
			return { 'ok' => 0,
				 'msg' => &text('brun_etar',
					"<tt>".&html_escape($out)."</tt>") };
			}
		}
	else {
		# Run tar and gzip piped into SSH on the target system
		&$first_print(&text('brun_tarring5', $server->{'host'}, $dir,
						     $destserver->{'host'}));
		my $tarcmd = "cd ".quotemeta($dir)." && ".
                        "tar cf - ".join(" ", map { quotemeta($_) } @paths).
                        " | gzip -c";
		$tarcmd = &niced_backup_command($vhost, $tarcmd);
		my $err = &pipe_command_to_server($vhost, $destserver, $tarcmd,
						  $dest->{'path'});
		if ($err) {
			return { 'ok' => 0,
				 'msg' => &text('brun_etar',
					"<tt>".&html_escape($err)."</tt>") };
			}
		}
	}

if ($server->{'xen_filesystem_lv'}) {
	# Close Webmin session, as it may have timed out
	&remote_finished();

	# Remove any snapshots we created
	if (@snaps) {
		&$first_print(&text('brun_unsnapshot', $server->{'host'}));
		&remote_foreign_require($vhost, "lvm", "lvm-lib.pl");
		foreach my $snaplv (@snaps) {
			&remote_foreign_call($vhost, "lvm",
					     "delete_logical_volume", $snaplv);
			}
		}

	if ($ddfail) {
		return { 'ok' => 0, 'msg' => $ddfail };
		}
	}

# Get the size
my @st;
if ($tarfile) {
	@st = &stat_file_on_server($vhost, $tarfile);
	if (!@st || !$st[7]) {
		return { 'ok' => 0,
			 'msg' => &text('brun_estat', $tarfile) };
		}
	}
else {
	@st = &stat_file_on_server($destserver, $dest->{'path'});
	if (!@st || !$st[7]) {
		return { 'ok' => 0,
			 'msg' => &text('brun_estat', $dest->{'path'}) };
		}
	}
my $size = $st[7];

# Delete the backup file with the other name (.gz vs .tar.gz)
my $otherdest = { %$dest };
if ($otherdest->{'path'} =~ /\.tar\.gz$/) {
	$otherdest->{'path'} =~ s/\.tar\.gz$/\.gz/;
	}
else {
	$otherdest->{'path'} =~ s/\.gz$/\.tar\.gz/;
	}
&delete_backup_dest($otherdest, $server);

# Transfer to the destination, if needed
if ($dest->{'mode'} != 1 && $tarfile) {
	# Check if size would exceed limits
	my $err = &check_backup_dest_size($server, $dest, $size, $owner);

	if (!$err) {
		# Send via SCP or FTP
		&$first_print(&text('brun_sending', $server->{'host'},
					    &nice_backup_destination($dest)));
		$err = &send_to_backup_dest($vhost, $tarfile, $dest);
		}

	# Delete from host
	&delete_file_on_server($vhost, $tarfile);

	if ($err) {
		return { 'ok' => 0,
			 'msg' => &text('brun_esend', $err) };
		}
	}

# Return status
return { 'ok' => 1,
	 'size' => $size };
}

# type_xen_restore_system(&server, dest, shutdown)
# Restore single Xen instance from the given destination, and return a backup
# status hash ref.
sub type_xen_restore_system
{
my ($server, $deststr) = @_;
my $vhost = &get_parent_server($server);
my $dest = &parse_backup_destination($deststr);
my $onefile = $dest->{'path'} =~ /tar\.gz$/ ? 0 : 1;
my $destserver = &get_backup_dest_server($dest, $vhost);
if ($destserver && $destserver->{'id'} eq $vhost->{'id'}) {
	$dest->{'mode'} = 1;
	}

# Pick a temp file on the host system, if needed
my $tarfile;
if ($dest->{'mode'} == 1) {
	# Source file is on host already
	$tarfile = $dest->{'path'};
	}
elsif (&can_transfer_directly($dest)) {
	# Transferring directly from source host
	$tarfile = undef;
	}
else {
	# Pick temp file for tar
	$tarfile = &tempname_on_server($vhost, $server->{'host'}.
						($onefile ? '.gz' : '.tar.gz'));
	if (!$tarfile) {
		return { 'ok' => 0,
			 'msg' => &text('brun_etemp', $vhost->{'host'}) };
		}
	}

# Get disk list
my $disks = &type_xen_list_disks($server, 0);
if (!ref($disks)) {
        return { 'ok' => 0,
                 'msg' => &text('brun_edisks', $disks) };
        }
my @lvmdisks = grep { $_->{'lv'} } @$disks;
my @filedisks = grep { !$_->{'device'} } @$disks;
if ($server->{'xen_filesystem_lv'}) {
	if (!@lvmdisks) {
		return { 'ok' => 0,
			 'msg' => &text('brun_elvmdisks') };
		}
	}
else {
	if (!@filedisks) {
		return { 'ok' => 0,
			 'msg' => &text('brun_efiledisks') };
		}
	}

# If possible, work out the correct filename (tar.gz vs gz)
if ($dest->{'mode'} == 0 || $dest->{'mode'} == 2) {
	my @dst = &stat_file_on_server($destserver, $dest->{'path'});
	if (!@dst) {
		my $altpath = $dest->{'path'};
		if ($altpath =~ /\.tar\.gz$/) {
			$altpath =~ s/\.tar\.gz$/\.gz/;
			}
		elsif ($altpath =~ /\.gz$/) {
			$altpath =~ s/\.gz$/\.tar\.gz/;
			}
		@dst = &stat_file_on_server($destserver, $altpath);
		if (!@dst) {
			$dest->{'path'} = $altpath;
			$onefile = $dest->{'path'} =~ /tar\.gz$/ ? 0 : 1;
			}
		}
	}

# Copy in the tar.gz or gz backup file, if needed
if ($dest->{'mode'} != 1 && $tarfile) {
        &$first_print(&text('rrun_getting', $server->{'host'},
                                            &nice_backup_destination($dest)));
        my $err = &get_from_backup_dest($vhost, $tarfile, $dest);
        if ($err) {
                return { 'ok' => 0,
                         'msg' => &text('rrun_eget', $err) };
                }
        }

# Get size, either from temp tar file or source
my @st;
if ($tarfile) {
	@st = &stat_file_on_server($vhost, $tarfile);
	if (!@st || !$st[7]) {
		return { 'ok' => 0,
			 'msg' => &text('rrun_estat', $tarfile) };
		}
	}
else {
	@st = &stat_file_on_server($destserver, $dest->{'path'});
	if (!@st || !$st[7]) {
		return { 'ok' => 0,
			 'msg' => &text('rrun_estat', $dest->{'path'}) };
		}
	}
my $size = $st[7];

my $tarerr;
if ($server->{'xen_filesystem_lv'} && $onefile) {
	# Overwrite first logical volume from un-compressed file with DD
	my $dev = $lvmdisks[0]->{'real'};
	if ($tarfile) {
		# From temp file
		&$first_print(&text('rrun_ddone', $server->{'host'}));
		my $ddcmd = "gunzip -c ".quotemeta($tarfile).
			    " | dd of=$dev bs=32768 2>&1";
		$ddcmd = &niced_backup_command($vhost, $ddcmd);
		my ($out, $ex, $err) = &execute_command_on_server(
					$vhost, $ddcmd);
		if ($ex || $err) {
			$tarerr = &text('rrun_edd', $dev, $err || $out);
			}
		}
	else {
		# Via direct SSH
		&$first_print(&text('rrun_ddone2', $server->{'host'},
					$destserver->{'host'}));
		my $ddcmd = "gunzip -c | dd of=$dev bs=32768";
		$ddcmd = &niced_backup_command($vhost, $ddcmd);
		my $err = &pipe_command_from_server($destserver, $vhost,
						    $ddcmd, $dest->{'path'});
		if ($err) {
			$tarerr = &text('rrun_edd', $dev, $err);
			}
		}
	}
elsif ($server->{'xen_filesystem_lv'}) {
	# Overwrite logical volumes with tar piped into DD
	if ($tarfile) {
		# From temp tar file
		&$first_print(&text('rrun_dd', $server->{'host'}));
		foreach my $d (@lvmdisks) {
			my $devfile = $d->{'real'};
			$devfile =~ s/^(.*)\///;
			my $tarcmd = "gunzip -c ".quotemeta($tarfile).
				     " | tar Oxpf - $devfile ./$devfile".
				     " | dd of=$d->{'real'} bs=32768 2>&1";
			$tarcmd = &niced_backup_command($vhost, $tarcmd);
			my ($out, $ex, $err) = &execute_command_on_server(
						$vhost, $tarcmd);
			if ($ex || $err) {
				$tarerr = &text('rrun_edd', $dev, $err || $out);
				last;
				}
			}
		}
	else {
		# From tar via SSH
		&$first_print(&text('rrun_dd2', $server->{'host'},
				    $destserver->{'host'}));
		foreach my $d (@lvmdisks) {
			my $devfile = $d->{'real'};
			$devfile =~ s/^(.*)\///;
			my $tarcmd = "gunzip -c ".
				     " | tar Oxpf - $devfile ./$devfile".
				     " | dd of=$d->{'real'} bs=32768";
			$tarcmd = &niced_backup_command($vhost, $tarcmd);
			my $err = &pipe_command_from_server($destserver, $vhost,
						    $tarcmd, $dest->{'path'});
			if ($err) {
				$tarerr = &text('rrun_edd', $dev, $err);
				last;
				}
			}
		}
	}
else {
	# Extract tar file of disk images in the disk base directory
	my $dir = $filedisks[0]->{'real'};
        $dir =~ s/\/[^\/]+$//;
	if ($tarfile) {
		# Extract temp tar file
		&$first_print(&text('rrun_untarring2',
				    $server->{'host'}, $dir));
		my $tarcmd = "cd ".quotemeta($dir)." && ".
			     "gunzip -c ".quotemeta($tarfile)." | tar xpf -";
		$tarcmd = &niced_backup_command($vhost, $tarcmd);
		my ($out, $ex, $err) = &execute_command_on_server(
					$vhost, $tarcmd);
		if ($ex || $err) {
			$err = $out if ($ex);
			$tarerr = &text('rrun_etar',
					"<tt>".&html_escape($out)."</tt>");
			}
		}
	else {
		# Fetch and extract tar file in one command
		&$first_print(&text('rrun_untarring5',
			    $server->{'host'}, $dir, $destserver->{'host'}));
		my $tarcmd = "cd ".quotemeta($dir)." && ".
			     "gunzip -c | tar xpf -";
		$tarcmd = &niced_backup_command($vhost, $tarcmd);
		my $err = &pipe_command_from_server($destserver, $vhost,
						    $tarcmd, $dest->{'path'});
		if ($err) {
			$tarerr = &text('rrun_etar',
					"<tt>".&html_escape($err)."</tt>");
			}
		}
	}
&remote_finished();

# Remove the temp file if needed
if ($dest->{'mode'} != 1 && $tarfile) {
        &delete_file_on_server($vhost, $tarfile);
        }
if ($tempdir) {
	&delete_file_on_server($vhost, $tempdir);
	}

# Return status
if ($tarerr) {
        return { 'ok' => 0, 'msg' => $tarerr };
        }
else {
        return { 'ok' => 1, 'size' => $st[7] };
        }
}

# type_xen_backup_extension(&server)
# Returns gz for single-disk LVM systems, tar.gz for multi-disk. This is needed
# to compress a single LV directly into the backup file.
sub type_xen_backup_extension
{
my ($server) = @_;
return "tar.gz" if (!$server->{'xen_filesystem_vg'});
my $disks = &type_xen_list_disks($server, 0);
my @nonswap = grep { !$_->{'swap'} } @$disks;
return scalar(@nonswap) == 1 ? "gz" : "tar.gz";
}

# type_xen_open_shell(&server)
# Runs SSH to connect to a remote system. Returns an error message
# (or undef if OK), and a file handle.
sub type_xen_open_shell
{
my ($server) = @_;
my ($out, $ex, $err, $status, $fh, $fpid) =
	&execute_command_on_server($server, "/bin/sh", 0, 1);
return $err ? ($err, undef) : $ex ? ($out, undef) : (undef, $fh);
}

# type_xen_get_dns_name(&server)
# Returns a system's DNS name and zone name
sub type_xen_get_dns_name
{
my $vhost = &get_parent_server($server);
return ( $server->{'xen_dns'}, $vhost->{'xen_zone'} );
}

# type_xen_image_formats()
# Any of these can be converted to a Xen partition
sub type_xen_image_formats
{
return ( "", "disk", "qcow2", "qcow", "vmdk" );
}

# type_xen_can_clone(&server)
# Any Xen instance can be cloned, if the host system is up
sub type_xen_can_clone
{
my ($server) = @_;
return $server->{'status'} ne 'nohost';
}

# type_xen_check_clone(&oldserver, &newserver, new-hostname, &ips)
# Checks if cloning of some system will work, before shutting it down. Returns
# undef on success or an error message on failure.
sub type_xen_check_clone
{
my ($oldserver, $newserver, $newhost, $ips) = @_;
my $vhost = &type_xen_parent($newserver);

# Make sure hostname is sensible
if ($vhost->{'xen_zone'}) {
	return $text{'clone_ehost2'} if ($newhost =~ /\./);
	}

# Check for IPs if needed
if (!$vhost->{'xen_start'}) {
	$ips && scalar(@$ips) || return &text('clone_eips', $vhost->{'host'});
	}
if ($ips) {
	my @oldips = split(/\s+/, $oldserver->{'xen_ip'});
	scalar(@oldips) == scalar(@$ips) ||
		return &text('clone_eips2', scalar(@oldips), scalar(@$ips));
	}

return undef;
}

# type_xen_clone_system(&oldserver, &newserver, new-hostname, &ips)
# Creates a clone of some Xen instance, updating the newserver hash.
# Returns an error message and a flag indicating if the failure was total or
# only partial.
sub type_xen_clone_system
{
my ($oldserver, $newserver, $newhost, $ips) = @_;
my $vhost = &type_xen_parent($newserver);

# Generate the new full hostname
$newserver->{'xen'} = $newhost;
if ($vhost->{'xen_zone'}) {
	$newserver->{'host'} = $newhost.".".$vhost->{'xen_zone'};
	}
else {
	$newserver->{'host'} = $newhost;
	}
my $clash = &get_managed_server_by("host", $newserver->{'host'});
if ($clash) {
	return (&text('clone_eclash', $newserver->{'host'}), 1);
	}

# Allocate new IP addresses
if ($ips) {
	# Use supplied list, keep old netmasks and gateways
	$newserver->{'xen_ip'} = join(" ", @$ips);
	}
else {
	# Allocate new IPs
	&$first_print($text{'clone_allocating'});
	my $err = &allocate_xen_ip_addresses($newserver, $vhost);
	return ($err, 1) if ($err);
	}
my @ips = split(/\s+/, $newserver->{'xen_ip'});
my @cidrs = split(/\s+/, $newserver->{'xen_cidr'});
my @netmasks = map { &net::prefix_to_mask($_) } @cidrs;
my $defgw = &get_server_default_gateway($vhost);
my @gws = split(/\s+/, $newserver->{'xen_gw'});
my @setgws = grep { $_ ne "*" } @gws;
if (!@setgws && $defgw) {
	@gws = ( $defgw );
	}
$newserver->{'ip_addresses'} = join(" ", @ips);
&complete_managed_server($newserver);
if (!$ips) {
	&$second_print(&text('clone_allocated', join(" ", @ips)));
	}

# Copy the Xen .cfg file
my $xen_dir = $newserver->{'xen_imagedir'} || $vhost->{'xen_dir'};
my $cfgfile = "$xen_dir/$newserver->{'xen'}.cfg";
&$first_print(&text('clone_xencfg', "<tt>$cfgfile</tt>"));
$newserver->{'xen_conf'} = $cfgfile;
delete($newserver->{'xen_console'});
my $err = &transfer_file_between_servers($vhost, $vhost,
					 $oldserver->{'xen_conf'},
				 	 $newserver->{'xen_conf'});
if ($err) {
	return (&text('clone_exencfg', $err), 1);
	}
my $conf = &get_xen_config($newserver, 0);
&xen_save_directive($conf, "name", $newserver->{'xen'});
&xen_save_directive($conf, "address", $ips[0]);
&$second_print($text{'create_done'});

# Copy and fix each of the disks in the .cfg file
&$first_print(&text('clone_copydisks'));
my $disks = &type_xen_list_disks($newserver, 0);
if (!ref($disks)) {
	return (&text('clone_ecopydisks', $disks), 1);
	}
&$indent_print();
my @disklist;
my $oldxen = $oldserver->{'xen'};
my $newxen = $newserver->{'xen'};
foreach my $d (@$disks) {
	&$first_print(&text('clone_copydisk', $d->{'real'}));
	if ($d->{'device'} && !$d->{'lv'}) {
		# A real device, which shouldn't be touched
		push(@disklist, $d->{'suffix'}.":".$d->{'real'}.",".
				$d->{'virtdev'}.",".$d->{'mode'});
		&$second_print(&text('smove_devdisk'));
		next;
		}
	my $err;
	if ($d->{'lv'}) {
		# Pick a new LV name
		my $newlv = $d->{'lv'};
		if ($newlv =~ /\Q$oldxen\E/) {
			$newlv =~ s/\Q$oldxen\E/$newxen/;
			}
		else {
			$newlv .= "-".$newxen;
			}
		$d->{'newreal'} = "/dev/".$d->{'vg'}."/".$newlv;

		# Create the new LV
		$last_remote_error = undef;
		my $lv = { 'name' => $newlv,
			   'size' => int($d->{'size'}/1024),
			   'perm' => 'rw',
			   'alloc' => 'n',
			   'vg' => $d->{'vg'} };
		&remote_finished();
		&remote_foreign_require($vhost, "lvm", "lvm-lib.pl");
		$err = &remote_foreign_call($vhost, "lvm",
					  "create_logical_volume", $lv);
		$err ||= $last_remote_error;

		# Copy the data using dd
		if (!$err) {
			$err = &transfer_file_between_servers(
			    $vhost, $vhost,
			    $d->{'real'}, $d->{'newreal'}, 1, 1,
			    int($d->{'size'}/(1024*1024))*1024*1024);
			}
		}
	else {
		# Pick a new filename
		$d->{'newreal'} = $d->{'real'};
		if ($d->{'newreal'} =~ /\Q$oldxen\E/) {
			$d->{'newreal'} =~ s/\Q$oldxen\E/$newxen/;
			}
		else {
			$d->{'newreal'} .= "-".$newxen;
			}
		# Copy the file
		$err = &transfer_file_between_servers($vhost, $vhost,
                                $d->{'real'}, $d->{'newreal'}, 0, 1);
		}

	if ($err) {
		# Failed .. but may be skippable
		&$second_print(&text('clone_ecopydisk', $err));
		if ($newserver->{'xen_filesystem'} eq $d->{'real'} ||
		    $newserver->{'xen_swap'} eq $d->{'real'}) {
			return (&text('clone_ecopydiskfatal', $err), 1);
			}
		}
	else {
		# Update list of Xen disk options for saving to .cfg file
		push(@disklist, $d->{'suffix'}.":".$d->{'newreal'}.",".
				$d->{'virtdev'}.",".$d->{'mode'});
		if ($newserver->{'xen_filesystem'} eq $d->{'real'}) {
			$newserver->{'xen_filesystem'} = $d->{'newreal'};
			}
		if ($newserver->{'xen_swap'} eq $d->{'real'}) {
			$newserver->{'xen_swap'} = $d->{'newreal'};
			}
		&$second_print(&text('clone_copydiskdone', $d->{'newreal'}));
		}
	}
&$outdent_print();
&xen_save_directive($conf, "disk", \@disklist);
my $err = &put_xen_config($newserver, $conf);
&$second_print($text{'create_done'});

# Register in DNS
if ($vhost->{'xen_zone'}) {
	my $dnsname = $newserver->{'xen'}.".".
		      $vhost->{'xen_zone'}.".";
	if (&add_bind8_record($dnsname, $ips[0],
			      $vhost->{'xen_zone'})) {
		$newserver->{'xen_dns'} = $dnsname;
		}
	else {
		delete($newserver->{'xen_dns'});
		}
	}

# Mount the filesystem
&$first_print($text{'create_xenmount'});
my $err = &mount_filesystem_if_needed($newserver);
my $mp = $newserver->{'xen_mount'};
if ($err || !$mp) {
	$err ||= "No mount point found!";
	&$second_print(&text('create_exenmount', $err));
	return ($err, 0);
	}
&$second_print(&text('create_xenmounted', "<tt>$mp</tt>"));

# Delete any virtual network interfaces if possible, and re-save all real
# interfaces to fix the vif line in the .cfg file
my $fixed_macs;
if (&type_xen_can_interfaces($newserver)) {
	my $ifaces = &type_xen_list_interfaces($newserver);
	if (ref($ifaces)) {
		foreach my $i (@$ifaces) {
			if ($i->{'virtual'} ne '' ||
			    $i->{'fullname'} =~ /^eth(\d+)$/ &&
			    $1 >= scalar(@ips)) {
				&type_xen_delete_interface($newserver, $i);
				}
			elsif ($i->{'virtual'} eq '' &&
			       $i->{'fullname'} =~ /^eth(\d+)$/) {
				my $oldi = { %$i };
				$i->{'address'} = $ips[$1];
				$i->{'ether'} = &generate_mac_address(
							$server, $1);
				&type_xen_modify_interface(
					$newserver, $i, $oldi);
				}
			}
		$fixed_macs = 1;
		}
	}
if (!$fixed_macs) {
	# Cloudmin doesn't know how to manage interfaces, so instead fix mac=
	# directives
	my $vifs = &xen_find_value("vif", $conf);
	for(my $i=0; $i<@$vifs; $i++) {
		if ($vifs->[$i] =~ /mac=[0-9a-f:]+/i) {
			my $mac = &generate_mac_address($server, $i);
			$vifs->[$i] =~ s/mac=[0-9a-f:]+/mac=$mac/;
			}
		}
	&xen_save_directive($conf, "vif", $vifs);
	}

# Fix other config files that refer to the hostname or IP
&$first_print($text{'create_xenfix'});
my $fullhost = $vhost->{'xen_zone'} ? 
		$newserver->{'xen'}.".".$vhost->{'xen_zone'} :
		$newserver->{'xen'};
my $err = &fix_hostname_config_files($newserver, $ips[0],
					$newserver->{'host'}, $fullhost, $vhost,
					$mp, $vhost->{'xen_resolv'});
if ($err) {
	&$second_print(&text('create_exenfix', $err));
	}       
else {
	&$second_print($text{'create_done'});
	}

# Setup networking config files to use the IP.
&$first_print($text{'create_xennet'});
my $err = &fix_network_config_files($newserver, $ips[0], $netmasks[0],
				    $gws[0], $vhost, $mp);
if ($err) {
	&$second_print(&text('create_exennet', $err));
	}       
else {
	&$second_print($text{'create_done'});
	}

# Add network configs for extra IPs
if (@ips > 1) {
	&$first_print($text{'create_xennet2'});
	my $err = &fix_additional_network_interface_files($newserver,
		[ @ips[1..$#ips] ], [ @netmasks[1..$#netmasks] ],
		[ @gws[1..$#gws] ], $vhost, $mp);
	if ($err) {
		&$second_print(&text('create_exennet2', $err));
		}       
	else {
		&$second_print($text{'create_done'});
		}
	}

# Un-mount
&$first_print($text{'create_xenumount'});
&umount_filesystem_if_needed($newserver);
&$second_print($text{'create_done'});

# Add DHCP entries
if ($config{'dhcp_add'}) {
	# Get MAC addresses
	my $fakeifaces = &type_xen_list_interfaces($newserver, 1);
	if (ref($fakeifaces)) {
		&add_all_iface_dhcpd($newserver, $fakeifaces);
		}
	}

return (undef, 0);
}

# type_xen_can_failover([&server, [&new-dest]])
# Return 1 if this virtual system can be failed over to a new host
sub type_xen_can_failover
{
my ($server, $newvhost) = @_;
if (!$server) {
	# Failovers are supported for Xen, in theory
	return 1;
	}
if ($server && $newvhost) {
	# Make sure disks exist on target
	return 0 if (!$server->{'data_files'});
	foreach my $f (split(/\s+/, $server->{'data_files'})) {
		my @st = &stat_file_on_server($newvhost, $f);
		return 0 if (@st <= 1);
		}
	return 1;
	}
return 1;
}

# type_xen_get_failover_files(&server, save-as)
# Fetches from the host of some Xen instance the config files needed to
# perform a failover.
sub type_xen_get_failover_files
{
my ($server, $savefile) = @_;
my $vhost = &type_xen_parent($server);
return &get_file_from_server($vhost, $server->{'xen_conf'}, $savefile);
}

# type_xen_put_failover_files(&server, save-from)
# Uploads to the new host of some Xen instance the config files needed to
# start it up after a failover.
sub type_xen_put_failover_files
{
my ($server, $savefile) = @_;
my $vhost = &type_xen_parent($server);
return &put_file_to_server($vhost, $savefile, $server->{'xen_conf'});
}

# type_xen_get_data_files(&server)
# Returns a list of files/directories that this virtual system uses for
# storage. Used when failing over to verify that storage is shared.
sub type_xen_get_data_files
{
my ($server) = @_;
my $disks = &type_xen_list_disks($server, 0);
return ( ) if (!ref($disks));
return map { $_->{'real'} } @$disks;
}

# type_xen_failover_system(&server, &dest)
# Move a Xen system to a new host, in the case where the old host is down.
# Returns undef on success, or an error message on failure.
sub type_xen_failover_system
{
my ($server, $dest) = @_;
my $vhost = &get_parent_server($server);

# Check for name clash on target
my $instances = &list_xen_instances($dest);
ref($instances) || return $instances;
my ($clash) = grep { $_->{'instance'} eq $server->{'xen'} } @$instances;
if ($clash) {
	return &text('smove_exenclash', $server->{'xen'});
	}

# Write out the Xen config file, if missing
&$first_print($text{'fail_xenconf'});
$server->{'xen_on'} = $dest->{'id'};
my $ffiles = $failover_files_dir."/".$server->{'id'};
my $err = &type_xen_put_failover_files($server, $ffiles);
if ($err) {
	$server->{'xen_on'} = $vhost->{'id'};
	return &text('fail_efiles', $err);
	}
else {
	# Fix VNC IP if needed
	my $conf = &get_xen_config($server, 0);
	my $vnclisten = &xen_find_value("vnclisten", $conf);
	if ($vnclisten && $vnclisten ne "0.0.0.0") {
		my $ip = &get_server_ip($newvhost);
		&xen_save_directive($conf, "vnclisten",
			$ip || &to_ipaddress($newvhost->{'host'}));
		}
	&put_xen_config($server, $conf);

	&$second_print(&text('fail_files', $server->{'xen_conf'}));
	}

return undef;
}



1;

